/** @class */
jscape.ApplicationModule = Class.extend(
    /** @lends {jscape.ApplicationModule.prototype} */
{
    managementService: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(listener, managementService) {
        this.listener = listener || jscape.ApplicationModule.LISTENER;
        this.managementService = managementService;
        this.listener.onStarted(this);
    },

    onMenuItemSelected: function(item) {
        return true;
    },

    submitChanges: function() {
        this.listener.onChangesSubmitted(this);
    },

    cancelChanges: function() {
        this.listener.onChangesCancelled(this);
    },

    onDone: function() {
        this.listener.onChangesSubmitted(this);
    },

    onFail: function() {
        this.listener.onSubmitError(this);
    }
});
jscape.ApplicationModule.LISTENER = {
    onStarted: $.noop(),
    onStartError: $.noop(),
    onChangesSubmitted: $.noop(),
    onChangesCancelled: $.noop(),
    onSubmitError: $.noop()
};

/** @class */
jscape.AuthenticationService = jscape.ApplicationModule.extend(
    /** @lends {jscape.AuthenticationService.prototype} */
{
    /** @constructor */
    initialize: function(resources, continuePath) {
        this._super(resources);
        this.continuePath = continuePath;
    },

    onMenuItemSelected: function(item) {
        if (jscape.Menu.ITEM.Logout == item) {
            this.logout();
            return false;
        }
        return true;
    },

    logout: function() {
        this.managementService.logout()
                .done($.proxy(this._redirect, this, this.continuePath))
                .fail($.proxy(this._redirect, this, this.continuePath));
    },

    _redirect: function(path) {
        window.location = path;
    }
});

/** @class */
jscape.Menu = Class.extend(
    /** @lends {jscape.Menu.prototype} */
{
    /** @constructor */
    initialize: function () {
        this.homeMenu = $("#menu-home");

        this.serverMenu = $("#menu-server");
        this.newDomainMenuItem = $("#menu-server-newdomain");
        this.startDomainsMenuItem = $("#menu-server-domains-start");
        this.stopDomainsMenuItem = $("#menu-server-domains-stop");
        this.pauseDomainsMenuItem = $("#menu-server-domains-pause");
        this.resumeDomainsMenuItem = $("#menu-server-domains-resume");
        this.serverKeystoreMenuItem = $("#menu-server-keystore");
        this.serverSettingsMenuItem = $("#menu-server-settings");

        this.helpMenu = $("#menu-help");
        this.logoutMenuItem = $("#menu-logout");

        this._init();
    },

    bind: function(m, namespace) {
        var ns = (namespace ? "." + namespace : "");
        $("body").on("select.menu" + ns, $.proxy(function(e, item) {
            if (!this.onMenuItemSelected(item)) {
                e.stopImmediatePropagation();
            }
        }, m));
    },

    unbind: function(ns) {
        $("body").off("select.menu." + ns);
    },

    _init: function() {
        this._initComponents();
        this._initListeners();
    },

    _initComponents: function () {
        this.homeMenu.linkbutton({plain: true});
        this.serverMenu.menubutton({plain: true, menu: "#menu-server-items"});
        this.helpMenu.menubutton({plain: true, menu: "#menu-help-items"});
        this.logoutMenuItem.linkbutton({plain: true});

        this.newDomainMenuItem.data("id", jscape.Menu.ITEM.NewDomain);
        this.startDomainsMenuItem.data("id", jscape.Menu.ITEM.StartDomains);
        this.stopDomainsMenuItem.data("id", jscape.Menu.ITEM.StopDomains);
        this.pauseDomainsMenuItem.data("id", jscape.Menu.ITEM.PauseDomains);
        this.resumeDomainsMenuItem.data("id", jscape.Menu.ITEM.ResumeDomains);
        this.serverKeystoreMenuItem.data("id", jscape.Menu.ITEM.KeyManager);
        this.serverSettingsMenuItem.data("id", jscape.Menu.ITEM.ServerSettings);
        this.logoutMenuItem.data("id", jscape.Menu.ITEM.Logout);
    },

    _initListeners: function () {
        var clickHandler = $.proxy(this._onClick$Menu, this);
        this.newDomainMenuItem.click(clickHandler);
        this.startDomainsMenuItem.click(clickHandler);
        this.stopDomainsMenuItem.click(clickHandler);
        this.pauseDomainsMenuItem.click(clickHandler);
        this.resumeDomainsMenuItem.click(clickHandler);
        this.serverKeystoreMenuItem.click(clickHandler);
        this.serverSettingsMenuItem.click(clickHandler);
        this.logoutMenuItem.click(clickHandler);
    },

    _onClick$Menu: function (e) {
        var id = $(e.delegateTarget).data("id");
        $("body").trigger("select.menu", id);
    }
});
/** @const */
jscape.Menu.ITEM = {
    NewDomain: "domains-newdomain",
    StartDomains: "domains-start",
    StopDomains: "domains-stop",
    PauseDomains: "domains-pause",
    ResumeDomains: "domains-resume",
    KeyManager: "server-keymanager",
    ServerSettings: "server-settings",
    Logout: "logout"
};