
/** @class */
jscape.ServiceController = Class.extend(
    /** @lends {jscape.ServiceController.prototype} */
{
    descriptor: null,
    serverParameters: null,
    privateKeys: [],
    listener: null,

    /** @constructor */
    initialize: function() {
    },

    start: function(listener, descriptor, serverParameters, privateKeys, managementService) {
        this.listener = listener || jscape.ServiceController.LISTENER;
        this.descriptor = descriptor;
        this.serverParameters = serverParameters;
        this.privateKeys = privateKeys;
        this.managementService = managementService;

        this._setupView();
    },

    update: function(descriptor) {
        this.descriptor = descriptor;
        this._setupView();
    },

    close: function() {
        this.listener = null;
        this.descriptor = null;
    },

    onApply: function() {
        var descriptor = this._createDescriptor();
        this.listener.onCompleted(descriptor, this);
    },

    onCancel: function() {
        // ok
    },

    _createDescriptor: function() {
        throw "Not Implemented";
    },

    _setupView: function() {
        // empty
    }
});
/** @const */
jscape.ServiceController.LISTENER = {
    onCompleted: $.noop(),
    onBroken: $.noop()
};

/** @class */
jscape.ServiceView = Class.extend(
    /** @lends {jscape.ServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    setListener: function(listener) {
        this.listener = listener || jscape.ServiceView.LISTENER;
    },

    valid: function() {
        return true;
    },

    applyChanges: function() {
        this.listener.onApply(this);
    },

    cancelChanges: function() {
        this.listener.onCancel(this);
    },

    _setValues: function(box, values) {
        var data = [];
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            data.push({"text": value, "value": value});
        }
        box.combobox("loadData", data);
    },

    _defaultValue: function(box) {
        var data = box.combobox("getData");
        return data.length != 0 ? data[0].value : null;
    }
});
/** @const */
jscape.ServiceView.LISTENER = {
    onApply: $.noop(),
    onCancel: $.noop()
};

/** @class */
jscape.FtpServiceDescriptor = Class.extend(
    /** @lends {jscape.FtpServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(host, port, type, keyAlias) {
        this.host = host;
        this.port = port;
        this.type = type;
        this.key = keyAlias;
    },

    toJSON: function() {
        return {
            "@class": "com.jscape.inet.mft.management.descriptors.FtpServiceDescriptor",
            host: this.host,
            port: this.port,
            type: this.type,
            keyAlias: this.key
        }
    }
});
jscape.FtpServiceDescriptor.fromJSON = function(data) {
    return new jscape.FtpServiceDescriptor(data.host, data.port, data.type, data.keyAlias);
};

/** @class */
jscape.FtpServiceController = jscape.ServiceController.extend(
    /** @lends {jscape.FtpServiceController.prototype} */
{
    /** @constructor */
    initialize: function(view) {
        this.view = view;
    },

    /** @override */
    _createDescriptor: function() {
        return new jscape.FtpServiceDescriptor(this.view.getHost(),
                                               this.view.getPort(),
                                               this.view.getType(),
                                               this.view.getPrivateKey());
    },

    _setupView: function() {
        this.view.setListener(this);
        this.view.setHosts(this.serverParameters.getInetAddresses());
        this.view.setPrivateKeys(this.privateKeys);
    }
});

/** @class */
jscape.FtpServiceView = jscape.ServiceView.extend(
    /** @lends {jscape.FtpServiceView.prototype} */
{
    /** @const */
    DEFAULT_HOST: "0.0.0.0",
    DEFAULT_TYPE: "explicit SSL",
    DEFAULT_PORTS: {"regular": 21, "explicit SSL": 21, "forced explicit SSL": 21, "implicit SSL": 990},
    DEFAULT_KEY_USAGE: {"regular": false, "explicit SSL": true, "forced explicit SSL": true, "implicit SSL": true},

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("#ftp-service-settings");
        this.host = $("#dsftp-host");
        this.port = $("#dsftp-port");
        this.type = $("#dsftp-type");
        this.privateKey = $("#dsftp-privatekey");

        this._init();
    },

    show: function() {
        this.fieldset.show();
        this._initState();
    },

    hide: function() {
        this.fieldset.hide();
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHost: function(value) {
        value = value || this.DEFAULT_HOST;
        this.host.combobox("setValue", value);
        this.host.combobox("select", value);
    },

    setHosts: function(values) {
        this._setValues(this.host, values);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value || this.DEFAULT_PORT);
    },

    getType: function() {
        return this.type.combobox("getValue");
    },

    setType: function(value) {
        this.type.combobox("setValue", value);
        this.type.combobox("select", value);
    },

    getPrivateKey: function() {
        return this.privateKey.combobox("getValue");
    },

    setPrivateKey: function(value) {
        this.privateKey.combobox("setValue", value);
    },

    setPrivateKeys: function(values) {
        this._setValues(this.privateKey, values);
    },

    /** @override */
    valid: function() {
        return this.host.combobox("isValid")
                && this.port.numberspinner("isValid")
                && this.type.combobox("isValid")
                && this.privateKey.combobox("isValid");
    },

    _init: function() {
        this.host.combobox({required: true, width: 120});
        this.port.numberspinner({required: true, min: 1, max: 65535});
        this.type.combobox({required: true, editable: false, onChange: $.proxy(this._onChange$Type, this)});
        this.privateKey.combobox({required: true, editable: false, disabled: true, width: 150});
        this.fieldset.hide();
    },

    _initState: function() {
        this.setHost(this._defaultValue(this.host));
        this.setPort(this.DEFAULT_PORT);
        this.setType(this.DEFAULT_TYPE);
        this.setPrivateKey(this._defaultValue(this.privateKey));

        this._adjustPort();
        this._adjustKeyUsage();
        this.fieldset.form("validate");
    },

    _onChange$Type: function() {
        this._adjustPort();
        this._adjustKeyUsage();
    },

    _adjustPort: function() {
        var type = this.getType();
        this.setPort(this.DEFAULT_PORTS[type]);
    },

    _adjustKeyUsage: function() {
        var type = this.getType();
        this.privateKey.combobox(this.DEFAULT_KEY_USAGE[type] ? "enable" : "disable");
    }
});

/** @class */
jscape.SftpServiceDescriptor = Class.extend(
    /** @lends {jscape.SftpServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(host, port, authType, keyAlias) {
        this.host = host;
        this.port = port;
        this.auth = authType;
        this.key = keyAlias;
    },

    toJSON: function() {
        return {
            "@class": "com.jscape.inet.mft.management.descriptors.SftpServiceDescriptor",
            host: this.host,
            port: this.port,
            authenticationType: this.auth,
            keyAlias: this.key
        }
    }
});
jscape.SftpServiceDescriptor.fromJSON = function(data) {
    return new jscape.SftpServiceDescriptor(data.host, data.port, data.authenticationType, data.keyAlias);
};

/** @class */
jscape.SftpServiceController = jscape.ServiceController.extend(
    /** @lends {jscape.SftpServiceController.prototype} */
{
    /** @constructor */
    initialize: function(view) {
        this.view = view;
    },

    /** @override */
    _createDescriptor: function() {
        return new jscape.SftpServiceDescriptor(this.view.getHost(),
                                                this.view.getPort(),
                                                this.view.getAuthType(),
                                                this.view.getPrivateKey());
    },

    _setupView: function() {
        this.view.setListener(this);
        this.view.setHosts(this.serverParameters.getInetAddresses());
        this.view.setPrivateKeys(this.privateKeys);
    }
});

/** @class */
jscape.SftpServiceView = jscape.ServiceView.extend(
    /** @lends {jscape.SftpServiceView.prototype} */
{
    /** @const */
    DEFAULT_HOST: "0.0.0.0",
    DEFAULT_PORT: 22,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("#sftp-service-settings");
        this.host = $("#dssftp-host");
        this.port = $("#dssftp-port");
        this.privateKey = $("#dssftp-privatekey");
        this.authType= $("#dssftp-auth");

        this._init();
    },

    show: function() {
        this.fieldset.show();
        this._initState();
    },

    hide: function() {
        this.fieldset.hide();
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHost: function(value) {
        value = value || this.DEFAULT_HOST;
        this.host.combobox("setValue", value);
        this.host.combobox("select", value);
    },

    setHosts: function(values) {
        this._setValues(this.host, values);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value || this.DEFAULT_PORT);
    },

    getPrivateKey: function() {
        return this.privateKey.combobox("getValue");
    },

    setPrivateKey: function(value) {
        this.privateKey.combobox("setValue", value);
        this.privateKey.combobox("select", value);
    },

    setPrivateKeys: function(values) {
        this._setValues(this.privateKey, values);
    },

    getAuthType: function() {
        return this.authType.combobox("getValue");
    },

    setAuthType: function(value) {
        this.authType.combobox("setValue", value);
        this.authType.combobox("select", value);
    },

    /** @override */
    valid: function() {
        return this.host.combobox("isValid")
                && this.port.numberspinner("isValid")
                && this.authType.combobox("isValid")
                && this.privateKey.combobox("isValid");
    },

    _init: function() {
        this.host.combobox({required: true, width: 120});
        this.port.numberspinner({required: true, min: 1, max: 65535});
        this.privateKey.combobox({required: true, editable: false, width: 150});
        this.authType.combobox({required: true, editable: false, width: 150});
        this.fieldset.hide();
    },

    _initState: function() {
        this.setHost(this._defaultValue(this.host));
        this.setPort(this.DEFAULT_PORT);
        this.setPrivateKey(this._defaultValue(this.privateKey));
        this.setAuthType(this._defaultValue(this.authType));
        this.fieldset.form("validate");
    }
});

/** @class */
jscape.AftpServiceDescriptor = Class.extend(
    /** @lends {jscape.AftpServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(host, port, keyAlias, secure) {
        this.host = host;
        this.port = port;
        this.key = keyAlias;
        this.secure = secure;
    },

    toJSON: function() {
        return {
            "@class": "com.jscape.inet.mft.AftpServiceDescriptor",
            address: {host: this.host, port: this.port},
            keyAlias: this.key,
            protectionRequired: this.secure
        }
    }
});
jscape.AftpServiceDescriptor.fromJSON = function(data) {
    return new jscape.AftpServiceDescriptor(data.address.host, data.address.port, data.keyAlias, data.protectionRequired);
};

/** @class */
jscape.AftpServiceController = jscape.ServiceController.extend(
    /** @lends {jscape.AftpServiceController.prototype} */
{
    /** @constructor */
    initialize: function(view) {
        this.view = view;
    },

    /** @override */
    _createDescriptor: function() {
        return new jscape.AftpServiceDescriptor(this.view.getHost(),
                                               this.view.getPort(),
                                               this.view.getPrivateKey(),
                                               this.view.getProtectionRequired());
    },

    _setupView: function() {
        this.view.setListener(this);
        this.view.setHosts(this.serverParameters.getInetAddresses());
        this.view.setPrivateKeys(this.privateKeys);
    }
});

/** @class */
jscape.AftpServiceView = jscape.ServiceView.extend(
    /** @lends {jscape.AftpServiceView.prototype} */
{
    /** @const */
    DEFAULT_HOST: "0.0.0.0",
    DEFAULT_PORT: 3000,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("#aftp-service-settings");
        this.host = $("#dsaftp-host");
        this.port = $("#dsaftp-port");
        this.privateKey = $("#dsaftp-privatekey");
        this.secure = $("#dsaftp-secure");

        this._init();
    },

    show: function() {
        this.fieldset.show();
        this._initState();
    },

    hide: function() {
        this.fieldset.hide();
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHost: function(value) {
        value = value || this.DEFAULT_HOST;
        this.host.combobox("setValue", value);
        this.host.combobox("select", value);
    },

    setHosts: function(values) {
        this._setValues(this.host, values);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value || this.DEFAULT_PORT);
    },

    getPrivateKey: function() {
        return this.privateKey.combobox("getValue");
    },

    setPrivateKey: function(value) {
        this.privateKey.combobox("setValue", value);
        this.privateKey.combobox("select", value);
    },

    setPrivateKeys: function(values) {
        this._setValues(this.privateKey, values);
    },

    getProtectionRequired: function() {
        return this.secure.prop("checked");
    },

    setProtectionRequired: function(value) {
        this.secure.prop("checked", value);
    },

    /** @override */
    valid: function() {
        return this.host.combobox("isValid")
               && this.port.numberspinner("isValid")
               && this.privateKey.combobox("isValid");
    },

    _init: function() {
        this.host.combobox({required: true, width: 120});
        this.port.numberspinner({required: true, min: 1, max: 65535});
        this.privateKey.combobox({required: true, editable: false, width: 150});
        this.fieldset.hide();
    },

    _initState: function() {
        this.setHost(this._defaultValue(this.host));
        this.setPort(this.DEFAULT_PORT);
        this.setPrivateKey(this._defaultValue(this.privateKey));
        this.setProtectionRequired(false);
        this.fieldset.form("validate");
    }
});

/** @class */
jscape.HttpServiceDescriptor = Class.extend(
    /** @lends {jscape.HttpServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(httpAllowed, httpsAllowed) {
        this.httpAllowed = httpAllowed;
        this.httpsAllowed = httpsAllowed;
    },

    toJSON: function() {
        return {
            "@class": "com.jscape.inet.mft.management.descriptors.HttpServiceDescriptor",
            httpAllowed: this.httpAllowed,
            httpsAllowed: this.httpsAllowed
        }
    }
});
jscape.HttpServiceDescriptor.fromJSON = function(data) {
    return new jscape.HttpServiceDescriptor(data.httpAllowed, data.httpsAllowed);
};

/** @class */
jscape.HttpServiceController = jscape.ServiceController.extend(
    /** @lends {jscape.HttpServiceController.prototype} */
{
    /** @constructor */
    initialize: function(view) {
        this.view = view;
    },

    /** @override */
    _createDescriptor: function() {
        return new jscape.HttpServiceDescriptor(this.view.getHttpEnabled(), this.view.getHttpsEnabled());
    },

    _setupView: function() {
        this.view.setListener(this);
    }
});

/** @class */
jscape.HttpServiceView = jscape.ServiceView.extend(
    /** @lends {jscape.HttpServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("#http-service-settings");
        this.httpEnabled = $("#dshttp-httpenabled");
        this.httpsEnabled = $("#dshttp-httpsenabled");

        this._init();
    },

    show: function() {
        this.fieldset.show();
        this._initState();
    },

    hide: function() {
        this.fieldset.hide();
    },

    getHttpEnabled: function() {
        return this.httpEnabled.prop("checked");
    },

    setHttpEnabled: function(value) {
        this.httpEnabled.prop("checked", value);
    },

    getHttpsEnabled: function() {
        return this.httpsEnabled.prop("checked");
    },

    setHttpsEnabled: function(value) {
        this.httpsEnabled.prop("checked", value);
    },

    _init: function() {
        this.fieldset.hide();
    },

    _initState: function() {
        this.setHttpEnabled(true);
        this.setHttpsEnabled(true);
    }
});

/** @class */
jscape.WebDavServiceDescriptor = Class.extend(
    /** @lends {jscape.WebDavServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(httpAllowed, httpsAllowed) {
        this.httpAllowed = httpAllowed;
        this.httpsAllowed = httpsAllowed;
    },

    toJSON: function() {
        return {
            "@class": "com.jscape.inet.mft.management.descriptors.WebDavServiceDescriptor",
            httpAllowed: this.httpAllowed,
            httpsAllowed: this.httpsAllowed
        }
    }
});
jscape.WebDavServiceDescriptor.fromJSON = function(data) {
    return new jscape.WebDavServiceDescriptor(data.httpAllowed, data.httpsAllowed);
};

/** @class */
jscape.WebDavServiceController = jscape.ServiceController.extend(
    /** @lends {jscape.WebDavServiceController.prototype} */
{
    /** @constructor */
    initialize: function(view) {
        this.view = view;
    },

    /** @override */
    _createDescriptor: function() {
        return new jscape.WebDavServiceDescriptor(this.view.getHttpEnabled(), this.view.getHttpsEnabled());
    },

    _setupView: function() {
        this.view.setListener(this);
    }
});

/** @class */
jscape.WebDavServiceView = jscape.ServiceView.extend(
    /** @lends {jscape.WebDavServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("#webdav-service-settings");
        this.httpEnabled = $("#dswebdav-httpenabled");
        this.httpsEnabled = $("#dswebdav-httpsenabled");

        this._init();
    },

    show: function() {
        this.fieldset.show();
        this._initState();
    },

    hide: function() {
        this.fieldset.hide();
    },

    getHttpEnabled: function() {
        return this.httpEnabled.prop("checked");
    },

    setHttpEnabled: function(value) {
        this.httpEnabled.prop("checked", value);
    },

    getHttpsEnabled: function() {
        return this.httpsEnabled.prop("checked");
    },

    setHttpsEnabled: function(value) {
        this.httpsEnabled.prop("checked", value);
    },

    _init: function() {
        this.fieldset.hide();
    },

    _initState: function() {
        this.setHttpEnabled(true);
        this.setHttpsEnabled(true);
    }
});

/** @class */
jscape.AS2ServiceDescriptor = Class.extend(
    /** @lends {jscape.AS2ServiceDescriptor .prototype} */
{
    /** @constructor */
    initialize: function(httpAllowed, httpsAllowed) {
        this.httpAllowed = httpAllowed;
        this.httpsAllowed = httpsAllowed;
    },

    toJSON: function() {
        return {
            "@class": "com.jscape.inet.mft.AS2ServiceDescriptor",
            httpAccessEnabled: this.httpAllowed,
            httpsAccessEnabled: this.httpsAllowed
        }
    }
});
jscape.AS2ServiceDescriptor.fromJSON = function(data) {
    return new jscape.AS2ServiceDescriptor(data.httpAccessEnabled, data.httpsAccessEnabled);
};

/** @class */
jscape.AS2ServiceController = jscape.ServiceController.extend(
    /** @lends {jscape.AS2ServiceController.prototype} */
{
    /** @constructor */
    initialize: function(view) {
        this.view = view;
    },

    /** @override */
    _createDescriptor: function() {
        return new jscape.AS2ServiceDescriptor(this.view.getHttpEnabled(), this.view.getHttpsEnabled());
    },

    _setupView: function() {
        this.view.setListener(this);
    }
});

/** @class */
jscape.AS2ServiceView = jscape.ServiceView.extend(
    /** @lends {jscape.AS2ServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("#as2-service-settings");
        this.httpEnabled = $("#dsas2-httpenabled");
        this.httpsEnabled = $("#dsas2-httpsenabled");

        this._init();
    },

    show: function() {
        this.fieldset.show();
        this._initState();
    },

    hide: function() {
        this.fieldset.hide();
    },

    getHttpEnabled: function() {
        return this.httpEnabled.prop("checked");
    },

    setHttpEnabled: function(value) {
        this.httpEnabled.prop("checked", value);
    },

    getHttpsEnabled: function() {
        return this.httpsEnabled.prop("checked");
    },

    setHttpsEnabled: function(value) {
        this.httpsEnabled.prop("checked", value);
    },

    _init: function() {
        this.fieldset.hide();
    },

    _initState: function() {
        this.setHttpEnabled(true);
        this.setHttpsEnabled(true);
    }
});

/** @class */
jscape.FileRecordDatastoreServiceDescriptor = Class.extend(
    /** @lends {jscape.FileRecordDatastoreServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(dir) {
        this.dir = dir;
    },

    toJSON: function() {
        return {
            "@class": "com.jscape.inet.mft.FileRecordStoreServiceDescriptor",
            directory: this.dir
        }
    }
});
jscape.FileRecordDatastoreServiceDescriptor.fromJSON = function(data) {
    return new jscape.FileRecordDatastoreServiceDescriptor(data.directory);
};

/** @class */
jscape.FileRecordDatastoreServiceController = jscape.ServiceController.extend(
    /** @lends {jscape.FileRecordDatastoreServiceController.prototype} */
{
    /** @constructor */
    initialize: function(view) {
        this.view = view;
    },

    /** @override */
    _createDescriptor: function() {
        return new jscape.FileRecordDatastoreServiceDescriptor(this.view.getDirectory());
    },

    _setupView: function() {
        this.view.setListener(this);
    }
});

/** @class */
jscape.FileRecordDatastoreServiceView = jscape.ServiceView.extend(
    /** @lends {jscape.FileRecordDatastoreServiceView.prototype} */
{
    /** @const */
    DEFAULT_DIR: "%installdir%/users/%domain%",

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("#datastore-service-filerecord");
        this.dir = $("#dssfr-dir");

        this._init();
    },

    show: function() {
        this.fieldset.show();
        this._initState();
    },

    hide: function() {
        this.fieldset.hide();
    },

    getDirectory: function() {
        return this.dir.val();
    },

    setDirectory: function(value) {
        this.dir.val(value);
    },

    /** @override */
    valid: function() {
        return this.dir.validatebox("isValid");
    },

    _init: function() {
        this.dir.validatebox({required: true, missingMessage: this.resources.getString("DATASTORE_ERROR_BAD_DIR")});
        this.fieldset.hide();
    },

    _initState: function() {
        this.setDirectory(this.DEFAULT_DIR);
        this.fieldset.form("validate");
    }
});

/** @class */
jscape.FileDatastoreServiceDescriptor = Class.extend(
    /** @lends {jscape.FileDatastoreServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(dir) {
        this.dir = dir;
    },

    toJSON: function() {
        return {
            "@class": "com.jscape.inet.mft.management.descriptors.FileStoreServiceDescriptor",
            directory: this.dir
        }
    }
});
jscape.FileDatastoreServiceDescriptor.fromJSON = function(data) {
    return new jscape.FileDatastoreServiceDescriptor(data.directory);
};

/** @class */
jscape.FileDatastoreServiceController = jscape.ServiceController.extend(
    /** @lends {jscape.FileDatastoreServiceController.prototype} */
{
    /** @constructor */
    initialize: function(view) {
        this.view = view;
    },

    /** @override */
    _createDescriptor: function() {
        return new jscape.FileDatastoreServiceDescriptor(this.view.getDirectory());
    },

    _setupView: function() {
        this.view.setListener(this);
    }
});

/** @class */
jscape.FileDatastoreServiceView = jscape.ServiceView.extend(
    /** @lends {jscape.FileDatastoreServiceView.prototype} */
{
    /** @const */
    DEFAULT_DIR: "%installdir%/users/%domain%",

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("#datastore-service-file");
        this.dir = $("#dssf-dir");

        this._init();
    },

    show: function() {
        this.fieldset.show();
        this._initState();
    },

    hide: function() {
        this.fieldset.hide();
    },

    getDirectory: function() {
        return this.dir.val();
    },

    setDirectory: function(value) {
        this.dir.val(value);
    },

    /** @override */
    valid: function() {
        return this.dir.validatebox("isValid");
    },

    _init: function() {
        this.dir.validatebox({required: true, missingMessage: this.resources.getString("DATASTORE_ERROR_BAD_DIR")});
        this.fieldset.hide();
    },

    _initState: function() {
        this.setDirectory(this.DEFAULT_DIR);
        this.fieldset.form("validate");
    }
});

/** @class */
jscape.DbDatastoreServiceDescriptor = Class.extend(
    /** @lends {jscape.DbDatastoreServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(url, username, password, poolSize, timeout) {
        this.url = url;
        this.username = username;
        this.password = password;
        this.poolSize = poolSize;
        this.timeout = timeout;
    },

    toJSON: function() {
        return {
            "@class": "com.jscape.inet.mft.management.descriptors.DBStoreServiceDescriptor",
            url: this.url,
            username: this.username,
            password: this.password,
            poolSize: this.poolSize,
            idleConnectionTTL: this.timeout
        }
    }
});
jscape.DbDatastoreServiceDescriptor.fromJSON = function(data) {
    return new jscape.DbDatastoreServiceDescriptor(data.url, data.username, data.password, data.poolSize, data.idleConnectionTTL);
};

/** @class */
jscape.DbDatastoreServiceController = jscape.ServiceController.extend(
    /** @lends {jscape.DbDatastoreServiceController.prototype} */
{
    /** @constructor */
    initialize: function(view) {
        this.view = view;
    },

    onTestService: function () {
        var descriptor = this._createDescriptor();
        this.managementService.testDatastoreService(descriptor).done($.proxy(function () {
            this.view.showTestSuccessMessage();
        }, this));
    },

    /** @override */
    _createDescriptor: function() {
        return new jscape.DbDatastoreServiceDescriptor(this.view.getUrl(),
                                                       this.view.getUsername(),
                                                       this.view.getPassword(),
                                                       this.view.getPoolSize(),
                                                       this.view.getTimeout());
    },

    _setupView: function() {
        this.view.setListener(this);
        if (this.descriptor) {
            this.view.setUrl(this.descriptor.url);
            this.view.setUsername(this.descriptor.username);
            this.view.setPassword(this.descriptor.password);
            this.view.setPoolSize(this.descriptor.poolSize);
            this.view.setTimeout(this.descriptor.timeout);
        }
    }
});

/** @class */
jscape.DbDatastoreServiceView = jscape.ServiceView.extend(
    /** @lends {jscape.DbDatastoreServiceView.prototype} */
{
    /** @const */
    TIMEOUT_FACTOR: 60 * 1000,
    DEFAULT_POOL_SIZE: 5,
    DEFAULT_TIMEOUT: 5,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("#datastore-service-database");
        this.url = $("#dssdb-url");
        this.user = $("#dssdb-user");
        this.password = $("#dssdb-pwd");
        this.poolSize = $("#dssdb-pool");
        this.timeout = $("#dssdb-idle");
        this.testButton = $("#dssdb-testbtn");

        this._init();
    },

    show: function() {
        this.fieldset.show();
        this._initState();
    },

    hide: function() {
        this.fieldset.hide();
    },

    showTestSuccessMessage: function() {
        $.messager.alert(this.resources.getString("DATASTORE_TEST_SUCCESS_TITLE"),
                         this.resources.getString("DATASTORE_TEST_SUCCESS_MESSAGE"),
                         "info");
    },

    getUrl: function() {
        return this.url.val();
    },

    setUrl: function(value) {
        this.url.val(value);
    },

    getUsername: function() {
        return this.user.val();
    },

    setUsername: function(value) {
        this.user.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getPoolSize: function() {
        return this.poolSize.numberspinner("getValue");
    },

    setPoolSize: function(value) {
        this.poolSize.numberspinner("setValue", value || this.DEFAULT_POOL_SIZE);
    },

    getTimeout: function() {
        return this.timeout.numberspinner("getValue") * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", value ? value / this.TIMEOUT_FACTOR : this.DEFAULT_TIMEOUT);
    },

    /** @override */
    valid: function() {
        return this.url.validatebox("isValid")
                && this.user.validatebox("isValid")
                && this.poolSize.numberspinner("isValid")
                && this.timeout.numberspinner("isValid");
    },

    _init: function() {
        var changeHandler = $.proxy(this._onChange, this);

        this.url.validatebox({required: true, missingMessage: this.resources.getString("DATASTORE_ERROR_BAD_URL")});
        this.url.on("input", changeHandler);

        this.user.validatebox({required: true, missingMessage: this.resources.getString("DATASTORE_ERROR_BAD_USERNAME")});
        this.user.on("input", changeHandler);

        this.poolSize.numberspinner({required: true, min: 1, max: 100});
        this.poolSize.numberspinner({onChange: changeHandler});

        this.timeout.numberspinner({required: true, min: 1, max: 100});
        this.timeout.numberspinner({onChange: changeHandler});

        this.testButton.linkbutton({disabled: true});
        this.testButton.click($.proxy(this._onClick$Test, this));

        this.fieldset.hide();
    },

    _initState: function() {
        this.setPoolSize(this.DEFAULT_POOL_SIZE);
        this.setTimeout(this.DEFAULT_TIMEOUT * this.TIMEOUT_FACTOR);
        this.fieldset.form("validate");
    },

    _onClick$Test: function() {
        this.listener.onTestService(this);
    },

    _onChange: function() {
        this._adjustButtons();
    },

    _adjustButtons: function() {
        this.testButton.linkbutton(this.valid() ? "enable" : "disable");
    }
});
jscape.DbDatastoreServiceView.LISTENER = $.extend({
    onTestService: $.noop()
}, jscape.ServiceView.LISTENER);


/** @class */
jscape.FileLogServiceDescriptor = Class.extend(
    /** @lends {jscape.FileLogServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(dir, strategy, filesize) {
        this.dir = dir;
        this.strategy = strategy;
        this.filesize = filesize;
    },

    toJSON: function() {
        return {
            "@class": "com.jscape.inet.mft.management.descriptors.FileLogServiceDescriptor",
            directory: this.dir,
            strategyName: this.strategy,
            fileSize: this.filesize
        }
    }
});
jscape.FileLogServiceDescriptor.fromJSON = function(data) {
    return new jscape.FileLogServiceDescriptor(data.directory, data.strategyName, data.fileSize);
};

/** @class */
jscape.FileLogServiceController = jscape.ServiceController.extend(
    /** @lends {jscape.FileLogServiceController.prototype} */
{
    /** @constructor */
    initialize: function(view) {
        this.view = view;
    },

    /** @override */
    _createDescriptor: function() {
        return new jscape.FileLogServiceDescriptor(this.view.getDirectory(),
                                                   this.view.getStrategy(),
                                                   this.view.getFileSize());
    },

    _setupView: function() {
        this.view.setListener(this);

        if (this.descriptor) {
            this.view.setDirectory(this.descriptor.dir);
            this.view.setStrategy(this.descriptor.strategy);
            this.view.setFileSize(this.descriptor.filesize);
        }
    }
});

/** @class */
jscape.FileLogServiceView = jscape.ServiceView.extend(
    /** @lends {jscape.FileLogServiceView.prototype} */
{
    /** @const */
    DEFAULT_DIR: "%installdir%/logs/%domain%",
    DEFAULT_STRATEGY: "Daily",
    DEFAULT_SIZE: 10,
    SIZE_FACTOR: 1024 * 1024,
    STRATEGIES: {DAILY: "Daily", WEEKLY: "Weekly", MONTHLY: "Monthly", FILESIZE: "FileSize"},

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("#log-service-file");
        this.dir = $("#lsf-dir");
        this.strategies = $("input[name=logfilerotation]:radio");
        this.filesize = $("#lsf-filesize");

        this._init();
    },

    show: function() {
        this.fieldset.show();
        this._initState();
    },

    hide: function() {
        this.fieldset.hide();
    },

    getDirectory: function() {
        return this.dir.val();
    },

    setDirectory: function(value) {
        this.dir.val(value || this.DEFAULT_DIR);
    },

    getStrategy: function() {
        return this.strategies.filter("input:checked").val();
    },

    setStrategy: function(value) {
        value = value || this.DEFAULT_STRATEGY;
        this.strategies.filter("input[value=" + value +"]").prop("checked", true);
    },

    getFileSize: function() {
        return this.filesize.numberspinner("getValue") * this.SIZE_FACTOR;
    },

    setFileSize: function(value) {
        this.filesize.numberspinner("setValue", value ? value / this.SIZE_FACTOR : this.DEFAULT_SIZE);
    },

    /** @override */
    valid: function() {
        if (this.STRATEGIES.FILESIZE == this.getStrategy()
            && !this.filesize.numberspinner("isValid")) {
            return false;
        }
        return this.dir.validatebox("isValid")
    },

    _init: function() {
        this.dir.validatebox({required: true});
        this.filesize.numberspinner({min: 1, max: 1000, disabled: true});
        this.strategies.click($.proxy(this._onClick$Strategy, this));
        this.fieldset.hide();
    },

    _initState: function() {
        this.setDirectory(this.DEFAULT_DIR);
        this.setStrategy(this.DEFAULT_STRATEGY);
        this.setFileSize(this.DEFAULT_SIZE * this.SIZE_FACTOR);
        this.fieldset.form("validate");
    },

    _onClick$Strategy: function() {
        this._adjustFieldState();
    },

    _adjustFieldState: function() {
        var enabled = this.STRATEGIES.FILESIZE == this.getStrategy();
        this.filesize.numberspinner(enabled ? "enable" : "disable");
    }
});


/** @class */
jscape.DbLogServiceDescriptor = Class.extend(
    /** @lends {jscape.DbLogServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(url, username, password, poolSize, timeout) {
        this.url = url;
        this.username = username;
        this.password = password;
        this.poolSize = poolSize;
        this.timeout = timeout;
    },

    toJSON: function() {
        return {
            "@class": "com.jscape.inet.mft.management.descriptors.DBLogServiceDescriptor",
            url: this.url,
            username: this.username,
            password: this.password,
            poolSize: this.poolSize,
            idleConnectionTTL: this.timeout
        }
    }
});
jscape.DbLogServiceDescriptor.fromJSON = function(data) {
    return new jscape.DbLogServiceDescriptor(data.url, data.username, data.password, data.poolSize, data.idleConnectionTTL);
};



/** @class */
jscape.DbLogServiceController = jscape.ServiceController.extend(
    /** @lends {jscape.DbLogServiceController.prototype} */
{
    /** @constructor */
    initialize: function(view) {
        this.view = view;
    },

    onTestService: function(view) {
        var descriptor = this._createDescriptor();
        this.managementService.testLogService(descriptor).done($.proxy(function() {
            this.view.showTestSuccessMessage();
        }, this));
    },

    /** @override */
    _createDescriptor: function() {
        return new jscape.DbLogServiceDescriptor(this.view.getUrl(),
                                                 this.view.getUsername(),
                                                 this.view.getPassword(),
                                                 this.view.getPoolSize(),
                                                 this.view.getTimeout());
    },

    _setupView: function() {
        this.view.setListener(this);

        if (this.descriptor) {
            this.view.setUrl(this.descriptor.url);
            this.view.setUsername(this.descriptor.username);
            this.view.setPassword(this.descriptor.password);
            this.view.setPoolSize(this.descriptor.poolSize);
            this.view.setTimeout(this.descriptor.timeout);
        }
    }
});

/** @class */
jscape.DbLogServiceView = jscape.ServiceView.extend(
    /** @lends {jscape.DbLogServiceView.prototype} */
{
    /** @const */
    TIMEOUT_FACTOR: 60 * 1000,
    DEFAULT_POOL_SIZE: 5,
    DEFAULT_TIMEOUT: 5,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("#log-service-database");
        this.url = $("#lsdb-url");
        this.user = $("#lsdb-user");
        this.password = $("#lsdb-pwd");
        this.poolSize = $("#lsdb-poolsize");
        this.timeout = $("#lsdb-idlettl");
        this.testButton = $("#lsdb-testbtn");

        this._init();
    },

    show: function() {
        this.fieldset.show();
        this._initState();
    },

    hide: function() {
        this.fieldset.hide();
    },

    showTestSuccessMessage: function() {
        $.messager.alert(this.resources.getString("LOG_SERVICE_TEST_SUCCESS_TITLE"),
                         this.resources.getString("LOG_SERVICE_TEST_SUCCESS_MESSAGE"),
                         "info");
    },

    getUrl: function() {
        return this.url.val();
    },

    setUrl: function(value) {
        this.url.val(value);
    },

    getUsername: function() {
        return this.user.val();
    },

    setUsername: function(value) {
        this.user.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getPoolSize: function() {
        return this.poolSize.numberspinner("getValue");
    },

    setPoolSize: function(value) {
        this.poolSize.numberspinner("setValue", value || this.DEFAULT_POOL_SIZE);
    },

    getTimeout: function() {
        return this.timeout.numberspinner("getValue") * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", value ? value / this.TIMEOUT_FACTOR : this.DEFAULT_TIMEOUT);
    },

    /** @override */
    valid: function() {
        return this.url.validatebox("isValid")
                && this.user.validatebox("isValid")
                && this.poolSize.numberspinner("isValid")
                && this.timeout.numberspinner("isValid");
    },

    _init: function() {
        var changeHandler = $.proxy(this._onChange, this);

        this.url.validatebox({required: true, missingMessage: this.resources.getString("LOG_SERVICE_ERROR_BAD_URL")});
        this.url.on("input", changeHandler);

        this.user.validatebox({required: true, missingMessage: this.resources.getString("LOG_SERVICE_ERROR_BAD_USERNAME")});
        this.user.on("input", changeHandler);

        this.poolSize.numberspinner({required: true, min: 1, max: 100});
        this.poolSize.numberspinner({onChange: changeHandler});

        this.timeout.numberspinner({required: true, min: 1, max: 100});
        this.timeout.numberspinner({onChange: changeHandler});

        this.testButton.linkbutton({disabled: true});
        this.testButton.click($.proxy(this._onClick$Test, this));

        this.fieldset.hide();
    },

    _initState: function() {
        this.setPoolSize(this.DEFAULT_POOL_SIZE);
        this.setTimeout(this.DEFAULT_TIMEOUT * this.TIMEOUT_FACTOR);
        this.fieldset.form("validate");
    },

    _onClick$Test: function() {
        this.listener.onTestService(this);
    },

    _onChange: function() {
        this._adjustButtons();
    },

    _adjustButtons: function() {
        this.testButton.linkbutton(this.valid() ? "enable" : "disable");
    }
});
jscape.DbLogServiceView.LISTENER = $.extend({
    onTestService: $.noop()
}, jscape.ServiceView.LISTENER);