jscape.keystore = jscape.keystore || {};
/** @class */
jscape.keystore.ClientKeysController = jscape.ApplicationModule.extend(
    /** @lends {jscape.keystore.ClientKeysController.prototype} */
{
    view: null,
    publicKeys: null,
    certificates: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.keystore.ClientKeysView(resources);

        this._initValidators(resources);
    },

    start: function(listener, managementService, keys) {
        this.publicKeys = keys.getClientKeys();
        this.certificates = keys.getClientCertificates();

        this._setupView();

        this._super(listener, managementService);
    },

    onImport: function() {
        this.view.showImportDialog({
            onSubmit: $.proxy(this._onSubmit$ImportDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onExportCertificate: function(certificate) {
        this.view.showExportCertificateDialog({
            onSubmit: $.proxy(this._onSubmit$ExportCertificateDialog, this, certificate),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, certificate.alias);
    },

    onExportCertificatePublicKey: function(certificate) {
        this.view.showExportPublicKeyDialog({
            onSubmit: $.proxy(this._onSubmit$ExportCertificatePublicKey, this, certificate),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, certificate.alias);
    },

    onExportPublicKey: function(key) {
        this.view.showExportPublicKeyDialog({
            onSubmit: $.proxy(this._onSubmit$ExportPublicKeyDialog, this, key),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, key.alias);
    },

    onGenerate: function() {
        this.view.showGenerateDialog({
            onSubmit: $.proxy(this._onSubmit$GenerateDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onDeletePublicKey: function(key) {
        this.view.showConfirmDeletePublicKeyDialog(key.alias).done($.proxy(function(key) {
            this.managementService.deleteClientPublicKey(key.alias)
                    .done($.proxy(this._onComplete$DeletePublicKey, this, key));
        }, this, key));
    },

    onDeleteCertificate: function(certificate) {
        this.view.showConfirmDeleteCertificateDialog(certificate.alias).done($.proxy(function(certificate) {
            this.managementService.deleteClientCertificate(certificate.alias)
                    .done($.proxy(this._onComplete$DeleteCertificate, this, certificate));
        }, this, certificate));
    },

    _onSubmit$ImportDialog: function(dialog) {
        dialog.hide();
        this.managementService.importClientKey(dialog.getAlias(), dialog.getFile())
                .done($.proxy(this._onComplete$ImportKey, this));
    },

    _onSubmit$ExportCertificateDialog: function(certificate, dialog) {
        dialog.hide();
        this.managementService.exportClientCertificate(certificate.alias,
                                                       dialog.getFormat(),
                                                       dialog.getFilename());
    },

    _onSubmit$ExportCertificatePublicKey: function(certificate, dialog) {
        dialog.hide();
        this.managementService.exportClientCertificatePublicKey(certificate.alias,
                                                                dialog.getFormat(),
                                                                dialog.getFilename());
    },

    _onSubmit$ExportPublicKeyDialog: function(key, dialog) {
        dialog.hide();
        this.managementService.exportClientPublicKey(key.alias,
                                                     dialog.getFormat(),
                                                     dialog.getFilename());
    },

    _onSubmit$GenerateDialog: function(dialog) {
        dialog.hide();

        var d = new jscape.keystore.ExportPrivateKeyDialog(this.resources);
        d.reset();
        d.setFilename(dialog.getAlias() + ".prv");
        d.show({
            onSubmit: $.proxy(this._onSubmit$GenerateDialogPrivateKey,
                              this,
                              dialog.getAlias(),
                              dialog.getAlgorithm(),
                              dialog.getKeyLength(),
                              dialog.getPeriod(),
                              dialog.getCommonName(),
                              dialog.getOrganizationUnit(),
                              dialog.getOrganization(),
                              dialog.getLocality(),
                              dialog.getState(),
                              dialog.getCountry()),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    _onSubmit$GenerateDialogPrivateKey: function(alias, algorithm, keyLength, period, commonName, organizationUnit, organization, locality, state, country, dialog) {
        dialog.hide();
        this.managementService.generateClientKey(alias, algorithm, keyLength, period, commonName, organizationUnit, organization, locality, state, country, dialog.getFilename(), dialog.getFormat(), dialog.getPassword())
                .done($.proxy(function(keyAlias) {
            this.managementService.getClientCertificate(keyAlias).done($.proxy(this._onComplete$GenerateKey, this));
        }, this, alias));
    },

    _onComplete$ImportKey: function(key) {
        if (key.serial) {
            this.certificates.push(key);
            this.view.setCertificates(this.certificates, $.inArray(key, this.certificates));
        } else {
            this.publicKeys.push(key);
            this.view.setPublicKeys(this.publicKeys, $.inArray(key, this.publicKeys));
        }
    },

    _onComplete$GenerateKey: function(certificate) {
        this.certificates.push(certificate);
        this.view.setCertificates(this.certificates, $.inArray(certificate, this.certificates));
    },

    _onComplete$DeletePublicKey: function(key) {
        var index = $.inArray(key, this.publicKeys);
        if (index != -1) {
            this.publicKeys.splice(index, 1);
            this.view.setPublicKeys(this.publicKeys, index);
        }
    },

    _onComplete$DeleteCertificate: function(certificate) {
        var index = $.inArray(certificate, this.certificates);
        if (index != -1) {
            this.certificates.splice(index, 1);
            this.view.setCertificates(this.certificates, index);
        }
    },

    _onCancel$Dialog: function(dialog) {
        dialog.hide();
    },

    _setupView: function() {
        this.view.show(this);
        this.view.setCertificates(this.certificates);
        this.view.setPublicKeys(this.publicKeys);
    },

    _initValidators: function(resources) {
        $.extend($.fn.validatebox.defaults.rules, {
            'clientkey_nonempty': {
                validator: function(value) { return value && value.trim().length != 0; },
                message: resources.getString("KEY_MANAGER_CLIENT_KEYS_ERROR_BAD_ALIAS")
            },
            'clientkey_notexists': {
                validator: $.proxy(this._validatorAliasExists, this),
                message: resources.getString("KEY_MANAGER_CLIENT_KEYS_ERROR_ALIAS_EXISTS")
            }
        });
    },

    _validatorAliasExists: function(value, excludes) {
        var keys = [].concat(this.publicKeys).concat(this.certificates);
        for (var i = 0; i < keys.length; i++) {
            var alias = keys[i].alias;
            if (excludes && $.inArray(alias, excludes) != -1) {
                continue;
            }
            if (alias == value) {
                return false;
            }
        }
        return true;
    }
});

/** @class */
jscape.keystore.ImportClientKeyDialog = jscape.keystore.ImportPublicKeyDialog.extend(
    /** @lends {jscape.keystore.ImportClientKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
    },

    /** @override */
    _initAliasValidator: function(value) {
        this.alias.validatebox({
            required: true,
            validType: ["clientkey_nonempty", "clientkey_notexists" + (value ? "['" + value + "']" : "")],
            missingMessage: this.resources.getString("KEY_MANAGER_CLIENT_KEYS_ERROR_BAD_ALIAS")
        });
    },

    /** @override */
    _initFileValidator: function() {
        this.file.validatebox({
            required: true,
            missingMessage: this.resources.getString("KEY_MANAGER_CLIENT_KEYS_ERROR_BAD_FILE")
        });
    }
});

/** @class */
jscape.keystore.GenerateClientKeyDialog = jscape.keystore.GenerateKeyDialog.extend(
    /** @lends {jscape.keystore.GenerateClientKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmck-dialog-generate"), resources);

        this.alias = $("#kmck-generate-alias");
        this.keyAlgorithm = $("#kmck-generate-algorithm");
        this.keyLength = $("#kmck-generate-length");
        this.period = $("#kmck-generate-period");
        this.commonName = $("#kmck-generate-cn");
        this.organizationUnit = $("#kmck-generate-ou");
        this.organization = $("#kmck-generate-o");
        this.locality = $("#kmck-generate-l");
        this.state = $("#kmck-generate-s");
        this.country = $("#kmck-generate-c");

        this._init();
    },

    /** @override */
    _initValidators: function() {
        this.alias.validatebox({
            required: true,
            validType: ["clientkey_nonempty", "clientkey_notexists"],
            missingMessage: this.resources.getString("KEY_MANAGER_CLIENT_KEYS_ERROR_BAD_ALIAS")
        });
    }
});

/** @class */
jscape.keystore.ClientKeysView = jscape.keystore.KeysTreeView.extend(
    /** @lends {jscape.keystore.ClientKeysView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.tree = $("#kmck-data");
        this.publicKeyInfo = new jscape.keystore.ClientPublicKeyView(resources);
        this.certificateInfo = new jscape.keystore.ClientCertificateView(resources);
        this.importButton = $("#kmck-import");
        this.exportButton = $("#kmck-export");
        this.exportMenu = $("#kmck-export-menu");
        this.generateButton = $("#kmck-generate");
        this.deleteButton = $("#kmck-delete");

        this._init();
    },

    show: function(listener) {
        this._super(listener || jscape.keystore.ClientKeysView.LISTENER);
    },

    showImportDialog: function(listener) {
        var dialog = new jscape.keystore.ImportClientKeyDialog(this.resources);
        dialog.reset();
        dialog.show(listener);
    },

    showExportCertificateDialog: function(listener, alias) {
        var dialog = new jscape.keystore.ExportCertificateDialog(this.resources);
        dialog.reset();
        dialog.setFilename(alias + ".crt");
        dialog.show(listener);
    },

    showExportPublicKeyDialog: function(listener, alias) {
        var dialog = new jscape.keystore.ExportPublicKeyDialog(this.resources);
        dialog.reset();
        dialog.setFilename(alias + ".pub");
        dialog.show(listener);
    },

    showGenerateDialog: function(listener) {
        var dialog = new jscape.keystore.GenerateClientKeyDialog(this.resources);
        dialog.setAlgorithms([
            jscape.keystore.KeyPairAlgorithm.rsa(),
            jscape.keystore.KeyPairAlgorithm.dsa()
        ]);
        dialog.reset();
        dialog.show(listener);
    },

    showConfirmDeletePublicKeyDialog: function(alias) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_CLIENT_KEYS_DELETE_PUBLIC_KEY_TITLE"),
                                            this.resources.getString("KEY_MANAGER_CLIENT_KEYS_DELETE_PUBLIC_KEY_MESSAGE", {0: alias}));
    },

    showConfirmDeleteCertificateDialog: function(alias) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_CLIENT_KEYS_DELETE_CERTIFICATE_TITLE"),
                                            this.resources.getString("KEY_MANAGER_CLIENT_KEYS_DELETE_CERTIFICATE_MESSAGE", {0: alias}));
    },

    getPublicKeys: function() {
        return this._getKeys(this._findPublicKeysRoot());
    },

    setPublicKeys: function(values, selectIndex) {
        this._setKeys(this._findPublicKeysRoot(), values, selectIndex);
    },

    getCertificates: function() {
        return this._getKeys(this._findCertificatesRoot());
    },

    setCertificates: function(values, selectIndex) {
        this._setKeys(this._findCertificatesRoot(), values, selectIndex);
    },

    _init: function() {
        this._initComponents();
        this._initListeners();
    },

    _initComponents: function() {
        this._super();

        this.importButton.linkbutton();
        this.exportButton.linkbutton({disabled: true});
        this.exportMenu.menu();
        this.generateButton.linkbutton();
        this.deleteButton.linkbutton({disabled: true});
    },

    _initListeners: function() {
        this.importButton.click($.proxy(this._onClick$Import, this));
        this.generateButton.click($.proxy(this._onClick$Generate, this));
        this.exportButton.click($.proxy(this._onClick$Export, this));
        this.exportMenu.menu({onClick: $.proxy(this._onMenu$Export, this)});
        this.deleteButton.click($.proxy(this._onClick$Delete, this));
    },

    _initState: function() {
        this.tree.tree("loadData", [
            {id: "certificates", text: this.resources.getString("KEY_MANAGER_CLIENT_KEYS_NODE_CERTIFICATES"), iconCls: "icon-keys"},
            {id: "public-keys", text: this.resources.getString("KEY_MANAGER_CLIENT_KEYS_NODE_PUBLIC_KEYS"), iconCls: "icon-keys"}
        ]);
        this._super();
    },

    /** @override */
    _adjustKeyInfo: function() {
        var key = this._getSelectedPublicKey();
        var certificate = this._getSelectedCertificate();
        if (key != null) {
            this.certificateInfo.hide();
            this.publicKeyInfo.show(key);
        } else {
            if (certificate != null) {
                this.publicKeyInfo.hide();
                this.certificateInfo.show(certificate);
            } else {
                this.publicKeyInfo.hide();
                this.certificateInfo.hide();
            }
        }
    },

    _onClick$Import: function() {
        this.listener.onImport();
        return false;
    },

    _onClick$Generate: function() {
        this.listener.onGenerate();
        return false;
    },

    _onClick$Export: function() {
        var publicKey = this._getSelectedPublicKey();
        if (publicKey != null) {
            this.listener.onExportPublicKey(publicKey);
        } else {
            var certificate = this._getSelectedCertificate();
            if (certificate && !this.exportButton.linkbutton("options").disabled) {
                this.exportMenu.data("certificate", certificate);
                this.exportMenu.menu("show", {alignTo: this.exportButton});
                this.exportButton.blur();
            }
        }
        return false;
    },

    _onClick$Delete: function() {
        var publicKey = this._getSelectedPublicKey();
        if (publicKey != null) {
            this.listener.onDeletePublicKey(publicKey);
        } else {
            var certificate = this._getSelectedCertificate();
            if (certificate != null) {
                this.listener.onDeleteCertificate(certificate);
            }
        }
        return false;
    },

    _onMenu$Export: function(item) {
        var certificate = this.exportMenu.data("certificate");
        if (certificate) {
            if ("menu-crt" == item.name) {
                this.listener.onExportCertificate(certificate);
            } else if ("menu-pubkey" == item.name) {
                this.listener.onExportCertificatePublicKey(certificate);
            } else {
                throw "Unsupported export format.";
            }
        }
        return false;
    },

    _adjustButtonState: function() {
        var enabled = this._getSelectedKey() != null || this._getSelectedCertificate() != null;

        this._setButtonEnabled(this.exportButton, enabled);
        this._setButtonEnabled(this.deleteButton, enabled);
    },

    _getSelectedCertificate: function() {
        var node = this.tree.tree("getSelected");
        var parent = node && this.tree.tree("getParent", node.target);
        return parent && parent.id == "certificates" ? this._node2Key(node) : null;
    },

    _getSelectedPublicKey: function() {
        var node = this.tree.tree("getSelected");
        var parent = node && this.tree.tree("getParent", node.target);
        return parent && parent.id == "public-keys" ? this._node2Key(node) : null;
    },

    _findCertificatesRoot: function() {
        return this.tree.tree("find", "certificates");
    },

    _findPublicKeysRoot: function() {
        return this.tree.tree("find", "public-keys");
    }
});
/** @const */
jscape.keystore.ClientKeysView.LISTENER = {
    onImport: $.noop(),
    onExportCertificate: $.noop(),
    onExportCertificatePublicKey: $.noop(),
    onExportPublicKey: $.noop(),
    onGenerate: $.noop(),
    onDeletePublicKey: $.noop(),
    onDeleteCertificate: $.noop()
};

/** @class */
jscape.keystore.ClientPublicKeyView = jscape.keystore.KeyInfoView.extend(
    /** @lends {jscape.keystore.ClientPublicKeyView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmck-info"), resources);

        this.algorithm = $("#kmck-info-algorithm");

        this._initComponents();
    },

    /** @override */
    _initState: function() {
        this.algorithm.text(this.data.algorithm);
    }
});


/** @class */
jscape.keystore.ClientCertificateView = jscape.keystore.KeyInfoView.extend(
    /** @lends {jscape.keystore.ClientCertificateView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmccrt-info"), resources);

        this.algorithm = $("#kmccrt-info-algorithm");
        this.certificate = $("#kmccrt-info-certificate");
        this.version = $("#kmccrt-info-version");
        this.serial = $("#kmccrt-info-serialnum");
        this.issuer = $("#kmccrt-info-issuer");
        this.subject = $("#kmccrt-info-subject");
        this.validFromDate = $("#kmccrt-info-validfrom");
        this.validToDate = $("#kmccrt-info-validto");
        this.signature = $("#kmccrt-info-signature");

        this._initComponents();
    },

    /** @override */
    _initState: function() {
        this.algorithm.text(this.data.algorithm);
        this.certificate.text(this.data.certificateType);
        this.version.text(this.data.version);
        this.serial.text(this.data.serial);
        this.issuer.text(this.data.issuer);
        this.subject.text(this.data.subject);
        this.validFromDate.text(this._formatDate(new Date(this.data.validNotBefore)));
        this.validToDate.text(this._formatDate(new Date(this.data.validNotAfter)));
        this.signature.text(this.data.signature);
    }
});