jscape.keystore = jscape.keystore || {};
/** @class */
jscape.keystore.HostKeysController = jscape.ApplicationModule.extend(
    /** @lends {jscape.keystore.HostKeysController.prototype} */
{
    view: null,
    publicKeys: null,
    certificates: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.keystore.HostKeysView(resources);

        this._initValidators(resources);
    },

    start: function(listener, managementService, keys) {
        this.publicKeys = keys.getHostKeys();
        this.certificates = keys.getHostCertificates();

        this._setupView();

        this._super(listener, managementService);
    },

    onImport: function() {
        this.view.showImportDialog({
            onSubmit: $.proxy(this._onSubmit$ImportDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onExportCertificate: function(certificate) {
        this.view.showExportCertificateDialog({
            onSubmit: $.proxy(this._onSubmit$ExportCertificateDialog, this, certificate),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, certificate.alias);
    },

    onExportCertificatePublicKey: function(key) {
        this.view.showExportPublicKeyDialog({
            onSubmit: $.proxy(this._onSubmit$ExportCertificatePublicKey, this, key),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, key.alias);
    },

    onExportPublicKey: function(key) {
        this.view.showExportPublicKeyDialog({
            onSubmit: $.proxy(this._onSubmit$ExportPublicKeyDialog, this, key),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, key.alias);
    },

    onDeletePublicKey: function(key) {
        this.view.showConfirmDeletePublicKeyDialog(key.alias).done($.proxy(function(key) {
            this.managementService.deleteHostPublicKey(key.alias)
                    .done($.proxy(this._onComplete$DeletePublicKey, this, key));
        }, this, key));
    },

    onDeleteCertificate: function(certificate) {
        this.view.showConfirmDeleteCertificateDialog(certificate.alias).done($.proxy(function(certificate) {
            this.managementService.deleteHostCertificate(certificate.alias)
                    .done($.proxy(this._onComplete$DeleteCertificate, this, certificate));
        }, this, certificate));
    },

    _onSubmit$ImportDialog: function(dialog) {
        dialog.hide();
        this.managementService.importHostKey(dialog.getAlias(), dialog.getFile())
                .done($.proxy(this._onComplete$Import, this));
    },

    _onSubmit$ExportCertificateDialog: function(certificate, dialog) {
        dialog.hide();
        this.managementService.exportHostCertificate(certificate.alias,
                                                     dialog.getFormat(),
                                                     dialog.getFilename());
    },

    _onSubmit$ExportCertificatePublicKey: function(key, dialog) {
        dialog.hide();
        this.managementService.exportHostCertificatePublicKey(key.alias,
                                                              dialog.getFormat(),
                                                              dialog.getFilename());
    },

    _onSubmit$ExportPublicKeyDialog: function(key, dialog) {
        dialog.hide();
        this.managementService.exportHostPublicKey(key.alias,
                                                   dialog.getFormat(),
                                                   dialog.getFilename());
    },

    _onCancel$Dialog: function(dialog) {
        dialog.hide();
    },

    _onComplete$Import: function(entity) {
        if (entity.serial) {
            this.certificates.push(entity);
            this.view.setCertificates(this.certificates, $.inArray(entity, this.certificates));
        } else {
            this.publicKeys.push(entity);
            this.view.setPublicKeys(this.publicKeys, $.inArray(entity, this.publicKeys));
        }
    },

    _onComplete$DeletePublicKey: function(key) {
        var index = $.inArray(key, this.publicKeys);
        if (index != -1) {
            this.publicKeys.splice(index, 1);
            this.view.setPublicKeys(this.publicKeys, index);
        }
    },

    _onComplete$DeleteCertificate: function(certificate) {
        var index = $.inArray(certificate, this.certificates);
        if (index != -1) {
            this.certificates.splice(index, 1);
            this.view.setCertificates(this.certificates, index);
        }
    },

    _setupView: function() {
        this.view.show(this);
        this.view.setPublicKeys(this.publicKeys);
        this.view.setCertificates(this.certificates);
    },

    _initValidators: function(resources) {
        $.extend($.fn.validatebox.defaults.rules, {
            'hostkey_nonempty': {
                validator: function(value) { return value && value.trim().length != 0; },
                message: resources.getString("KEY_MANAGER_HOST_KEYS_ERROR_BAD_ALIAS")
            },
            'hostkey_notexists': {
                validator: $.proxy(this._validatorAliasExists, this),
                message: resources.getString("KEY_MANAGER_HOST_KEYS_ERROR_ALIAS_EXISTS")
            }
        });
    },

    _validatorAliasExists: function(value, excludes) {
        var keys = [].concat(this.publicKeys).concat(this.certificates);
        for (var i = 0; i < keys.length; i++) {
            var alias = keys[i].alias;
            if (excludes && $.inArray(alias, excludes) != -1) {
                continue;
            }
            if (alias == value) {
                return false;
            }
        }
        return true;
    }
});

/** @class */
jscape.keystore.ImportHostKeyDialog = jscape.keystore.ImportPublicKeyDialog.extend(
    /** @lends {jscape.keystore.ImportHostKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
    },

    /** @override */
    _initAliasValidator: function() {
        this.alias.validatebox({
            required: true,
            validType: ["hostkey_nonempty", "hostkey_notexists"],
            missingMessage: this.resources.getString("KEY_MANAGER_HOST_KEYS_ERROR_BAD_ALIAS")
        });
    },

    /** @override */
    _initFileValidator: function() {
        this.file.validatebox({
            required: true,
            missingMessage: this.resources.getString("KEY_MANAGER_HOST_KEYS_ERROR_BAD_FILE")
        });
    }
});

/** @class */
jscape.keystore.HostKeysView = jscape.keystore.KeysTreeView.extend(
    /** @lends {jscape.keystore.HostKeysView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.tree = $("#kmhk-data");
        this.publicKeyInfo = new jscape.keystore.HostPublicKeyView(resources);
        this.certificateInfo = new jscape.keystore.HostCertificateView(resources);
        this.importButton = $("#kmhk-import");
        this.exportButton = $("#kmhk-export");
        this.exportMenu = $("#kmhk-export-menu");
        this.deleteButton = $("#kmhk-delete");

        this._init();
    },

    /** @override */
    show: function(listener) {
        this._super(listener || jscape.keystore.HostKeysView.LISTENER);
    },

    showImportDialog: function(listener) {
        var dialog = new jscape.keystore.ImportHostKeyDialog(this.resources);
        dialog.reset();
        dialog.show(listener);
    },

    showExportCertificateDialog: function(listener, alias) {
        var dialog = new jscape.keystore.ExportCertificateDialog(this.resources);
        dialog.reset();
        dialog.setFilename(alias + ".crt");
        dialog.show(listener);
    },

    showExportPublicKeyDialog: function(listener, alias) {
        var dialog = new jscape.keystore.ExportPublicKeyDialog(this.resources);
        dialog.reset();
        dialog.setFilename(alias + ".pub");
        dialog.show(listener);
    },

    showConfirmDeletePublicKeyDialog: function(alias) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_HOST_KEYS_DELETE_PUBLIC_KEY_TITLE"),
                                            this.resources.getString("KEY_MANAGER_HOST_KEYS_DELETE_PUBLIC_KEY_MESSAGE", {0: alias}));
    },

    showConfirmDeleteCertificateDialog: function(alias) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_HOST_KEYS_DELETE_CERTIFICATE_TITLE"),
                                            this.resources.getString("KEY_MANAGER_HOST_KEYS_DELETE_CERTIFICATE_MESSAGE", {0: alias}));
    },

    getPublicKeys: function() {
        return this._getKeys(this._findPublicKeysRoot());
    },

    setPublicKeys: function(values, selectIndex) {
        this._setKeys(this._findPublicKeysRoot(), values, selectIndex);
    },

    getCertificates: function() {
        return this._getKeys(this._findCertificatesRoot());
    },

    setCertificates: function(values, selectIndex) {
        this._setKeys(this._findCertificatesRoot(), values, selectIndex);
    },

    _init: function() {
        this._initComponents();
        this._initListeners();
    },

    _initComponents: function() {
        this._super();

        this.importButton.linkbutton();
        this.exportButton.linkbutton({disabled: true});
        this.exportMenu.menu();
        this.deleteButton.linkbutton({disabled: true});
    },

    _initListeners: function() {
        this.importButton.click($.proxy(this._onClick$Import, this));
        this.exportButton.click($.proxy(this._onClick$Export, this));
        this.deleteButton.click($.proxy(this._onClick$Delete, this));
        this.exportMenu.menu({onClick: $.proxy(this._onMenu$Export, this)});
    },

    _initState: function() {
        this.tree.tree("loadData", [
            {id: jscape.keystore.HostKeysView.NODE_IDS.CERTIFICATES, text: this.resources.getString("KEY_MANAGER_HOST_KEYS_NODE_CERTIFICATES"), iconCls: "icon-keys"},
            {id: jscape.keystore.HostKeysView.NODE_IDS.PUBLIC_KEYS, text: this.resources.getString("KEY_MANAGER_HOST_KEYS_NODE_PUBLIC_KEYS"), iconCls: "icon-keys"}
        ]);
        this._super();
    },

    /** @override */
    _adjustKeyInfo: function() {
        var key = this._getSelectedPublicKey();
        var certificate = this._getSelectedCertificate();
        if (key != null) {
            this.certificateInfo.hide();
            this.publicKeyInfo.show(key);
        } else {
            if (certificate != null) {
                this.publicKeyInfo.hide();
                this.certificateInfo.show(certificate);
            } else {
                this.publicKeyInfo.hide();
                this.certificateInfo.hide();
            }
        }
    },

    _onClick$Import: function() {
        this.listener.onImport();
        return false;
    },

    _onClick$Export: function() {
        var publicKey = this._getSelectedPublicKey();
        if (publicKey != null) {
            this.listener.onExportPublicKey(publicKey);
        } else {
            var certificate = this._getSelectedCertificate();
            if (certificate && this._buttonEnabled(this.exportButton)) {
                this.exportMenu.data("certificate", certificate);
                this.exportMenu.menu("show", {alignTo: this.exportButton});
                this.exportButton.blur();
            }
        }
        return false;
    },

    _onClick$Delete: function() {
        var publicKey = this._getSelectedPublicKey();
        if (publicKey != null) {
            this.listener.onDeletePublicKey(publicKey);
        } else {
            var certificate = this._getSelectedCertificate();
            if (certificate != null) {
                this.listener.onDeleteCertificate(certificate);
            }
        }
        return false;
    },

    _onMenu$Export: function(item) {
        var certificate = this.exportMenu.data("certificate");
        if (certificate) {
            if ("menu-crt" == item.name) {
                this.listener.onExportCertificate(certificate);
            } else if ("menu-pubkey" == item.name) {
                this.listener.onExportCertificatePublicKey(certificate);
            } else {
                throw "Unsupported export format.";
            }
        }
        return false;
    },

    _adjustButtonState: function() {
        var enabled = this._getSelectedPublicKey() != null || this._getSelectedCertificate() != null;

        this._setButtonEnabled(this.exportButton, enabled);
        this._setButtonEnabled(this.deleteButton, enabled);
    },

    _getSelectedPublicKey: function() {
        var node = this.tree.tree("getSelected");
        var parent = node && this.tree.tree("getParent", node.target);
        return parent && parent.id == jscape.keystore.HostKeysView.NODE_IDS.PUBLIC_KEYS ? this._node2Key(node) : null;
    },

    _getSelectedCertificate: function() {
        var node = this.tree.tree("getSelected");
        var parent = node && this.tree.tree("getParent", node.target);
        return parent && parent.id == jscape.keystore.HostKeysView.NODE_IDS.CERTIFICATES ? this._node2Key(node) : null;
    },

    _findCertificatesRoot: function() {
        return this.tree.tree("find", jscape.keystore.HostKeysView.NODE_IDS.CERTIFICATES);
    },

    _findPublicKeysRoot: function() {
        return this.tree.tree("find", jscape.keystore.HostKeysView.NODE_IDS.PUBLIC_KEYS);
    }
});
/** @const */
jscape.keystore.HostKeysView.NODE_IDS = {CERTIFICATES: "certificates", PUBLIC_KEYS: "public-keys"};

/** @const */
jscape.keystore.HostKeysView.LISTENER = {
    onImport: $.noop(),
    onExportPublicKey: $.noop(),
    onExportCertificate: $.noop(),
    onExportCertificatePublicKey: $.noop(),
    onDeletePublicKey: $.noop(),
    onDeleteCertificate: $.noop()
};

/** @class */
jscape.keystore.HostPublicKeyView = jscape.keystore.KeyInfoView.extend(
    /** @lends {jscape.keystore.HostPublicKeyView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmhk-info"), resources);

        this.algorithm = $("#kmhk-info-algorithm");

        this._initComponents();
    },

    /** @override */
    _initState: function() {
        this.algorithm.text(this.data.algorithm);
    }
});

/** @class */
jscape.keystore.HostCertificateView = jscape.keystore.KeyInfoView.extend(
    /** @lends {jscape.keystore.HostCertificateView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmhcrt-info"), resources);

        this.algorithm = $("#kmhcrt-info-algorithm");
        this.certificate = $("#kmhcrt-info-certificate");
        this.version = $("#kmhcrt-info-version");
        this.serial = $("#kmhcrt-info-serialnum");
        this.issuer = $("#kmhcrt-info-issuer");
        this.subject = $("#kmhcrt-info-subject");
        this.validFromDate = $("#kmhcrt-info-validfrom");
        this.validToDate = $("#kmhcrt-info-validto");
        this.signature = $("#kmhcrt-info-signature");

        this._initComponents();
    },

    /** @override */
    _initState: function() {
        this.algorithm.text(this.data.algorithm);
        this.certificate.text(this.data.certificateType);
        this.version.text(this.data.version);
        this.serial.text(this.data.serial);
        this.issuer.text(this.data.issuer);
        this.subject.text(this.data.subject);
        this.validFromDate.text(this._formatDate(new Date(this.data.validNotBefore)));
        this.validToDate.text(this._formatDate(new Date(this.data.validNotAfter)));
        this.signature.text(this.data.signature);
    }
});