var jscape = jscape || {};
jscape.keystore = jscape.keystore || {};

/** @class */
jscape.keystore.KeyPairAlgorithm = Class.extend(
    /** @lends {jscape.keystore.KeyPairAlgorithm.prototype} */
{
    name: null,
    lengths: null,

    /** @constructor */
    initialize: function(name, lengths) {
        this.name = name;
        this.lengths = [].concat(lengths);
    }
});
/** @const */
jscape.keystore.KeyPairAlgorithm.rsa = function() {
    return new jscape.keystore.KeyPairAlgorithm("RSA", [1024, 2048]);
};
/** @const */
jscape.keystore.KeyPairAlgorithm.dsa = function() {
    return new jscape.keystore.KeyPairAlgorithm("DSA", [1024]);
};

/** @class */
jscape.keystore.KeysTreeView = Class.extend(
    /** @lends {jscape.keystore.KeysTreeView.prototype} */
{
    tree: null,
    keyInfo: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    show: function(listener) {
        this.listener = listener;
        this._initState();
    },

    _getKeys: function(parentNode) {
        var keys = [];
        var target = parentNode ? parentNode.target : null;
        var nodes = this.tree.tree("getChildren", target);
        for (var i = 0; i < nodes.length; i++) {
            keys.push(this._node2Key(nodes[i]));
        }
        return keys;
    },

    _setKeys: function(parentNode, keys, selectIndex) {
        var nodes = [];
        for (var i = 0; i < keys.length; i++) {
            nodes.push(this._key2Node(keys[i]));
        }

        if (parentNode) {
            var children = this.tree.tree("getChildren", parentNode.target);
            for (var j = 0; j < children.length; j++) {
                this.tree.tree("remove", children[j].target);
            }
            this.tree.tree("append", {parent: parentNode.target, data: nodes});
        } else {
            this.tree.tree("loadData", nodes);
        }

        if (selectIndex !== undefined) {
            var selectNode;
            if (nodes.length != 0) {
                var index = Math.max(0, Math.min(selectIndex, nodes.length - 1));
                selectNode = this.tree.tree("find", nodes[index].id);
            } else {
                selectNode = parentNode;
            }
            selectNode && this.tree.tree("select", selectNode.target);
        }
    },

    _initComponents: function() {
        this.tree.tree({
            lines: true,
            onSelect: $.proxy(this._adjustUI, this),
            onLoadSuccess: $.proxy(this._adjustUI, this)
        });
    },

    _initState: function() {
        this._adjustUI();
    },

    _key2Node: function(key) {
        return {id: key.alias, text: key.alias, iconCls: "icon-key", attributes: {key: key}, children: []};
    },

    _node2Key: function(node) {
        return node && node.attributes ? node.attributes.key : null;
    },

    _adjustUI: function() {
        this._adjustKeyInfo();
        this._adjustButtonState();
    },

    _adjustKeyInfo: function() {
        var key = this._getSelectedKey();
        if (key != null) {
            this.keyInfo.show(key);
        } else {
            this.keyInfo.hide();
        }
    },

    _adjustButtonState: function() {
    },

    _getSelectedKey: function() {
        var node = this.tree.tree("getSelected");
        return node != null ? this._node2Key(node) : null;
    },

    _buttonEnabled: function (button) {
        return !button.linkbutton("options").disabled;
    },

    _setButtonEnabled: function(button, enabled) {
        button.linkbutton(enabled ? "enable" : "disable");
    }
});

/** @class */
jscape.keystore.KeyInfoView = Class.extend(
    /** @lends {jscape.keystore.KeyInfoView.prototype} */
{
    container: null,
    data: null,

    /** @constructor */
    initialize: function(container, resources) {
        this.container = container;
        this.resources = resources;
    },

    show: function(data) {
        this.data = data;
        this.container.show();
        this._initState();
    },

    hide: function() {
        this.container.hide();
    },

    _initComponents: function() {
        this.container.hide();
    },

    _initState: function() {
    },

    _formatDate: function(value) {
        return new jscape.DateFormat("{MM}-{dd}-{yy}").format(value);
    },

    _formatBoolean: function(value) {
        return !!value ? "yes" : "no"
    }
});

/** @class */
jscape.keystore.ImportPublicKeyDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.ImportPublicKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#km-dialog-importpub"), resources);

        this.alias = $("#km-importpub-alias");
        this.file = $("#km-importpub-file");

        this._init();
    },

    reset: function() {
        this.setAlias("");
        this.file.val("");
    },

    getAlias: function() {
        return this.alias.val();
    },

    setAlias: function(value) {
        this.alias.val(value);
        this._initAliasValidator(value);
    },

    getFile: function() {
        return this.file[0].files[0];
    },

    /** @override */
    _valid: function() {
        return this.alias.validatebox("isValid") && this.file.validatebox("isValid");
    },

    _init: function() {
        this._initAliasValidator();
        this._initFileValidator();
    },

    _initAliasValidator: function(value) {
        this.alias.validatebox({required: true});
    },

    _initFileValidator: function() {
        this.file.validatebox({required: true});
    }
});

/** @class */
jscape.keystore.ExportCertificateDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.ExportCertificateDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#km-dialog-exportcrt"), resources);

        this.filename = $("#km-exportcrt-file");
        this.format = $("#km-exportcrt-format");

        this._initComponents();
    },

    reset: function() {
        this.filename.val("");
    },

    getFilename: function() {
        return this.filename.val();
    },

    setFilename: function(value) {
        this.filename.val(value);
    },

    getFormat: function() {
        return this.format.combobox("getValue");
    },

    /** @override */
    _valid: function() {
        return this.filename.validatebox("isValid");
    },

    _initComponents: function() {
        this.filename.validatebox({
            required: true,
            missingMessage: this.resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_BAD_FILENAME")
        });
        this.format.combobox({editable: false});
    }
});

/** @class */
jscape.keystore.ExportPublicKeyDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.ExportPublicKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#km-dialog-exportpub"), resources);

        this.file = $("#km-exportpub-file");
        this.format = $("#km-exportpub-format");

        this._initComponents();
    },

    reset: function() {
        this.file.val("");
    },

    getFilename: function() {
        return this.file.val();
    },

    setFilename: function(value) {
        this.file.val(value);
    },

    getFormat: function() {
        return this.format.combobox("getValue");
    },

    /** @override */
    _valid: function() {
        return this.file.validatebox("isValid");
    },

    _initComponents: function() {
        this.file.validatebox({required: true, missingMessage: this.resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_BAD_FILENAME")});
        this.format.combobox({editable: false});
    }
});

/** @class */
jscape.keystore.ExportPrivateKeyDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.ExportPrivateKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#km-dialog-exportprv"), resources);

        this.file = $("#km-exportprv-file");
        this.password = $("#km-exportprv-pwd");
        this.confirmPassword = $("#km-exportprv-confirmpwd");
        this.format = $("#km-exportprv-format");

        this._init();
    },

    /** @override */
    show: function(listener) {
        this._initState();
        this._super(listener);
    },

    reset: function() {
        this.file.val("");
        this.password.val("");
        this.confirmPassword.val("");
    },

    getFilename: function() {
        return this.file.val();
    },

    setFilename: function(value) {
        this.file.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    getFormat: function() {
        return this.format.combobox("getValue");
    },

    /** @override */
    _valid: function() {
        return this.file.validatebox("isValid") && this.confirmPassword.validatebox("isValid");
    },

    _init: function() {
        this.file.validatebox({
            required: true,
            missingMessage: this.resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_BAD_FILENAME")
        });
        this.password.on("input", $.proxy(this._onChange$Password, this));
        this.confirmPassword.validatebox({
            required: true,
            validType: "equals['#" + this.password.prop("id") + "']",
            missingMessage: this.resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_CONFIRM_PASSWORD"),
            invalidMessage: this.resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_CONFIRM_PASSWORD")
        });
        this.format.combobox({editable: false});
    },

    _initState: function() {
        this._adjustFieldState();
    },

    _onChange$Password: function() {
        this._adjustFieldState();
    },

    _adjustFieldState: function() {
        this.confirmPassword.validatebox(this.getPassword().length != 0 ? "enableValidation" : "disableValidation");
        this.confirmPassword.validatebox("validate");
    }
});

/** @class */
jscape.keystore.GenerateKeyDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.GenerateKeyDialog.prototype} */
{
    alias: null,
    keyAlgorithm: null,
    keyLength: null,
    period: null,
    commonName: null,
    organizationUnit: null,
    organization: null,
    locality: null,
    state: null,
    country: null,

    reset: function() {
        this.alias.val("");
        this.commonName.val("");
        this.organizationUnit.val("");
        this.organization.val("");
        this.locality.val("");
        this.state.val("");
        this.country.val("");
    },

    getAlias: function() {
        return this.alias.val();
    },

    getAlgorithm: function() {
        return this.keyAlgorithm.combobox("getValue");
    },

    setAlgorithms: function(algorithms) {
        var lengths = {};
        var values = [];
        for (var i = 0; i < algorithms.length; i++) {
            var algorithm = algorithms[i];
            values.push(algorithm.name);
            lengths[algorithm.name] = algorithm.lengths;
        }
        this.keyAlgorithm.data("algorithmLengths", lengths);
        this._setValues(this.keyAlgorithm, values, values[0]);
    },

    getKeyLength: function() {
        return this.keyLength.combobox("getValue");
    },

    getPeriod: function() {
        return this.period.numberspinner("getValue");
    },

    getCommonName: function() {
        return this.commonName.val();
    },

    getOrganizationUnit: function() {
        return this.organizationUnit.val();
    },

    getOrganization: function() {
        return this.organization.val();
    },

    getLocality: function() {
        return this.locality.val();
    },

    getState: function() {
        return this.state.val();
    },

    getCountry: function() {
        return this.country.val();
    },

    /** @override */
    _valid: function() {
        return this.alias.validatebox("isValid") && this.period.numberspinner("isValid");
    },

    _init: function() {
        this._initComponents();
        this._initListeners();
        this._initValidators();
    },

    _initComponents: function() {
        this.keyAlgorithm.combobox({editable: false, width: 83});
        this.keyLength.combobox({editable: false, width: 83});
        this.period.numberspinner({min: 1, max: 10000, value: 365, increment: 10});
    },

    _initListeners: function() {
        this.keyAlgorithm.combobox({onChange: $.proxy(this._onChange$KeyAlgorithm, this)});
    },

    _initValidators: function() {
    },

    _onChange$KeyAlgorithm: function(newValue, oldValue) {
        var algorithmLengths = this.keyAlgorithm.data("algorithmLengths")[newValue];
        this._setValues(this.keyLength, algorithmLengths, algorithmLengths[0]);
    },

    _setValues: function(box, values, selected) {
        var data = [];
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            data.push({"text": value, "value": value});
        }
        box.combobox("loadData", data);
        if (selected) {
            box.combobox("select", selected);
        }
    }
});

/** @class */
jscape.keystore.KeyManagerController = Class.extend(
    /** @lends {jscape.keystore.KeyManagerController.prototype} */
{
    view: null,
    modules: null,
    moduleQueue: null,
    managementService: null,
    keys: null,

    /** @constructor */
    initialize: function(resources, modules) {
        this.modules = modules || [];
        this.view = new jscape.keystore.KeyManagerView(resources);
        this.menu = new jscape.Menu();

        this._initValidators();
    },

    start: function(managementService) {
        this.managementService = managementService;

        this._initModuleQueue();

        this.managementService.getKeys().done($.proxy(function(keys) {
            this.keys = keys;
            this._startNextModule();
        }, this));
    },

    onSubmit: function() {
        this._initModuleQueue();
        this._submitNextModule();
    },

    onCancel: function() {
        this._initModuleQueue();
        this._cancelNextModule();
    },

    onStarted: function() {
        if (this.moduleQueue.length == 0) {
            this.view.show(this);
        } else {
            this._startNextModule();
        }
    },

    onStartError: function() {},
    onSubmitError: function() {},

    onChangesSubmitted: function() {
        if (this.moduleQueue.length == 0) {
            window.location = "/";
        } else {
            this._submitNextModule();
        }
    },

    onChangesCancelled: function() {
        if (this.moduleQueue.length == 0) {
            window.location = "/";
        } else {
            this._cancelNextModule();
        }
    },

    _initModuleQueue: function() {
        this.moduleQueue = [].concat(this.modules);
    },

    _startNextModule: function() {
        var m = this.moduleQueue.pop();
        m.start(this, this.managementService, this.keys);
        this.menu.bind(m, "keystore");
    },

    _submitNextModule: function() {
        var m = this.moduleQueue.pop();
        m.submitChanges();
        this.menu.unbind("keystore");
    },

    _cancelNextModule: function() {
        var m = this.moduleQueue.pop();
        m.cancelChanges();
        this.menu.unbind("keystore");
    },

    _initValidators: function() {
        $.extend($.fn.validatebox.defaults.rules, {
            equals: {
                validator: function(value, param) {
                    return param && param.length == 1 && $(param[0]).val() == value;
                }
            }
        });
    }
});

/** @class */
jscape.keystore.KeyManagerView = Class.extend(
    /** @lends {jscape.keystore.KeyManagerView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.panel = $("#key-manager-pane");
        this.tabs = $("#key-manager-tabs");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.keystore.KeyManagerView.LISTENER;
    },

    _init: function() {
        this.panel.panel({border: false});
        this.tabs.tabs({tabPosition: "top", border: true, onSelect: $.proxy(this._onSelect$Tab, this)});
    },

    _onSelect$Tab: function(_, index) {
        var tab = this.tabs.tabs("getTab", index);
        $(tab).children("fieldset").trigger("focus");
    }
});
jscape.keystore.KeyManagerView.LISTENER = {};