jscape.keystore = jscape.keystore || {};
/** @class */
jscape.keystore.KeyExpirationController = jscape.ApplicationModule.extend(
    /** @lends {jscape.keystore.KeyExpirationController.prototype} */
{
    view: null,
    keys: null,
    operation: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.keystore.KeyExpirationView(resources);
        this.operation = new jscape.PrintOperation(resources)
    },

    start: function(listener, managementService, keys) {
        this.keys = keys || [];

        this.view.show(this);
        this._setupView();

        this._super(listener, managementService);
    },

    onActive: function() {
        this._setupView();
    },

    onPrint: function() {
        this.operation.start({
            onCompleted: function() {},
            onBroken: function() {}
        }, this.view.getPrintData());
    },

    _setupView: function() {
        this.view.setKeys(
                this.keys.getServerKeys(),
                this.keys.getClientKeys().concat(this.keys.getClientCertificates()),
                this.keys.getPgpPublicKeys().concat(this.keys.getPgpSecretKeys())
        );
    }
});
/** @class */
jscape.keystore.KeyExpirationView = Class.extend(
    /** @lends {jscape.keystore.KeyExpirationView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset#kmker-fields");
        this.report = $("#kmker-data");
        this.printButton = $("#kmker-print");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.keystore.KeyExpirationView.LISTENER;
        this._initState();
    },

    getPrintData: function() {
        return this.report.propertygrid("getPanel").html();
    },

    setKeys: function(serverKeys, clientKeys, pgpKeys) {
        this.report.propertygrid("loading");
        var data = []
                .concat(this._formatData(serverKeys, this.resources.getString("KEY_MANAGER_REPORT_GROUP_SERVER_KEYS")))
                .concat(this._formatData(clientKeys, this.resources.getString("KEY_MANAGER_REPORT_GROUP_CLIENT_KEYS")))
                .concat(this._formatData(pgpKeys, this.resources.getString("KEY_MANAGER_REPORT_GROUP_PGP_KEYS")));
        this.report.propertygrid("loadData", data);
        this.report.propertygrid("loaded");
    },

    _formatData: function(keys, group) {
        var data = [];
        for (var i = 0; i < keys.length; i++) {
            data.push(this._key2Row(keys[i], group));
        }
        return data;
    },

    _key2Row: function(key, group) {
        return {
            name: key.alias,
            value: this._formatExpirationDate(key),
            group: group,
            editor: null
        };
    },

    _formatExpirationDate: function(key) {
        return key.validNotAfter
                ? new jscape.DateFormat("{MM}-{dd}-{yy}").format(new Date(key.validNotAfter))
                : this.resources.getString("KEY_MANAGER_REPORT_DATE_NON_EXPIRING");
    },

    _init: function() {
        this.report.propertygrid({
            showGroup: true,
            columns: [[
                {field: "name", title: this.resources.getString("KEY_MANAGER_REPORT_COLUMN_KEY"), width: 100, sortable:true},
                {field: "value", title: this.resources.getString("KEY_MANAGER_REPORT_COLUMN_EXPIRES"), width:100, resizable: false}
            ]]
        });
        this.printButton.linkbutton();
        this.printButton.click($.proxy(this._onClick$Print, this));
        this.fieldset.on("focus", $.proxy(this._onActive, this));
    },

    _initState: function() {
    },

    _onActive: function() {
        this.listener.onActive();
    },

    _onClick$Print: function() {
        this.listener.onPrint(this);
        return false;
    }
});
/** @const */
jscape.keystore.KeyExpirationView.LISTENER = {
    onActive: $.noop(),
    onPrint: $.noop()
};

/** @class */
jscape.PrintOperation = Class.extend(
    /** @lends {jscape.PrintOperation.prototype} */
{
    start: function(listener, data) {
        this.listener = listener || jscape.RestOperation.LISTENER;

        this._print(data);
    },

    _print: function(html) {
        var win;
        try {
            win = window.open("", "printable");
            win.document.write("<html><head>"
                               + $("<div>").append($("head>link").clone()).html()
                               + "</head><body>" + html + "</body></html>");
            win.document.close();
            win.print();
            this._close(win);

            this.listener.onCompleted(html);
        } catch(e) {
            this._close(win);
            this.listener.onBroken();
        }
    },

    _close: function(win) {
        if (win) {
            win.close();
        }
    }
});
/** @const */
jscape.PrintOperation.LISTENER = {
    onCompleted: function(data) {},
    onBroken: function() {}
};