jscape.keystore = jscape.keystore || {};
/** @class */
jscape.keystore.PgpKeysController = jscape.ApplicationModule.extend(
    /** @lends {jscape.keystore.PgpKeysController.prototype} */
{
    view: null,
    secretKeys: null,
    publicKeys: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.keystore.PgpKeysView(resources);

        this._initValidators(resources);
    },

    start: function(listener, managementService, keys) {
        this.secretKeys = keys.getPgpSecretKeys();
        this.publicKeys = keys.getPgpPublicKeys();

        this._setupView();

        this._super(listener, managementService);
    },

    onImport: function() {
        this.view.showImportDialog({
            onSubmit: $.proxy(this._onSubmit$ImportDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onGenerate: function() {
        this.view.showGenerateDialog({
            onSubmit: $.proxy(this._onSubmit$GenerateDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onExportPublicKey: function(key) {
        this.view.showExportPublicKeyDialog({
            onSubmit: $.proxy(this._onSubmit$ExportPublicKeyDialog, this, key.alias),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onExportSecretKey: function(key) {
        this.view.showExportSecretKeyDialog({
            onSubmit: $.proxy(this._onSubmit$ExportSecretKeyDialog, this, key.alias),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onExportSecretPublicKey: function(key) {
        this.view.showExportPublicKeyDialog({
            onSubmit: $.proxy(this._onSubmit$ExportSecretPublicKeyDialog, this, key.alias),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onDeletePublicKey: function(key) {
        this.view.showConfirmDeletePublicKeyDialog(key.alias)
                .done($.proxy(function(key) {
            this.managementService.deletePgpPublicKey(key.alias)
                    .done($.proxy(this._onComplete$DeletePublicKey, this, key));
        }, this, key));
    },

    onDeleteSecretKey: function(key) {
        this.view.showConfirmDeleteSecretKeyDialog(key.alias).done($.proxy(function(key) {
            this.managementService.deletePgpSecretKey(key.alias)
                    .done($.proxy(this._onComplete$DeleteSecretKey, this, key));
        }, this, key));
    },

    onSign: function(key) {
        this.view.showSignKeyDialog({
            onSubmit: $.proxy(this._onSubmit$SignKeyDialog, this, key),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, this.secretKeys);
    },

    onViewSecretKeySignatures: function(key) {
        this.managementService.getPgpSecretKeySignatures(key.alias)
                .done($.proxy(this._onComplete$GetSignatures, this, key.alias));
    },

    onViewPublicKeySignatures: function(key) {
        this.managementService.getPgpPublicKeySignatures(key.alias)
                .done($.proxy(this._onComplete$GetSignatures, this, key.alias));
    },

    _onSubmit$ImportDialog: function(dialog) {
        dialog.hide();

        if (dialog.isSecretKey()) {
            this.managementService.importPgpSecretKey(dialog.getFile(), dialog.getFilePassword())
                    .done($.proxy(this._onComplete$ImportSecretKey, this));
        } else if (dialog.isPublicKey()) {
            this.managementService.importPgpPublicKey(dialog.getFile())
                    .done($.proxy(this._onComplete$ImportPublicKey, this));
        }
    },

    _onSubmit$ExportPublicKeyDialog: function(alias, dialog) {
        dialog.hide();
        this.managementService.exportPgpPublicKey(alias, dialog.getFilename());
    },

    _onSubmit$ExportSecretKeyDialog: function(alias, dialog) {
        dialog.hide();
        this.managementService.exportPgpSecretKey(alias,
                                                  dialog.getFilename(),
                                                  dialog.getPassword());
    },

    _onSubmit$ExportSecretPublicKeyDialog: function(alias, dialog) {
        dialog.hide();
        this.managementService.exportPgpSecretPublicKey(alias, dialog.getFilename());
    },

    _onSubmit$GenerateDialog: function(dialog) {
        dialog.hide();
        this.managementService.generatePgpKey(dialog.getName(),
                                              dialog.getEmail(),
                                              dialog.getAlgorithm(),
                                              dialog.getKeyLength())
                .done($.proxy(this._onComplete$GenerateKey, this));
    },

    _onSubmit$SignKeyDialog: function(key, dialog) {
        dialog.hide();
        this.managementService.signPgpPublicKey(key.alias, dialog.getSigningKey())
                .done($.proxy(this._onComplete$SingKey, this));
    },

    _onCancel$Dialog: function(dialog) {
        dialog.hide();
    },

    _onComplete$ImportPublicKey: function(key) {
        this.publicKeys.push(key);
        this.view.setPublicKeys(this.publicKeys, $.inArray(key, this.publicKeys));
    },

    _onComplete$ImportSecretKey: function(key) {
        this.secretKeys.push(key);
        this.view.setSecretKeys(this.secretKeys, $.inArray(key, this.secretKeys));
    },

    _onComplete$GenerateKey: function(key) {
        this.secretKeys.push(key);
        this.view.setSecretKeys(this.secretKeys, $.inArray(key, this.secretKeys));
    },

    _onComplete$DeletePublicKey: function(key) {
        var index = $.inArray(key, this.publicKeys);
        if (index != -1) {
            this.publicKeys.splice(index, 1);
            this.view.setPublicKeys(this.publicKeys, index);
        }
    },

    _onComplete$DeleteSecretKey: function(key) {
        var index = $.inArray(key, this.secretKeys);
        if (index != -1) {
            this.secretKeys.splice(index, 1);
            this.view.setSecretKeys(this.secretKeys, index);
        }
    },

    _onComplete$SingKey: function(key) {
    },

    _onComplete$GetSignatures: function(alias, signatures) {
         this.view.showViewSignaturesDialog(alias, signatures);
    },

    _setupView: function() {
        this.view.show(this);
        this.view.setSecretKeys(this.secretKeys);
        this.view.setPublicKeys(this.publicKeys);
    },

    _initValidators: function (resources) {
        $.extend($.fn.validatebox.defaults.rules, {
            'realname_nonempty': {
                validator: function (value) { return value && value.trim().length != 0; },
                message: resources.getString("KEY_MANAGER_PGP_KEYS_ERROR_BAD_NAME")
            },
            'alias_notexists': {
                validator: $.proxy(this._validatorAliasNotExists, this),
                message: resources.getString("KEY_MANAGER_PGP_KEYS_ERROR_ALIAS_EXISTS")
            }
        });
    },

    _validatorAliasNotExists: function(_, params) {
        var realName = $(params[0]).val();
        var email = $(params[1]).val();
        var alias = email.trim().length != 0 ? "{0} <{1}>".supplant({0: realName, 1: email}) : realName;
        var keys = [].concat(this.publicKeys).concat(this.secretKeys);

        for (var i = 0; i < keys.length; i++) {
            if (alias == keys[i].alias) {
                return false;
            }
        }
        return true;
    }
});

/** @class */
jscape.keystore.ExportPgpPublicKeyDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.ExportPgpPublicKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmpgp-dialog-exportpub"), resources);

        this.file = $("#kmpgp-exportpub-file");

        this._initComponents();
    },

    reset: function() {
        this.file.val("pgp-key.pub");
    },

    getFilename: function() {
        return this.file.val();
    },

    /** @override */
    _valid: function() {
        return this.file.validatebox("isValid");
    },

    _initComponents: function() {
        this.file.validatebox({required: true, missingMessage: this.resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_BAD_FILENAME")});
    }
});

/** @class */
jscape.keystore.ExportPgpSecretKeyDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.ExportPgpSecretKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmpgp-dialog-exportsec"), resources);

        this.file = $("#kmpgp-exportsec-file");
        this.password = $("#kmpgp-exportsec-pwd");

        this._initComponents();
    },

    reset: function() {
        this.file.val("pgp-key.sec");
        this.password.val("");
    },

    getFilename: function() {
        return this.file.val();
    },

    getPassword: function() {
        return this.password.val();
    },

    /** @override */
    _valid: function() {
        return this.file.validatebox("isValid") && this.password.validatebox("isValid");
    },

    _initComponents: function() {
        this.file.validatebox({required: true, missingMessage: this.resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_BAD_FILENAME")});
        this.password.validatebox({required: true, missingMessage: this.resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_BAD_FILE_PASSWORD")});
    }
});

/** @class */
jscape.keystore.ImportPgpKeyDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.ImportPgpKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmpgpk-dialog-import"), (resources));

        this.types = $("input:radio[name=kmpgpk-import-type]");
        this.file = $("#kmpgpk-import-file");
        this.filePassword = $("#kmpgpk-import-pwd");

        this._initComponents();
    },

    /** @override */
    show: function(listener) {
        this._super(listener);
        this._initState();
    },

    reset: function() {
        $(this.types[0]).prop("checked", true);
        this.file.val("");
        this.filePassword.val("");
    },

    getType: function() {
        return this.types.filter("input:checked").val();
    },

    isSecretKey: function() {
        return /secret/i.test(this.getType());
    },

    isPublicKey: function() {
        return /public/i.test(this.getType());
    },

    getFile: function() {
        return this.file[0].files[0];
    },

    getFilePassword: function() {
        return this.filePassword.val();
    },

    /** @override */
    _valid: function() {
        return this.file.validatebox("isValid") && this.filePassword.validatebox("isValid");
    },

    _initComponents: function() {
        this.types.on("change", $.proxy(this._onChange$Type, this));
        this.file.validatebox({
            required: true,
            missingMessage: this.resources.getString("KEY_MANAGER_PGP_KEYS_ERROR_BAD_FILE")
        });
        this.filePassword.validatebox({
            required: true,
            missingMessage: this.resources.getString("KEY_MANAGER_PGP_KEYS_ERROR_BAD_FILE_PASSWORD")
        });
    },

    _initState: function() {
        this._adjustFieldState();
    },

    _onChange$Type: function() {
        this._adjustFieldState();
        this.filePassword.validatebox("validate");
    },

    _adjustFieldState: function() {
        if (this.isSecretKey()) {
            this.filePassword.prop("disabled", false);
            this.filePassword.validatebox("enableValidation");
        } else {
            this.filePassword.prop("disabled", true);
            this.filePassword.validatebox("disableValidation");
        }
    }
});

/** @class */
jscape.keystore.GeneratePgpKeyDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.GeneratePgpKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmpgpk-dialog-generate"), resources);

        this.name = $("#kmpgpk-generate-name");
        this.email = $("#kmpgpk-generate-email");
        this.keyAlgorithm = $("#kmpgpk-generate-algorithm");
        this.keyLength = $("#kmpgpk-generate-length");

        this._init();
    },

    reset: function() {
        this.name.val("");
        this.email.val("");
    },

    getName: function() {
        return this.name.val();
    },

    getEmail: function() {
        return this.email.val();
    },

    getAlgorithm: function() {
        return this.keyAlgorithm.combobox("getValue");
    },

    setAlgorithms: function(algorithms) {
        var lengths = {};
        var values = [];
        for (var i = 0; i < algorithms.length; i++) {
            var algorithm = algorithms[i];
            values.push(algorithm.name);
            lengths[algorithm.name] = algorithm.lengths;
        }
        this.keyAlgorithm.data("algorithmLengths", lengths);
        this._setValues(this.keyAlgorithm, values, values[0]);
    },

    getKeyLength: function() {
        return this.keyLength.combobox("getValue");
    },

    /** @override */
    _valid: function() {
        return this.name.validatebox("isValid") && this.email.validatebox("isValid");
    },

    _init: function() {
        this.name.validatebox({required: true, validType: ["realname_nonempty", "alias_notexists['#" + this.name.prop("id") + "', '#" + this.email.prop("id") + "']"], missingMessage: this.resources.getString("KEY_MANAGER_PGP_KEYS_ERROR_BAD_NAME")});
        this.name.on('input', $.proxy(function() { this.email.validatebox("validate");}, this));

        this.email.validatebox({validType: ["email", "alias_notexists['#" + this.name.prop("id") + "', '#" + this.email.prop("id") + "']"]});
        this.email.on('input', $.proxy(function() { this.name.validatebox("validate");}, this));

        this.keyAlgorithm.combobox({editable: false, width: 80});
        this.keyAlgorithm.combobox({onChange: $.proxy(this._onChange$KeyAlgorithm, this)});
        this.keyLength.combobox({editable: false, width: 80});
    },

    _onChange$KeyAlgorithm: function(newValue, oldValue) {
        var algorithmLengths = this.keyAlgorithm.data("algorithmLengths")[newValue];
        this._setValues(this.keyLength, algorithmLengths, algorithmLengths[0]);
    },

    _setValues: function(box, values, selected) {
        var data = [];
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            data.push({"text": value, "value": value});
        }
        box.combobox("loadData", data);
        if (selected) {
            box.combobox("select", selected);
        }
    }
});

/** @class */
jscape.keystore.SingPgpKeyDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.SingPgpKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmpgpk-dialog-sign"), resources);

        this.signingKey = $("#kmpgpk-sign-key");

        this._initComponents();
    },

    reset: function() {
        this.signingKey.combobox("select", this.signingKey.combobox("getData")[0].value);
    },

    getSigningKey: function() {
        return this.signingKey.combobox("getValue");
    },

    setSigningKeys: function(keys) {
        var data = [];
        for (var i = 0; i < keys.length; i++) {
            var alias = keys[i].alias;
            data.push({text: alias, value: alias});
        }
        this.signingKey.combobox("loadData", data);
    },

    _initComponents: function() {
        this.signingKey.combobox({editable: false, width: 180});
    }
});

/** @class */
jscape.keystore.PgpKeySignaturesDialog = Class.extend(
    /** @lends {jscape.keystore.PgpKeySignaturesDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    show: function(signatures) {
        var dialog = $(this._messageHtml(signatures));
        dialog.dialog({
            title: this.resources.getString("KEY_MANAGER_PGP_KEYS_SIGNATURES_TITLE"),
            width: 500, height: 300, closed: false, modal: true,
            buttons: [{text: "Ok", handler: function() {dialog.dialog("close");}}],
            onClose: function() { setTimeout(function() {dialog.dialog("destroy");}, 100); }
        });
    },

    _messageHtml: function(signatures) {
        var message = "";
        var template = this.resources.getString("KEY_MANAGER_PGP_KEYS_SIGNATURE_INFO_TEMPLATE");

        for (var i = 0; i < signatures.length; i++) {
            var signature = signatures[i];
            message += template.supplant({
                "number": i + 1,
                "signing-key": signature.signingKey,
                "creation-time": this._formatDate(new Date(signature.creationTime)),
                "type": this._formatType(signature.type),
                "version": signature.version
            });
        }

        return "<div>" + message + "</div>";
    },

    _formatType: function(type) {
        return this.resources.getString("KEY_MANAGER_PGP_KEYS_SIGNATURE_TYPE_" + type);
    },

    _formatDate: function(value) {
        return new jscape.DateFormat("{MM}-{dd}-{yy}").format(value);
    }
});

/** @class */
jscape.keystore.PgpKeysView = jscape.keystore.KeysTreeView.extend(
    /** @lends {jscape.keystore.PgpKeysView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.tree = $("#kmpgpk-data");
        this.keyInfo = new jscape.keystore.PgpKeyView(resources);
        this.importButton = $("#kmpgpk-import");
        this.exportButton = $("#kmpgpk-export");
        this.exportMenu = $("#kmpgpk-export-menu");
        this.signButton = $("#kmpgpk-sign");
        this.generateButton = $("#kmpgpk-generate");
        this.deleteButton = $("#kmpgpk-delete");
        this.signaturesButton = $("#kmpgpk-signatures");

        this._init();
    },

    /** @override */
    show: function(listener) {
        this._super(listener || jscape.keystore.PgpKeysView.LISTENER);
    },

    showImportDialog: function(listener) {
        var dialog = new jscape.keystore.ImportPgpKeyDialog(this.resources);
        dialog.reset();
        dialog.show(listener);
    },

    showExportPublicKeyDialog: function(listener) {
        var dialog = new jscape.keystore.ExportPgpPublicKeyDialog(this.resources);
        dialog.reset();
        dialog.show(listener);
    },

    showExportSecretKeyDialog: function(listener) {
        var dialog = new jscape.keystore.ExportPgpSecretKeyDialog(this.resources);
        dialog.reset();
        dialog.show(listener);
    },

    showGenerateDialog: function(listener) {
        var dialog = new jscape.keystore.GeneratePgpKeyDialog(this.resources);
        dialog.setAlgorithms([
            jscape.keystore.KeyPairAlgorithm.rsa(),
            jscape.keystore.KeyPairAlgorithm.dsa()
        ]);
        dialog.reset();
        dialog.show(listener);
    },

    showSignKeyDialog: function(listener, signingKeys) {
        var dialog = new jscape.keystore.SingPgpKeyDialog(this.resources);
        dialog.setSigningKeys(signingKeys);
        dialog.reset();
        dialog.show(listener);
    },

    showViewSignaturesDialog: function(alias, signatures) {
        var dialog = new jscape.keystore.PgpKeySignaturesDialog(this.resources);
        dialog.show(signatures);
    },

    showConfirmDeleteSecretKeyDialog: function(alias) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_PGP_KEYS_DELETE_PUBLIC_KEY_TITLE"),
                                            this.resources.getString("KEY_MANAGER_PGP_KEYS_DELETE_PUBLIC_KEY_MESSAGE", {0: alias}));
    },

    showConfirmDeletePublicKeyDialog: function(alias) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_PGP_KEYS_DELETE_PUBLIC_KEY_TITLE"),
                                            this.resources.getString("KEY_MANAGER_PGP_KEYS_DELETE_PUBLIC_KEY_MESSAGE", {0: alias}));
    },

    getPublicKeys: function() {
        return this._getKeys(this._findPublicKeysRoot());
    },

    setPublicKeys: function(values, selectIndex) {
        this._setKeys(this._findPublicKeysRoot(), values, selectIndex);
    },

    getSecretKeys: function() {
        return this._getKeys(this._findSecretKeysRoot());
    },

    setSecretKeys: function(values, selectIndex) {
        this._setKeys(this._findSecretKeysRoot(), values, selectIndex);
    },

    _init: function() {
        this._initComponents();
        this._initListeners();
    },

    /** @override */
    _initComponents: function() {
        this._super();

        this.importButton.linkbutton();
        this.exportButton.linkbutton({disabled: true});
        this.exportMenu.menu();
        this.signButton.linkbutton({disabled: true});
        this.generateButton.linkbutton();
        this.deleteButton.linkbutton({disabled: true});
        this.signaturesButton.linkbutton({disabled: true});
    },

    _initListeners: function() {
        this.importButton.click($.proxy(this._onClick$Import, this));
        this.generateButton.click($.proxy(this._onClick$Generate, this));
        this.exportButton.click($.proxy(this._onClick$Export, this));
        this.deleteButton.click($.proxy(this._onClick$Delete, this));
        this.signButton.click($.proxy(this._onClick$Sign, this));
        this.signaturesButton.click($.proxy(this._onClick$ViewSignatures, this));
        this.exportMenu.menu({onClick: $.proxy(this._onMenu$Export, this)});
    },

    /** @override */
    _initState: function() {
        this.tree.tree("loadData", [
            {id: "secret-keys", text: this.resources.getString("KEY_MANAGER_PGP_KEYS_NODE_SECRET_KEYS"), iconCls: "icon-keys"},
            {id: "public-keys", text: this.resources.getString("KEY_MANAGER_PGP_KEYS_NODE_PUBLIC_KEYS"), iconCls: "icon-keys"}
        ]);
        this._super();
    },

    /** @override */
    _key2Node: function (key) {
        var entities = {"<": "&lt;", ">": "&gt;", "&": "&amp;", '"': "&quot;"};
        return {
            id: key.alias,
            text: String(key.alias).replace(/[<>&"]/g, function (e) {return entities[e];}),
            iconCls: "icon-key",
            attributes: {key: key},
            children: []
        };
    },

    /** @override */
    _adjustButtonState: function() {
        var secretKey = this._getSelectedSecretKey();
        var publicKey = this._getSelectedPublicKey();

        this._setButtonEnabled(this.exportButton, secretKey != null || publicKey != null);
        this._setButtonEnabled(this.signButton, publicKey != null && this.getSecretKeys().length != 0);
        this._setButtonEnabled(this.deleteButton, secretKey != null || publicKey != null);
        this._setButtonEnabled(this.signaturesButton, secretKey != null || publicKey != null);
    },

    _onClick$Import: function() {
        this.listener.onImport();
        return false;
    },

    _onClick$Generate: function() {
        this.listener.onGenerate();
        return false;
    },

    _onClick$Export: function() {
        var publicKey = this._getSelectedPublicKey();
        if (publicKey) {
            this.listener.onExportPublicKey(publicKey);
        } else {
            if (this._buttonEnabled(this.exportButton) && this._getSelectedSecretKey() != null) {
                this.exportMenu.menu("show", {alignTo: this.exportButton});
                this.exportButton.blur();
            }
        }
        return false;
    },

    _onClick$Sign: function() {
        if (this._buttonEnabled(this.signButton)) {
            this.listener.onSign(this._getSelectedPublicKey());
        }
        return false;
    },

    _onClick$Delete: function() {
        var secretKey = this._getSelectedSecretKey();
        if (secretKey) {
            this.listener.onDeleteSecretKey(secretKey);
        } else {
            var publicKey = this._getSelectedPublicKey();
            if (publicKey) {
                this.listener.onDeletePublicKey(publicKey);
            }
        }
        return false;
    },

    _onClick$ViewSignatures: function() {
        var key = this._getSelectedSecretKey();
        if (key != null) {
            this.listener.onViewSecretKeySignatures(key);
        } else {
            key = this._getSelectedPublicKey();
            if (key != null) {
                this.listener.onViewPublicKeySignatures(key);
            }
        }
        return false;
    },

    _onMenu$Export: function(item) {
        var type = item.name;
        if ("menu-secret" == type) {
            this.listener.onExportSecretKey(this._getSelectedSecretKey());
        } else if ("menu-pubkey" == type) {
            this.listener.onExportSecretPublicKey(this._getSelectedSecretKey());
        } else {
            throw "Unsupported export format.";
        }
        return false;
    },

    _getSelectedSecretKey: function() {
        var node = this.tree.tree("getSelected");
        var parent = node && this.tree.tree("getParent", node.target);
        return parent && parent.id == "secret-keys" ? this._node2Key(node) : null;
    },

    _getSelectedPublicKey: function() {
        var node = this.tree.tree("getSelected");
        var parent = node && this.tree.tree("getParent", node.target);
        return parent && parent.id == "public-keys" ? this._node2Key(node) : null;
    },

    _findSecretKeysRoot: function() {
        return this.tree.tree("find", "secret-keys");
    },

    _findPublicKeysRoot: function() {
        return this.tree.tree("find", "public-keys");
    }
});
/** @const */
jscape.keystore.PgpKeysView.LISTENER = {
    onImport: $.noop(),
    onGenerate: $.noop(),
    onExportSecretKey: $.noop(),
    onExportPublicKey: $.noop(),
    onDeleteSecretKey: $.noop(),
    onDeletePublicKey: $.noop(),
    onSign: $.noop(),
    onViewSecretKeySignatures: $.noop(),
    onViewPublicKeySignatures: $.noop()
};

/** @class */
jscape.keystore.PgpKeyView = jscape.keystore.KeyInfoView.extend(
    /** @lends {jscape.keystore.PgpKeyView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmpgpk-info"), resources);

        this.algorithm = $("#kmpgpk-info-algorithm");
        this.canEncrypt = $("#kmpgpk-info-encrypt");
        this.canDecrypt = $("#kmpgpk-info-decrypt");
        this.canSign = $("#kmpgpk-info-sign");
        this.canVerify = $("#kmpgpk-info-verify");
        this.fingerprint = $("#kmpgpk-info-fingerprint");

        this._initComponents();
    },

    /** @override */
    _initState: function() {
        this.algorithm.text(this.data.algorithm);
        this.canEncrypt.text(this._formatBoolean(this.data.canEncrypt));
        this.canDecrypt.text(this._formatBoolean(this.data.canDecrypt));
        this.canSign.text(this._formatBoolean(this.data.canSign));
        this.canVerify.text(this._formatBoolean(this.data.canVerify));
        this.fingerprint.text(this.data.fingerprint);
    }
});