jscape.keystore = jscape.keystore || {};
/** @class */
jscape.keystore.ServerKeysController = jscape.ApplicationModule.extend(
    /** @lends {jscape.keystore.ServerKeysController.prototype} */
{
    view: null,
    serverKeys: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.keystore.ServerKeysView(resources);

        this._initValidators(resources);
    },

    start: function(listener, managementService, keys) {
        this.serverKeys = keys.getServerKeys();

        this._setupView();

        this._super(listener, managementService);
    },

    onImport: function() {
        this.view.showImportDialog({
            onSubmit: $.proxy(this._onSubmit$ImportDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onImportCertificates: function(key) {
        this.view.showImportCertificatesDialog({
            onSubmit: $.proxy(this._onSubmit$ImportCertificatesDialog, this, key),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onExportCertificate: function(certificate) {
        this.view.showExportCertificateDialog({
            onSubmit: $.proxy(this._onSubmit$ExportCertificateDialog, this, certificate),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, certificate.alias);
    },

    onExportPublicKey: function(key) {
        this.view.showExportPublicKeyDialog({
            onSubmit: $.proxy(this._onSubmit$ExportPublicKeyDialog, this, key),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, key.alias);
    },

    onExportPrivateKey: function(key) {
        this.view.showExportPrivateKeyDialog({
            onSubmit: $.proxy(this._onSubmit$ExportPrivateKeyDialog, this, key),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, key.alias);
    },

    onGenerate: function() {
        this.view.showGenerateDialog({
            onSubmit: $.proxy(this._onSubmit$GenerateDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onGenerateCsr: function(key) {
        this.view.showGenerateCsrDialog({
            onSubmit: $.proxy(this._onSubmit$GenerateCsrDialog, this, key),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onDelete: function(key) {
        this.view.showConfirmDeleteDialog(key.alias).done($.proxy(function(key) {
            var d = $.Deferred();
            d.then($.proxy(function(key) {
                var p = this.managementService.deleteServerKey(key.alias);
                p.done($.proxy(this._onComplete$DeleteKey, this, key));
            }, this, key));
            d.fail($.proxy(function(alias) {
                this.view.showKeyUsedError(alias);
            }, this, key.alias));

            this.managementService.keyUsage(key.alias).done(function(used) {
                if (used) {
                    d.reject();
                } else {
                    d.resolve();
                }
            });
        }, this, key));
    },

    _onSubmit$ImportDialog: function(dialog) {
        dialog.hide();
        this.managementService.importServerKey(dialog.getAlias(),
                                               dialog.getFile(),
                                               dialog.getFilePassword(),
                                               dialog.getKeyAlias(),
                                               dialog.getKeyPassword())
                .done($.proxy(this._onComplete$ImportKey, this));
    },

    _onSubmit$ImportCertificatesDialog: function(certificate, dialog) {
        dialog.hide();
        this.managementService.importServerKeyCertificates(certificate.alias,
                                                           dialog.getFile(),
                                                           dialog.getFilePassword(),
                                                           dialog.getAlias());
    },

    _onSubmit$ExportCertificateDialog: function(certificate, dialog) {
        dialog.hide();
        this.managementService.exportServerCertificate(certificate.alias,
                                                       dialog.getFormat(),
                                                       dialog.getFilename());
    },

    _onSubmit$ExportPublicKeyDialog: function(key, dialog) {
        dialog.hide();
        this.managementService.exportServerPublicKey(key.alias,
                                                     dialog.getFormat(),
                                                     dialog.getFilename());
    },

    _onSubmit$ExportPrivateKeyDialog: function(key, dialog) {
        dialog.hide();
        this.managementService.exportServerPrivateKey(key.alias,
                                                      dialog.getFormat(),
                                                      dialog.getPassword(),
                                                      dialog.getFilename());
    },

    _onSubmit$GenerateDialog: function(dialog) {
        dialog.hide();
        this.managementService.generateServerKey(dialog.getAlias(),
                                                 dialog.getAlgorithm(),
                                                 dialog.getKeyLength(),
                                                 dialog.getPeriod(),
                                                 dialog.getCommonName(),
                                                 dialog.getOrganizationUnit(),
                                                 dialog.getOrganization(),
                                                 dialog.getLocality(),
                                                 dialog.getState(),
                                                 dialog.getCountry())
                .done($.proxy(this._onComplete$GenerateKey, this));
    },

    _onSubmit$GenerateCsrDialog: function(key, dialog) {
        dialog.hide();
        this.managementService.generateServerKeyCSR(key.alias, dialog.getFilename())
                .done($.proxy(this._onComplete$GenerateCSR, this));
    },

    _onCancel$Dialog: function(dialog) {
        dialog.hide();
    },

    _onComplete$ImportKey: function(key) {
        this.serverKeys.push(key);
        this.view.setKeys(this.serverKeys, $.inArray(key, this.serverKeys));
    },

    _onComplete$GenerateKey: function(key) {
        this.serverKeys.push(key);
        this.view.setKeys(this.serverKeys, $.inArray(key, this.serverKeys));
    },

    _onComplete$GenerateCSR: function(key) {
        // ok
    },

    _onComplete$DeleteKey: function(key) {
        var index = $.inArray(key, this.serverKeys);
        if (index != -1) {
            this.serverKeys.splice(index, 1);
            this.view.setKeys(this.serverKeys, index);
        }
    },

    _setupView: function() {
        this.view.show(this);
        this.view.setKeys(this.serverKeys);
    },

    _initValidators: function(resources) {
        $.extend($.fn.validatebox.defaults.rules, {
            'serverkeys_filename_nonempty': {
                validator: $.proxy(this._validatorNonEmptyField, this),
                message: resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_FILENAME")
            },
            'serverkeys_file_nonempty': {
                validator: $.proxy(this._validatorNonEmptyField, this),
                message: resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_FILE")
            },
            'serverkeys_alias_nonempty': {
                validator: $.proxy(this._validatorNonEmptyField, this),
                message: resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_ALIAS")
            },
            'serverkeys_notexists': {
                validator: $.proxy(this._validatorAliasExists, this),
                message: resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_ALIAS_EXISTS")
            }
        });
    },

    _validatorNonEmptyField: function(value) {
        return value && value.trim().length != 0;
    },

    _validatorAliasExists: function(value, excludes) {
        var keys = this.serverKeys;
        for (var i = 0; i < keys.length; i++) {
            var alias = keys[i].alias;
            if (excludes && $.inArray(alias, excludes) != -1) {
                continue;
            }
            if (alias == value) {
                return false;
            }
        }
        return true;
    }
});

/** @class */
jscape.keystore.ImportServerKeyDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.ImportServerKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmsk-dialog-import"), resources);

        this.alias = $("#kmsk-import-alias");
        this.file = $("#kmsk-import-file");
        this.filePassword = $("#kmsk-import-filepwd");
        this.keyAlias = $("#kmsk-import-keyalias");
        this.keyPassword = $("#kmsk-import-keypwd");

        this._initValidators();
    },

    reset: function() {
        this.setAlias("");
        this.file.val("");
        this.filePassword.val("");
        this.keyAlias.val("");
        this.keyPassword.val("");
    },

    getAlias: function() {
        return this.alias.val();
    },

    setAlias: function(value) {
        this.alias.val(value);
        this._initAliasValidator(value);
    },

    getFile: function() {
        var files = this.file[0].files;
        return files && files[0];
    },

    getFilePassword: function() {
        return this.filePassword.val();
    },

    getKeyAlias: function() {
        return this.keyAlias.val();
    },

    getKeyPassword: function() {
        return this.keyPassword.val();
    },

    /** @override */
    _valid: function() {
        return this.alias.validatebox("isValid") && this.file.validatebox("isValid");
    },

    _initValidators: function() {
        this._initAliasValidator();
        this._initFileValidator();
    },

    _initAliasValidator: function(value) {
        this.alias.validatebox({
            required: true,
            validType: ["serverkeys_alias_nonempty", "serverkeys_notexists" + (value ? "['" + value + "']" : "")],
            missingMessage: this.resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_ALIAS")
        });
    },

    _initFileValidator: function() {
        this.file.validatebox({
            required: true,
            validType: "serverkeys_file_nonempty",
            missingMessage: this.resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_FILE")
        });
    }
});

/** @class */
jscape.keystore.ImportCertificatesDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.ImportCertificatesDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmsk-dialog-importcertificates"), resources);

        this.file = $("#kmsk-importcrt-file");
        this.filePassword = $("#kmsk-importcrt-filepwd");
        this.alias = $("#kmsk-importcrt-crtalias");

        this._initValidators();
    },

    reset: function() {
        this.file.val("");
        this.filePassword.val("");
        this.alias.val("");
    },

    getFile: function() {
        var files = this.file[0].files;
        return files && files[0];
    },

    getFilePassword: function() {
        return this.filePassword.val();
    },

    getAlias: function() {
        return this.alias.val();
    },

    /** @override */
    _valid: function() {
        return this.file.validatebox("isValid");
    },

    _initValidators: function() {
        this._initFileValidator();
    },

    _initFileValidator: function() {
        this.file.validatebox({
            required: true,
            validType: "serverkeys_file_nonempty",
            missingMessage: this.resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_FILE")
        });
    }
});

/** @class */
jscape.keystore.GenerateServerKeyDialog = jscape.keystore.GenerateKeyDialog.extend(
    /** @lends {jscape.keystore.GenerateServerKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmsk-dialog-generate"), resources);

        this.alias = $("#kmsk-generate-alias");
        this.keyAlgorithm = $("#kmsk-generate-algorithm");
        this.keyLength = $("#kmsk-generate-length");
        this.period = $("#kmsk-generate-period");
        this.commonName = $("#kmsk-generate-cn");
        this.organizationUnit = $("#kmsk-generate-ou");
        this.organization = $("#kmsk-generate-o");
        this.locality = $("#kmsk-generate-l");
        this.state = $("#kmsk-generate-s");
        this.country = $("#kmsk-generate-c");

        this._init();
    },

    /** @override */
    _initValidators: function() {
        this.alias.validatebox({
            required: true,
            validType: ["serverkeys_alias_nonempty", "serverkeys_notexists"],
            missingMessage: this.resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_ALIAS")
        });
    }
});

/** @class */
jscape.keystore.GenerateCsrDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.keystore.GenerateCsrDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmsk-dialog-generatecsr"), resources);

        this.filename = $("#kmsk-generatecsr-filename");

        this._initValidators();
    },

    reset: function() {
        this.filename.val("");
    },

    getFilename: function() {
        return this.filename.val();
    },

    /** @override */
    _valid: function() {
        return this.filename.validatebox("isValid");
    },

    _initValidators: function() {
        this._initFileValidator();
    },

    _initFileValidator: function() {
        this.filename.validatebox({
            required: true,
            validType: "serverkeys_filename_nonempty",
            missingMessage: this.resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_FILENAME")
        });
    }
});

/** @class */
jscape.keystore.ServerKeysView = jscape.keystore.KeysTreeView.extend(
    /** @lends {jscape.keystore.ServerKeysView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.tree = $("#kmsk-data");
        this.keyInfo = new jscape.keystore.ServerKeyView(resources);
        this.importButton = $("#kmsk-import");
        this.importCertificatesButton = $("#kmsk-importcrt");
        this.exportButton = $("#kmsk-export");
        this.exportMenu = $("#kmsk-export-menu");
        this.generateButton = $("#kmsk-generate");
        this.generateCsrButton = $("#kmsk-generatecsr");
        this.deleteButton = $("#kmsk-delete");

        this._init();
    },

    /** @override */
    show: function(listener) {
        this._super(listener || jscape.keystore.ServerKeysView.LISTENER);
    },

    showImportDialog: function(listener) {
        var dialog = new jscape.keystore.ImportServerKeyDialog(this.resources);
        dialog.reset();
        dialog.show(listener);
    },

    showImportCertificatesDialog: function(listener) {
        var dialog = new jscape.keystore.ImportCertificatesDialog(this.resources);
        dialog.reset();
        dialog.show(listener);
    },

    showExportCertificateDialog: function(listener, alias) {
        var dialog = new jscape.keystore.ExportCertificateDialog(this.resources);
        dialog.reset();
        dialog.setFilename(alias + ".crt");
        dialog.show(listener);
    },

    showExportPublicKeyDialog: function(listener, alias) {
        var dialog = new jscape.keystore.ExportPublicKeyDialog(this.resources);
        dialog.reset();
        dialog.setFilename(alias + ".pub");
        dialog.show(listener);
    },

    showExportPrivateKeyDialog: function(listener, alias) {
        var dialog = new jscape.keystore.ExportPrivateKeyDialog(this.resources);
        dialog.reset();
        dialog.setFilename(alias + ".prv");
        dialog.show(listener);
    },

    showGenerateDialog: function(listener) {
        var dialog = new jscape.keystore.GenerateServerKeyDialog(this.resources);
        dialog.setAlgorithms([
            jscape.keystore.KeyPairAlgorithm.rsa(),
            jscape.keystore.KeyPairAlgorithm.dsa()
        ]);
        dialog.reset();
        dialog.show(listener);
    },

    showGenerateCsrDialog: function(listener) {
        var dialog = new jscape.keystore.GenerateCsrDialog(this.resources);
        dialog.reset();
        dialog.show(listener);
    },

    showConfirmDeleteDialog: function(alias) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_SERVER_KEYS_DELETE_TITLE"),
                                            this.resources.getString("KEY_MANAGER_SERVER_KEYS_DELETE_MESSAGE", {0: alias}));
    },

    showKeyUsedError: function(alias) {
        $.messager.alert(this.resources.getString("DIALOG_TITLE_ERROR"),
                         this.resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_KEY_USED", {0: alias}),
                         "error");
    },

    getKeys: function() {
        return this._getKeys(null);
    },

    setKeys: function(keys, selectIndex) {
        this._setKeys(null, keys, selectIndex);
    },

    _init: function() {
        this._initComponents();
        this._initListeners();
    },

    _initComponents: function() {
        this._super();

        this.importButton.linkbutton();
        this.generateButton.linkbutton();
        this.exportButton.linkbutton({disabled: true});
        this.exportMenu.menu();
        this.deleteButton.linkbutton({disabled: true});
        this.generateCsrButton.linkbutton({disabled: true});
        this.importCertificatesButton.linkbutton({disabled: true});
    },

    _initListeners: function() {
        this.exportMenu.menu({onClick: $.proxy(this._onMenu$Export, this)});
        this.exportButton.click($.proxy(this._onClick$Export, this));

        this.importButton.click($.proxy(this._onClick$Import, this));
        this.importCertificatesButton.click($.proxy(this._onClick$ImportCertificates, this));
        this.generateButton.click($.proxy(this._onClick$Generate, this));
        this.generateCsrButton.click($.proxy(this._onClick$GenerateCSR, this));
        this.deleteButton.click($.proxy(this._onClick$Delete, this));
    },

    _onClick$Export: function() {
        if (this._buttonEnabled(this.exportButton) && this._getSelectedKey() != null) {
            this.exportMenu.menu("show", {alignTo: this.exportButton});
            this.exportButton.blur();
        }
        return false;
    },

    _onClick$Import: function () {
        if (this._buttonEnabled(this.importButton)) {
            this.listener.onImport();
        }
        return false;
    },

    _onClick$ImportCertificates: function () {
        if (this._buttonEnabled(this.importCertificatesButton)) {
            this.listener.onImportCertificates(this._getSelectedKey());
        }
        return false;
    },

    _onClick$Generate: function() {
        if (this._buttonEnabled(this.generateButton)) {
            this.listener.onGenerate();
        }
        return false;
    },

    _onClick$GenerateCSR: function() {
        if (this._buttonEnabled(this.generateCsrButton)) {
            this.listener.onGenerateCsr(this._getSelectedKey());
        }
        return false;
    },

    _onClick$Delete: function() {
        if (this._buttonEnabled(this.deleteButton)) {
            this.listener.onDelete(this._getSelectedKey());
        }
        return false;
    },

    _onMenu$Export: function(item) {
        var type = item.name;
        if ("menu-crt" == type) {
            this.listener.onExportCertificate(this._getSelectedKey());
        } else if ("menu-pubkey" == item.name) {
            this.listener.onExportPublicKey(this._getSelectedKey());
        } else if ("menu-prvkey" == item.name) {
            this.listener.onExportPrivateKey(this._getSelectedKey());
        } else {
            throw "Unsupported export format.";
        }
    },

    /** @override */
    _adjustButtonState: function() {
        var enabled = this._getSelectedKey() != null;

        this._setButtonEnabled(this.importCertificatesButton, enabled);
        this._setButtonEnabled(this.exportButton, enabled);
        this._setButtonEnabled(this.generateCsrButton, enabled);
        this._setButtonEnabled(this.deleteButton, enabled);
    }
});
/** @const */
jscape.keystore.ServerKeysView.LISTENER = {
    onImport: function() {},
    onImportCertificates: function(key) {},
    onExportCertificate: function(crt) {},
    onExportPublicKey: function(key) {},
    onExportPrivateKey: function(key) {},
    onGenerate: function() {},
    onGenerateCsr: function(key) {},
    onDelete: function(key) {}
};

/** @class */
jscape.keystore.ServerKeyView = jscape.keystore.KeyInfoView.extend(
    /** @lends {jscape.keystore.ServerKeyView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#kmsk-info"), resources);

        this.algorithm = $("#kmsk-info-algorithm");
        this.certificate = $("#kmsk-info-certificate");
        this.version = $("#kmsk-info-version");
        this.serial = $("#kmsk-info-serialnum");
        this.issuer = $("#kmsk-info-issuer");
        this.subject = $("#kmsk-info-subject");
        this.validFromDate = $("#kmsk-info-validfrom");
        this.validToDate = $("#kmsk-info-validto");
        this.signature = $("#kmsk-info-signature");

        this._initComponents();
    },

    /** @override */
    _initState: function() {
        this.algorithm.text(this.data.algorithm);
        this.certificate.text(this.data.certificateType);
        this.version.text(this.data.version);
        this.serial.text(this.data.serial);
        this.issuer.text(this.data.issuer);
        this.subject.text(this.data.subject);
        this.validFromDate.text(this._formatDate(new Date(this.data.validNotBefore)));
        this.validToDate.text(this._formatDate(new Date(this.data.validNotAfter)));
        this.signature.text(this.data.signature);
    }
});