/** @class */
jscape.AuthCredentials = Class.extend(
    /** @lends {jscape.AuthCredentials.prototype} */
{
    /** @constructor */
    initialize: function(username, password) {
        this.username = username;
        this.password = password;
    }
});

/** @class */
jscape.LoginController = Class.extend(
    /** @lends {jscape.LoginController.prototype} */
{
    /** @constructor */
    initialize: function(config, resources) {
        this.resources = resources;
        this.continueUrl = config.continuePath || "/";
        this.view = new jscape.LoginView(resources);
        this.operation = new jscape.LoginOperation(config.timeout, resources);
    },

    start: function() {
        this.view.show(this);
    },

    onLogin: function() {
        this.operation.start(this, this.view.getCredentials());
    },

    onCompleted: function() {
        window.location = this.continueUrl;
    },

    onBroken: function() {
        // ok
    }
});

/** @class */
jscape.LoginView = Class.extend(
    /** @lends {jscape.LoginView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.form = $("#signin-form");
        this.panel = $("#signin-box");
        this.username = $("#username");
        this.password = $("#password");

        this._initComponents();
    },

    show: function(listener) {
        this.listener = listener || jscape.LoginView.LISTENER;
        this.username.focus();
    },

    getCredentials: function() {
        return new jscape.AuthCredentials(this.username.val().trim(), this.password.val());
    },

    _initComponents: function() {
        this.panel.panel({fit: true, border: true});
        this.username.validatebox({
            required: true,
            missingMessage: this.resources.getString("LOGIN_EMPTY_USERNAME"),
            invalidMessage: this.resources.getString("LOGIN_EMPTY_USERNAME")
        });
        this.form.form({onSubmit: $.proxy(this._onSubmit$Form, this)});
    },

    _onSubmit$Form: function() {
        if (this.form.form("validate")) {
            this.listener.onLogin();
        }
        return false;
    }
});
/** @const */
jscape.LoginView.LISTENER = {
    onLogin: function() {}
};

/** @class */
jscape.LoginOperation = jscape.RestOperation.extend(
    /** @lends {jscape.LoginOperation.prototype} */
{
    /** @constructor */
    initialize: function(timeout, resources) {
        this._super("/rest/management/login",
                    "POST",
                    timeout,
                    new jscape.WaitCursorDialog(),
                    new jscape.RestOperationErrorDialog("operationErrorDialog",
                                                        resources.getString("DIALOG_BUTTON_CLOSE"),
                                                        resources.getString("DIALOG_BUTTON_DETAILS"),
                                                        resources.getString("DIALOG_TITLE"),
                                                        resources));
    }
});