jscape.management = jscape.management || {};
/** @class */
jscape.management.ManagementService = Class.extend(
    /** @lends {jscape.management.ManagementService.prototype} */
{
    config: null,
    timeout: null,
    resources: null,

    /** @constructor */
    initialize: function(config, resources) {
        this.config = config;
        this.timeout = config.timeout || 180000;
        this.resources = resources;
    },

    logout: function() {
        return this._promise(new jscape.management.Operation("POST", "/logout", this.timeout, this.resources));
    },

    getServerSettings: function () {
        return $.when(this.getServerParameters(),
                      this.getManagementServiceConfiguration(),
                      this.getManagementAccessServiceConfiguration(),
                      this.getAdministrators(),
                      this.getDomainsAutoStart(),
                      this.getDbDrivers(),
                      this.getWebServiceConfiguration(),
                      this.getManagementWebServerConfiguration(),
                      this.getDocumentViewerConfiguration(),
                      this.getEmailServiceConfiguration(),
                      this.getRegistrationServiceConfiguration(),
                      this.getFailoverServiceConfiguration(),
                      this.getSearchingServiceConfiguration(),
                      this.getJmxServiceConfiguration(),
                      this.getKeys())
                .then(function(serverParameters,
                               managementServiceConfig,
                               managementAccessServiceConfig,
                               administrators,
                               domainsAutoStart,
                               dbDrivers,
                               webServiceDescriptor,
                               managementWebServerConfig,
                               documentViewerConfig,
                               emailServiceDescriptor,
                               selfRegistrationServiceDescriptor,
                               failoverServiceConfig,
                               searchingServiceConfig,
                               jmxServiceConfig,
                               keys) {
                           var settings = new jscape.ServerSettings();
                           settings.setServerParameters(serverParameters);
                           settings.setServerManagementServiceConfiguration(managementServiceConfig);
                           settings.setManagementAccessServiceConfiguration(managementAccessServiceConfig);
                           settings.setAdministrators(administrators);
                           settings.setDomainsAutoStart(domainsAutoStart);
                           settings.setDbDrivers(dbDrivers);
                           settings.setWebServiceDescriptor(webServiceDescriptor);
                           settings.setManagementWebServerConfiguration(managementWebServerConfig);
                           settings.setDocumentViewerConfiguration(documentViewerConfig);
                           settings.setEmailServiceDescriptor(emailServiceDescriptor);
                           settings.setSelfRegistrationServiceDescriptor(selfRegistrationServiceDescriptor);
                           settings.setFailoverServiceConfiguration(failoverServiceConfig);
                           settings.setSearchingServiceConfiguration(searchingServiceConfig);
                           settings.setJmxServiceConfiguration(jmxServiceConfig);
                           settings.setKeys(keys);

                           return settings;
                       });
    },

    setServerSettings: function(settings) {
        var d = $.Deferred();
        d.then(this.setManagementServiceConfiguration(settings.getServerManagementServiceConfiguration())
                .done($.proxy(function(settings) {
                    $.when(this.setManagementAccessServiceConfiguration(settings.getManagementAccessServiceConfiguration()),
                           this.setAdministrators(settings.getAdministrators()),
                           this.setDomainsAutoStart(settings.getDomainsAutoStart()),
                           this.setWebServiceConfiguration(settings.getWebServiceDescriptor()),
                           //this.setManagementWebServerConfiguration(settings.getManagementWebServerConfiguration()),
                           this.setRegistrationServiceConfiguration(settings.getSelfRegistrationServiceDescriptor()),
                           this.setDocumentViewerConfiguration(settings.getDocumentViewerConfiguration()),
                           this.setDbDrivers(settings.getDbDrivers()),
                           this.setEmailServiceConfiguration(settings.getEmailServiceDescriptor()),
                           this.setFailoverServiceConfiguration(settings.getFailoverServiceConfiguration()),
                           this.setSearchingServiceConfiguration(settings.getSearchingServiceConfiguration()),
                           this.setJmxServiceConfiguration(settings.getJmxServiceConfiguration()))
                    .done(d.resolve)
                    .fail(d.reject);
                }, this, settings))
                .fail(d.reject));
        return d.promise();
    },

    getKeys: function () {
        return $.when(this.getServerKeys(),
                      this.getClientKeys(),
                      this.getClientCertificates(),
                      this.getHostKeys(),
                      this.getHostCertificates(),
                      this.getPgpPublicKeys(),
                      this.getPgpSecretKeys())
                .then(function (serverKeys, clientKeys, clientCertificates, hostKeys, hostCertificates, pgpPublicKeys, pgpSecretKeys) {
                        var keys = new jscape.Keys();
                        keys.setServerKeys(serverKeys);
                        keys.setClientKeys(clientKeys);
                        keys.setClientCertificates(clientCertificates);
                        keys.setHostKeys(hostKeys);
                        keys.setHostCertificates(hostCertificates);
                        keys.setPgpPublicKeys(pgpPublicKeys);
                        keys.setPgpSecretKeys(pgpSecretKeys);

                        return keys;
                });
    },

    getServerParameters: function() {
        var op = new jscape.management.Operation("GET", "/server/parameters", this.timeout, this.resources);
        return this._promise(op).then(function(data) {
            return jscape.ServerParameters.fromJSON(data);
        });
    },

    getServerStatistics: function() {
        var op = new jscape.management.Operation("GET", "/server/statistics", this.timeout, this.resources);
        return this._promise(op).then(function(data) {
            return jscape.ServerStatistics.fromJSON(data);
        });
    },

    getManagementServiceConfiguration: function() {
        var op = new jscape.management.Operation("GET", "/server/management-service/configuration", this.timeout, this.resources);
        return this._promise(op).then(function(data) {
            return jscape.ServerManagementServiceConfiguration.fromJSON(data);
        });
    },

    setManagementServiceConfiguration: function(config) {
        var op = new jscape.management.Operation("PUT", "/server/management-service/configuration", this.timeout, this.resources);
        return this._promise(op, config);
    },

    getManagementAccessServiceConfiguration: function() {
        var op = new jscape.management.Operation("GET", "/management-access-service/configuration", this.timeout, this.resources);
        return this._promise(op).then(function(data) {
            return jscape.ManagementAccessConfiguration.fromJSON(data);
        });
    },

    setManagementAccessServiceConfiguration: function(config) {
        var op = new jscape.management.Operation("PUT", "/management-access-service/configuration", this.timeout, this.resources);
        return this._promise(op, config);
    },

    getAdministrators: function() {
        var op = new jscape.management.Operation("GET", "/administrators", this.timeout, this.resources);
        return this._promise(op).then(function(data) {
            return jscape.AdministratorList.fromJSON(data);
        });
    },

    setAdministrators: function(administrators) {
        var op = new jscape.management.Operation("PUT", "/administrators", this.timeout, this.resources);
        return this._promise(op, administrators);
    },

    getDomainsAutoStart: function () {
        var op = new jscape.management.Operation("GET", "/domains-auto-start", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            return jscape.DomainsAutoStart.fromJSON(data);
        });
    },

    setDomainsAutoStart: function (domainAutoStart) {
        var op = new jscape.management.Operation("PUT", "/domains-auto-start", this.timeout, this.resources);
        return this._promise(op, domainAutoStart);
    },

    getWebServiceConfiguration: function () {
        var op = new jscape.management.Operation("GET", "/web-service/configuration", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            return jscape.WebServiceDescriptor.fromJSON(data);
        });
    },

    setWebServiceConfiguration: function (descriptor) {
        var op = new jscape.management.Operation("PUT", "/web-service/configuration", this.timeout, this.resources);
        return this._promise(op, descriptor);
    },

    getDbDrivers: function () {
        var op = new jscape.management.Operation("GET", "/db-drivers", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            return jscape.DbDrivers.fromJSON(data);
        });
    },

    setDbDrivers: function (drivers) {
        var op = new jscape.management.Operation("PUT", "/db-drivers", this.timeout, this.resources);
        return this._promise(op, drivers);
    },

    getManagementWebServerConfiguration: function () {
        var op = new jscape.management.Operation("GET", "/web-management/configuration", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            return jscape.ManagementWebServerConfiguration.fromJSON(data);
        });
    },

    setManagementWebServerConfiguration: function (config) {
        var op = new jscape.management.Operation("PUT", "/web-management/configuration", this.timeout, this.resources);
        return this._promise(op, config);
    },

    getRegistrationServiceConfiguration: function () {
        var op = new jscape.management.Operation("GET", "/registration-service/configuration", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            return jscape.SelfRegistrationServiceDescriptor.fromJSON(data);
        });
    },

    setRegistrationServiceConfiguration: function (config) {
        var op = new jscape.management.Operation("PUT", "/registration-service/configuration", this.timeout, this.resources);
        return this._promise(op, config);
    },

    getDocumentViewerConfiguration: function () {
        var op = new jscape.management.Operation("GET", "/document-viewer/configuration", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            return jscape.DocumentViewerConfiguration.fromJSON(data);
        });
    },

    setDocumentViewerConfiguration: function (config) {
        var op = new jscape.management.Operation("PUT", "/document-viewer/configuration", this.timeout, this.resources);
        return this._promise(op, config);
    },

    getEmailServiceConfiguration: function () {
        var op = new jscape.management.Operation("GET", "/email-service/configuration", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            return jscape.EmailServiceDescriptor.fromJSON(data);
        });
    },

    setEmailServiceConfiguration: function (config) {
        var op = new jscape.management.Operation("PUT", "/email-service/configuration", this.timeout, this.resources);
        return this._promise(op, config);
    },

    testEmailServer: function (config, recipient) {
        var op = new jscape.management.TestEmailServerOperation("POST", "/email-service/configuration/{recipient}/test", this.timeout, this.resources);
        return this._promise(op, config, recipient);
    },

    getFailoverServiceConfiguration: function () {
        var op = new jscape.management.Operation("GET", "/failover-service/configuration", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            return jscape.FailoverConfiguration.fromJSON(data);
        });
    },

    setFailoverServiceConfiguration: function (config) {
        var op = new jscape.management.Operation("PUT", "/failover-service/configuration", this.timeout, this.resources);
        return this._promise(op, config);
    },

    syncFailoverServerState: function(config) {
        var op = new jscape.management.Operation("POST", "/failover/sync", this.timeout, this.resources);
        return this._promise(op, config);
    },

    failoverSyncComplete: function() {
        return this._promise(new jscape.management.Operation("GET", "/failover/sync", this.timeout, this.resources));
    },

    getSearchingServiceConfiguration: function () {
        var op = new jscape.management.Operation("GET", "/searching-service/configuration", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            return jscape.SearchingServiceConfiguration.fromJSON(data);
        });
    },

    setSearchingServiceConfiguration: function (config) {
        var op = new jscape.management.Operation("PUT", "/searching-service/configuration", this.timeout, this.resources);
        return this._promise(op, config);
    },

    startSearchFileIndexing: function (config) {
        var op = new jscape.management.Operation("POST", "/searching-service/index", this.timeout, this.resources);
        return this._promise(op, config);
    },

    getJmxServiceConfiguration: function () {
        var op = new jscape.management.Operation("GET", "/jmx-service/configuration", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            return jscape.JmxServiceConfiguration.fromJSON(data);
        });
    },

    setJmxServiceConfiguration: function (config) {
        var op = new jscape.management.Operation("PUT", "/jmx-service/configuration", this.timeout, this.resources);
        return this._promise(op, config);
    },

    getDomainNames: function() {
        return this._promise(new jscape.management.DomainOperation("GET", "/domains", this.timeout, this.resources));
    },

    startDomain: function(name) {
        var op = new jscape.management.DomainOperation("POST", "/domains/{domainName}/start", this.timeout, this.resources);
        return this._promise(op, name);
    },

    stopDomain: function(name) {
        var op = new jscape.management.DomainOperation("POST", "/domains/{domainName}/stop", this.timeout, this.resources);
        return this._promise(op, name);
    },

    pauseDomain: function(name) {
        var op = new jscape.management.DomainOperation("POST", "/domains/{domainName}/pause", this.timeout, this.resources);
        return this._promise(op, name);
    },

    resumeDomain: function(name) {
        var op = new jscape.management.DomainOperation("POST", "/domains/{domainName}/resume", this.timeout, this.resources);
        return this._promise(op, name);
    },

    deleteDomain: function(name) {
        var op = new jscape.management.DomainOperation("DELETE", "/domains/{domainName}", this.timeout, this.resources);
        return this._promise(op, name);
    },

    getDomainStatus: function(name) {
        var op = new jscape.management.DomainOperation("POST", "/domains", this.timeout, this.resources);
        return this._promise(op, name);
    },

    storeDomain: function(descriptor) {
        var op = new jscape.management.DomainOperation("PUT", "/domains", this.timeout, this.resources);
        return this._promise(op, descriptor);
    },

    testDatastoreService: function(descriptor) {
        var op = new jscape.management.Operation("POST", "/services/datastore/test", this.timeout, this.resources);
        return this._promise(op, descriptor);
    },

    testLogService: function(descriptor) {
        var op = new jscape.management.Operation("POST", "/services/log/test", this.timeout, this.resources);
        return this._promise(op, descriptor);
    },

    getServerKeys: function() {
        var op = new jscape.management.Operation("GET", "/keys/server", this.timeout, this.resources);
        return this._promise(op).then(function(data) {
            var keys = new Array(data.length);
            for (var i = 0; i < data.length; i++) {
                keys[i] = jscape.KeyEntryDescriptor.fromJSON(data[i]);
            }
            return keys;
        });
    },

    getClientKeys: function() {
        var op = new jscape.management.Operation("GET", "/keys/client", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            var keys = new Array(data.length);
            for (var i = 0; i < data.length; i++) {
                keys[i] = jscape.KeyEntryDescriptor.fromJSON(data[i]);
            }
            return keys;
        });
    },

    getClientCertificates: function() {
        var op = new jscape.management.Operation("GET", "/certificates/client", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            var certificates = new Array(data.length);
            for (var i = 0; i < data.length; i++) {
                certificates[i] = jscape.KeyEntryDescriptor.fromJSON(data[i]);
            }
            return certificates;
        });
    },

    getHostKeys: function() {
        var op = new jscape.management.Operation("GET", "/keys/host", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            var keys = new Array(data.length);
            for (var i = 0; i < data.length; i++) {
                keys[i] = jscape.KeyEntryDescriptor.fromJSON(data[i]);
            }
            return keys;
        });
    },

    getHostCertificates: function() {
        var op = new jscape.management.Operation("GET", "/certificates/host", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            var certificates = new Array(data.length);
            for (var i = 0; i < data.length; i++) {
                certificates[i] = jscape.KeyEntryDescriptor.fromJSON(data[i]);
            }
            return certificates;
        });
    },

    getPgpPublicKeys: function() {
        var op = new jscape.management.Operation("GET", "/keys/pgp/public", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            var keys = new Array(data.length);
            for (var i = 0; i < data.length; i++) {
                keys[i] = jscape.PgpKeyEntryDescriptor.fromJSON(data[i]);
            }
            return keys;
        });
    },

    getPgpSecretKeys: function() {
        var op = new jscape.management.Operation("GET", "/keys/pgp/secret", this.timeout, this.resources);
        return this._promise(op).then(function (data) {
            var keys = new Array(data.length);
            for (var i = 0; i < data.length; i++) {
                keys[i] = jscape.PgpKeyEntryDescriptor.fromJSON(data[i]);
            }
            return keys;
        });
    },

    getClientPublicKey: function(alias) {
        var op = new jscape.management.KeyOperation("GET", "/keys/client/{alias}", this.timeout, this.resources);
        return this._promise(op, alias).then(function(data) {
            return jscape.KeyEntryDescriptor.fromJSON(data);
        });
    },

    getClientCertificate: function(alias) {
        var op = new jscape.management.KeyOperation("GET", "/certificates/client/{alias}", this.timeout, this.resources);
        return this._promise(op, alias).then(function(data) {
            return jscape.KeyEntryDescriptor.fromJSON(data);
        });
    },

    importServerKey: function(alias, file, filePassword, keyAlias, keyPassword) {
        var op = new jscape.management.ImportKeyOperation("POST", "/keys/server/{alias}/import", this.timeout, this.resources);
        return this._promise(op, alias, {
            file: file,
            filePassword: filePassword,
            keyAlias: keyAlias,
            keyPassword: keyPassword
        });
    },

    importServerKeyCertificates: function(alias, file, filePassword, keyAlias) {
        var op = new jscape.management.ImportKeyOperation("POST", "/keys/server/{alias}/import-crt", this.timeout, this.resources);
        return this._promise(op, alias, {
            file: file,
            filePassword: filePassword,
            keyAlias: keyAlias
        });
    },

    importHostKey: function(alias, file) {
        var op = new jscape.management.ImportKeyOperation("POST", "/keys/host/{alias}/import", this.timeout, this.resources);
        return this._promise(op, alias, {file: file});
    },

    importClientKey: function(alias, file) {
        var op = new jscape.management.ImportKeyOperation("POST", "/keys/client/{alias}/import", this.timeout, this.resources);
        return this._promise(op, alias, {file: file});
    },

    importPgpPublicKey: function(file) {
        var op = new jscape.management.ImportKeyOperation("POST", "/keys/pgp/public/import", this.timeout, this.resources);
        return this._promise(op, null, {file: file});
    },

    importPgpSecretKey: function(file, password) {
        var op = new jscape.management.ImportKeyOperation("POST", "/keys/pgp/secret/import", this.timeout, this.resources);
        return this._promise(op, null, {
            file: file,
            filePassword: password
        });
    },

    generateServerKey: function(alias, algorithm, keyLength, period, commonName, organizationUnit, organization, locality, state, country) {
        var op = new jscape.management.GenerateKeyOperation("POST", "/keys/server/{alias}/generate", this.timeout, this.resources);
        return this._promise(op, alias, {
            keyType: algorithm,
            keyLength: keyLength,
            period: period,
            commonName: commonName,
            organizationUnit: organizationUnit,
            organization: organization,
            locality: locality,
            state: state,
            country: country
        });
    },

    generateServerKeyCSR: function(alias, filename) {
        var op = new jscape.management.ExportKeyOperation("GET"/*todo: POST*/, "/keys/server/{alias}/generate-csr", this.timeout, this.resources);
        return this._promise(op, alias, {filename: filename});
    },

    generateClientKey: function(alias, algorithm, keyLength, period, commonName, organizationUnit, organization, locality, state, country, privateKeyFilename, privateKeyFormat, privateKeyPassword) {
        var op = new jscape.management.ExportKeyOperation("GET"/*todo: POST*/, "/keys/client/{alias}/generate", this.timeout, this.resources);
        return this._promise(op, alias, {
            keyType: algorithm,
            keyLength: keyLength,
            period: period,
            commonName: commonName,
            organizationUnit: organizationUnit,
            organization: organization,
            locality: locality,
            state: state,
            country: country,
            privateKeyFilename: privateKeyFilename,
            privateKeyFormat: privateKeyFormat,
            privateKeyPassword: privateKeyPassword
        });
    },

    generatePgpKey: function(realName, email, keyType, keyLength) {
        var op = new jscape.management.GeneratePgpKeyOperation("POST", "/keys/pgp/generate", this.timeout, this.resources);
        return this._promise(op, {
            realName: realName,
            email: email,
            keyType: keyType,
            keyLength: keyLength
        });
    },

    exportServerCertificate: function(alias, format, filename) {
        var op = new jscape.management.ExportKeyOperation("GET", "/keys/server/{alias}/export-crt", this.timeout, this.resources);
        return this._promise(op, alias, {
            format: format,
            filename: filename
        });
    },

    exportServerPublicKey: function(alias, format, filename) {
        var op = new jscape.management.ExportKeyOperation("GET", "/keys/server/{alias}/export-pub", this.timeout, this.resources);
        return this._promise(op, alias, {
            format: format,
            filename: filename
        });
    },

    exportServerPrivateKey: function(alias, format, password, filename) {
        var op = new jscape.management.ExportKeyOperation("GET", "/keys/server/{alias}/export-prv", this.timeout, this.resources);
        return this._promise(op, alias, {
            format: format,
            filename: filename,
            filePassword: password
        });
    },

    exportHostCertificate: function(alias, format, filename) {
        var op = new jscape.management.ExportKeyOperation("GET", "/certificates/host/{alias}/export", this.timeout, this.resources);
        return this._promise(op, alias, {
            format: format,
            filename: filename
        });
    },

    exportHostCertificatePublicKey: function(alias, format, filename) {
        var op = new jscape.management.ExportKeyOperation("GET", "/certificates/host/{alias}/export-pub", this.timeout, this.resources);
        return this._promise(op, alias, {
            format: format,
            filename: filename
        });
    },

    exportHostPublicKey: function(alias, format, filename) {
        var op = new jscape.management.ExportKeyOperation("GET", "/keys/host/{alias}/export", this.timeout, this.resources);
        return this._promise(op, alias, {
            format: format,
            filename: filename
        });
    },

    exportClientCertificate: function(alias, format, filename) {
        var op = new jscape.management.ExportKeyOperation("GET", "/certificates/client/{alias}/export", this.timeout, this.resources);
        return this._promise(op, alias, {
            format: format,
            filename: filename
        });
    },

    exportClientCertificatePublicKey: function(alias, format, filename) {
        var op = new jscape.management.ExportKeyOperation("GET", "/certificates/client/{alias}/export-pub", this.timeout, this.resources);
        return this._promise(op, alias, {
            format: format,
            filename: filename
        });
    },

    exportClientPublicKey: function(alias, format, filename) {
        var op = new jscape.management.ExportKeyOperation("GET", "/keys/client/{alias}/export", this.timeout, this.resources);
        return this._promise(op, alias, {
            format: format,
            filename: filename
        });
    },

    exportPgpPublicKey: function(alias, filename) {
        var op = new jscape.management.ExportKeyOperation("GET", "/keys/pgp/public/{alias}/export", this.timeout, this.resources);
        return this._promise(op, alias, {filename: filename});
    },

    exportPgpSecretKey: function(alias, filename, password) {
        var op = new jscape.management.ExportKeyOperation("GET", "/keys/pgp/secret/{alias}/export", this.timeout, this.resources);
        return this._promise(op, alias, {
            filename: filename,
            password: password
        });
    },

    exportPgpSecretPublicKey: function(alias, filename) {
        var op = new jscape.management.ExportKeyOperation("GET", "/keys/pgp/secret/{alias}/export-pub", this.timeout, this.resources);
        return this._promise(op, alias, {filename: filename});
    },

    deleteServerKey: function(alias) {
        var op = new jscape.management.KeyOperation("DELETE", "/keys/server/{alias}", this.timeout, this.resources);
        return this._promise(op, alias);
    },

    deleteHostPublicKey: function(alias) {
        var op = new jscape.management.KeyOperation("DELETE", "/keys/host/{alias}", this.timeout, this.resources);
        return this._promise(op, alias);
    },

    deleteHostCertificate: function(alias) {
        var op = new jscape.management.KeyOperation("DELETE", "/certificates/host/{alias}", this.timeout, this.resources);
        return this._promise(op, alias);
    },

    deleteClientPublicKey: function(alias) {
        var op = new jscape.management.KeyOperation("DELETE", "/keys/client/{alias}", this.timeout, this.resources);
        return this._promise(op, alias);
    },

    deleteClientCertificate: function(alias) {
        var op = new jscape.management.KeyOperation("DELETE", "/certificates/client/{alias}", this.timeout, this.resources);
        return this._promise(op, alias);
    },

    deletePgpPublicKey: function(alias) {
        var op = new jscape.management.KeyOperation("DELETE", "/keys/pgp/public/{alias}", this.timeout, this.resources);
        return this._promise(op, alias);
    },

    deletePgpSecretKey: function(alias) {
        var op = new jscape.management.KeyOperation("DELETE", "/keys/pgp/secret/{alias}", this.timeout, this.resources);
        return this._promise(op, alias);
    },

    getPgpPublicKeySignatures: function(alias) {
        var op = new jscape.management.SignaturesPgpKeyOperation("GET", "/keys/pgp/public/{alias}/signatures", this.timeout, this.resources);
        return this._promise(op, alias);
    },

    getPgpSecretKeySignatures: function(alias) {
        var op = new jscape.management.SignaturesPgpKeyOperation("GET", "/keys/pgp/secret/{alias}/signatures", this.timeout, this.resources);
        return this._promise(op, alias);
    },

    signPgpPublicKey: function(alias, secretKeyAlias) {
        var op = new jscape.management.SignPgpKeyOperation("PUT", "/keys/pgp/public/{alias}/sign", this.timeout, this.resources);
        return this._promise(op, alias, {secretKeyAlias: secretKeyAlias});
    },

    keyUsage: function(alias) {
        var op = new jscape.management.KeyOperation("GET", "/key-usage/{alias}", this.timeout, this.resources);
        return this._promise(op, alias);
    },

    _promise: function(operation) {
        var deferred = $.Deferred();
        operation.start.apply(operation, [
            {
                onCompleted: deferred.resolve,
                onBroken: deferred.reject
            }
        ].concat(Array.prototype.slice.call(arguments, 1)));
        return deferred.promise();
    }
});

/** @class */
jscape.management.Operation = jscape.RestOperation.extend(
    /** @lends {jscape.management.Operation.prototype} */
{
    /** @constructor */
    initialize: function (method, url, timeout, resources) {
        this._super("/rest/management" + url,
                    method || "GET",
                    timeout,
                    new jscape.WaitCursorDialog(),
                    new jscape.RestOperationErrorDialog("operationErrorDialog",
                                                        resources.getString("DIALOG_BUTTON_CLOSE"),
                                                        resources.getString("DIALOG_BUTTON_DETAILS"),
                                                        resources.getString("DIALOG_TITLE"),
                                                        resources));
    },

    /** @override */
    start: function (listener, data) {
        var tmp = this.url;
        try {
            var args = Array.prototype.slice.call(arguments, 1);
            this.url = this._formatUrl.apply(this, [this.url].concat(args));
            this._super(listener, data);
        } finally {
            this.url = tmp;
        }
    },

    /** @override */
    _parseResponse: function(response, status) {
        return status == 200 || status == 204 ? $.parseJSON(response) : response;
    },

    _formatUrl: function (url) {
        return url;
    }
});

/** @class */
jscape.management.TestEmailServerOperation = jscape.management.Operation.extend(
    /** @lends {jscape.TestEmailServerOperation.prototype} */
{
    /** @override */
    _formatUrl: function(url, config, recipient) {
        return url.supplant({recipient: recipient});
    }
});

/** @class */
jscape.management.DomainOperation = jscape.management.Operation.extend(
    /** @lends {jscape.DomainOperation.prototype} */
{
    /** @override */
    _formatUrl: function(url, domainName) {
        return domainName ? url.supplant({domainName: domainName}) : url;
    }
});

/** @class */
jscape.management.KeyOperation = jscape.management.Operation.extend(
    /** @lends {jscape.management.KeyOperation.prototype} */
{
    /** @override */
    _formatUrl: function(url, alias) {
        return url.supplant({alias: alias});
    }
});

/** @class */
jscape.management.ImportKeyOperation = jscape.management.KeyOperation.extend(
    /** @lends {jscape.management.ImportKeyOperation.prototype} */
{
    /** @override */
    start: function(listener, alias, data) {
        this.listener = listener || jscape.RestOperation.LISTENER;
        this.progressDialog.show();

        var operation = new jscape.AjaxOperation(this.timeout);
        operation.method = this.method || "POST";
        operation.contentType = false; // append boundary to multipart/form-data type
        operation.dataType = "text";

        operation.start({
            onComplete: $.proxy(this._handleComplete, this),
            onError: $.proxy(this._handleError, this)
        }, this._formatUrl(this.url, alias), this._formatData(data));
    },

    /** @override */
    _formatUrl: function(url, alias) {
        return alias ? this._super(url, alias) : url;
    },

    /** @override */
    _formatData: function(data) {
        var formData = new FormData();
        $.each(data, function(key, value) {
            formData.append(key, value);
        });
        return formData;
    },

    /** @override */
    _parseResponse: function(response, statusСode) {
        return statusСode == 201 ? this._parseKey($.parseJSON(response)) : this._super(response, statusСode);
    },

    _parseKey: function(data) {
        return jscape.KeyEntryDescriptor.fromJSON(data);
    }
});

/** @class */
jscape.management.ExportKeyOperation = jscape.management.KeyOperation.extend(
    /** @lends {jscape.management.ExportKeyOperation.prototype} */
{
    /** @override */
    start: function(listener, alias, data) {
        this.listener = listener || jscape.RestOperation.LISTENER;
        this.progressDialog.show();

        var token = "_" + new Date().getTime().toString(36);
        data["x-load-complete"] = token;

        jQuery.ajax(this._formatUrl(this.url, alias, data), {
            type: this.method,
            cache: false,
            timeout: this.timeoutMillis,
            contentType: this.contentType,
            processData: false,
            dataType: "iframe",
            loadCompleteToken: token,
            success: $.proxy(this._handleComplete, this),
            error: $.proxy(this._handleError, this)
        });
    },

    _handleError: function(request, status, error) {
        var errorData = new jscape.ErrorData(request.status, request.statusText, request.responseText, status, error);
        this.progressDialog.hide();
        this.errorMessageDialog.setError(errorData);
        this.errorMessageDialog.show({
            onSubmit: $.proxy(this._handleErrorDialog, this),
            onCancel: $.proxy(this._handleErrorDialog, this)
        });
    },

    _handleErrorDialog: function() {
        this.errorMessageDialog.hide();
        this.listener.onBroken();
    },

    /** @override */
    _formatUrl: function(url, alias, data) {
        return this._super(url, alias) + (data ? "?" + $.param(data) : "");
    },

    /** @override */
    _parseResponse: function(response) {
        return response;
    }
});

/** @class */
jscape.management.GenerateKeyOperation = jscape.management.KeyOperation.extend(
    /** @lends {jscape.management.GenerateKeyOperation.prototype} */
{
    /** @override */
    _formatUrl: function(url, alias, data) {
        return this._super(url, alias) + "?" + $.param(data);
    },

    /** @override */
    _parseResponse: function(response, statusСode) {
        return statusСode == 201 ? this._parseKey($.parseJSON(response)) : this._super(response, statusСode);
    },

    _parseKey: function(data) {
        return jscape.KeyEntryDescriptor.fromJSON(data);
    }
});

/** @class */
jscape.management.GeneratePgpKeyOperation = jscape.management.GenerateKeyOperation.extend(
    /** @lends {jscape.management.GeneratePgpKeyOperation.prototype} */
{
    /** @override */
    _formatUrl: function(url, data) {
        return url + "?" + $.param(data);
    },

    /** @override */
    _parseKey: function(data) {
        return jscape.PgpKeyEntryDescriptor.fromJSON(data);
    }
});

/** @class */
jscape.management.SignPgpKeyOperation = jscape.management.KeyOperation.extend(
    /** @lends {jscape.management.SignPgpKeyOperation.prototype} */
{
    /** @override */
    _formatUrl: function(url, alias, data) {
        return this._super(url, alias) + "?" + $.param(data);
    },

    /** @override */
    _parseResponse: function(response, statusСode) {
        return statusСode == 200
                ? jscape.PgpKeyEntryDescriptor.fromJSON($.parseJSON(response))
                : this._super(response, statusСode);
    }
});

/** @class */
jscape.management.SignaturesPgpKeyOperation = jscape.management.KeyOperation.extend(
    /** @lends {jscape.management.SignaturesPgpKeyOperation.prototype} */
{
    /** @override */
    _parseResponse: function(response, statusСode) {
        var signatures = [];
        if (200 == statusСode) {
            var data = $.parseJSON(response);
            for (var i = 0; i < data.length; i++) {
                signatures.push(jscape.PgpSignatureDescriptor.fromJSON(data[i]));
            }
        }
        return signatures;
    }
});


(function() {
    $.ajaxTransport("iframe", function(s) {
        if ("GET" !== s.type || !s.async) {
            return; // todo: POST
        }
        var iframe;
        var timerId;

        return {
            send: function(headers, callback) {
                iframe = document.createElement("iframe");
                $(iframe).css({width: 0, height: 0, border: "none", visibility: "hidden"});

                iframe.onload = function(_, abort) {
                    var success = document.cookie.indexOf(s.loadCompleteToken) != -1;
                    var doc = iframe && (iframe.contentDocument || iframe.contentWindow.document);
                    var error = doc && doc.body && doc.body.innerHTML.length != 0;

                    if (abort || success || error) {
                        var responses = error ? {text: doc.body.innerHTML} : {};

                        clearTimeout(timerId);
                        timerId = undefined;
                        iframe.onload = null;
                        setTimeout(function() { // FF bug fix
                            iframe && iframe.parentNode && iframe.parentNode.removeChild(iframe);
                            iframe = undefined;
                        }, 0);

                        if (success) {
                            callback(200, "success", responses);
                        } else if (error) {
                            callback(500, "error", responses);
                        }
                    } else {
                        if (iframe) {
                            timerId = setTimeout(iframe.onload, 1000);
                        }
                    }
                };
                iframe.src = s.url;
                document.body.appendChild(iframe);
                timerId = setTimeout(iframe.onload, 100);
            },

            abort: function() {
                iframe && iframe.onload(0, 1);
            }
        };
    });
})();