var jscape = jscape || {};

/** @class */
jscape.ServerInfoController = Class.extend(
    /** @lends {jscape.ServerInfoController.prototype} */
{
    view: null,
    modules: null,
    moduleQueue: null,
    managementService: null,
    serverParameters: null,

    /** @constructor */
    initialize: function (resources, modules) {
        this.modules = modules || [];
        this.view = new jscape.ServerInfoView(resources);
        this.menu = new jscape.Menu();
    },

    start: function (managementService) {
        this.managementService = managementService;

        this._initModuleQueue();

        this.managementService.getServerParameters().done($.proxy(function (parameters) {
            this.serverParameters = parameters;
            this._startNextModule();
        }, this));

        this.view.show(this);
    },

    onSubmit: function () {
        this._initModuleQueue();
        this._submitNextModule();
    },

    onCancel: function () {
        this._initModuleQueue();
        this._cancelNextModule();
    },

    onStarted: function () {
        if (this.moduleQueue.length == 0) {
            this.view.show(this);
        } else {
            this._startNextModule();
        }
    },

    onStartError: function () {},
    onSubmitError: function () {},

    onChangesSubmitted: function () {
        if (this.moduleQueue.length == 0) {
            this._redirect();
        } else {
            this._submitNextModule();
        }
    },

    onChangesCancelled: function () {
        if (this.moduleQueue.length == 0) {
            this._redirect();
        } else {
            this._cancelNextModule();
        }
    },

    _redirect: function() {
        window.location = "/";
    },

    _initModuleQueue: function () {
        this.moduleQueue = [].concat(this.modules);
    },

    _startNextModule: function () {
        var m = this.moduleQueue.pop();
        m.start(this, this.managementService, this.serverParameters);
        this.menu.bind(m, "server");
    },

    _submitNextModule: function () {
        var m = this.moduleQueue.pop();
        m.submitChanges();
        this.menu.unbind("server");
    },

    _cancelNextModule: function () {
        var m = this.moduleQueue.pop();
        m.cancelChanges();
        this.menu.unbind("server");
    }
});

/** @class */
jscape.ServerInfoView = Class.extend(
    /** @lends {jscape.ServerInfoView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.tabs = $("#viewpane");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.ServerInfoView.LISTENER;
        this._initState();
    },

    _init: function() {
        this._initComponents();
        this._initListeners();
    },

    _initComponents: function() {
        this.tabs.tabs({border: true, plain: true});
    },

    _initListeners: function() {
    },

    _initState: function() {
        this.tabs.css("visibility", "visible");
    }
});
jscape.ServerInfoView.LISTENER = {};


/** @class */
jscape.ServerStatisticsController = jscape.ApplicationModule.extend(
    /** @lends {jscape.ServerStatisticsController.prototype} */
{
    /** @const */
    UPDATE_PERIOD: 15000,

    serverParameters: null,
    statistics: null,
    timer: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.newDomainController = new jscape.NewDomainController(resources);
        this.serverStatsView = new jscape.ServerStatisticsView(resources);
        this.domainsView = new jscape.ServerDomainsView(resources);
    },

    start: function(listener, managementService, serverParameters) {
        this._super(listener, managementService);

        this.parameters = serverParameters;

        this._setupView();
    },

    onMenuItemSelected: function (item) {
        switch (item) {
            case jscape.Menu.ITEM.NewDomain:
                this.onAddDomain();
                return false;
            case jscape.Menu.ITEM.StartDomains:
                this._startAllDomains();
                return false;
            case jscape.Menu.ITEM.StopDomains:
                this._stopAllDomains();
                return false;
            case jscape.Menu.ITEM.PauseDomains:
                this._pauseAllDomains();
                return false;
            case jscape.Menu.ITEM.ResumeDomains:
                this._resumeAllDomains();
                return false;
            default:
                return true;
        }
    },

    onStartDomain: function(name) {
        this.managementService.startDomain(name).done($.proxy(this._refreshStatistics, this));
    },

    onStopDomain: function(name) {
        this.managementService.stopDomain(name).done($.proxy(this._refreshStatistics, this));
    },

    onPauseDomain: function(name) {
        this.managementService.pauseDomain(name).done($.proxy(this._refreshStatistics, this));
    },

    onResumeDomain: function(name) {
        this.managementService.resumeDomain(name).done($.proxy(this._refreshStatistics, this));
    },

    onAddDomain: function() {
        var d = new $.Deferred();
        this.newDomainController.start({
            onComplete: d.resolve,
            onCancel: d.reject
        }, this.managementService, this.parameters);

        d.done($.proxy(this._refreshStatistics, this));
    },

    onEditDomain: function() {
        //todo:
    },

    onDeleteDomain: function (name) {
        this.domainsView.showConfirmDeleteDialog(name).done($.proxy(function () {
            this.managementService.deleteDomain(name).done($.proxy(this._refreshStatistics, this));
        }, this));
    },

    onStatusDomain: function(name) {
        //todo: this.managementService.getDomainStatus(name).done($.proxy(this._refreshStatistics, this));
    },

    _startAllDomains: function() {
        this._domainBatchOperation(this.managementService.startDomain);
    },

    _stopAllDomains: function() {
        this._domainBatchOperation(this.managementService.stopDomain);
    },

    _pauseAllDomains: function() {
        this._domainBatchOperation(this.managementService.pauseDomain);
    },

    _resumeAllDomains: function() {
        this._domainBatchOperation(this.managementService.resumeDomain);
    },

    _domainBatchOperation: function (operation) {
        this.managementService.getDomainNames().done($.proxy(function (names) {
            for (var i = 0; i < names.length; i++) {
                operation.call(this.managementService, names[i]);
            }
            //todo: synchronize on done
            this._refreshStatistics();
        }, this));
    },

    _update: function() {
        clearTimeout(this.timer);
        this.timer = setTimeout($.proxy(function() {
            this._refreshStatistics().done($.proxy(this._update, this));
        }, this), this.UPDATE_PERIOD);
    },

    _refreshStatistics: function () {
        return this.managementService.getServerStatistics().done($.proxy(function (statistics) {
            this.statistics = statistics;
            this.serverStatsView.setStatistics(this.parameters, this.statistics);
            this.domainsView.setDomains(this.statistics.domainStatistics);
        }, this));
    },

    _setupView: function() {
        this.serverStatsView.show(this);
        this.domainsView.show(this);
        this._refreshStatistics();
        this._update();
    }
});

/** @class */
jscape.ServerStatisticsView = Class.extend(
    /** @lends {jscape.ServerStatisticsView.prototype} */
{
    parameters: null,
    statistics: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.data = $("#server-statistics");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.ServerStatisticsView.LISTENER;
    },

    setStatistics: function(serverParameters, serverStatistics) {
        this.parameters = serverParameters;
        this.statistics = serverStatistics;
        this._refresh();
    },

    _init: function() {
        this._initComponents();
    },

    _initComponents: function() {
        this.data.propertygrid({
            showGroup: false,
            loadFilter: $.proxy(this._loadFilter, this)
        });
    },

    _refresh: function() {
        this.data.propertygrid("loadData", this);
    },

    _loadFilter: function() {
        var params = this.parameters;
        var stats = this.statistics;
        var licenseType, licenseOwner, licenseHost, licenseExpires = "";
        if (params && params.license) {
            licenseType = params.license.type;
            licenseOwner = params.license.owner;
            licenseHost = this._formatBindingHost(params.license.host);
            licenseExpires = this._formatExpirationDate(params.license.expirationDate);
        }
        var domains = stats ? stats.domainStatistics.length : "";
        var uptime = stats ? this._formatUptime(stats.uptime) : "";
        var connections = stats ? this._countCurrentConnections(stats.domainStatistics) : "";
        var memory = stats ? this._formatMemoryUsage(stats.usedMemory, stats.maxMemory) : "";
        var threads = stats ? stats.threads : "";
        var rows = [
            {name: this.resources.getString("SERVER_VERSION"), value: params && params.appVersion},
            {name: this.resources.getString("SERVER_LICENSE_TYPE"), value: licenseType},
            {name: this.resources.getString("SERVER_LICENSE_OWNER"), value: licenseOwner},
            {name: this.resources.getString("SERVER_LICENSE_HOST"), value: licenseHost},
            {name: this.resources.getString("SERVER_LICENSE_EXPIRES"), value: licenseExpires},
            {name: this.resources.getString("SERVER_DOMAINS"), value: domains},
            {name: this.resources.getString("SERVER_UPTIME"), value: uptime},
            {name: this.resources.getString("SERVER_MANAGEMENT_CONNECTIONS"), value: stats && stats.managementConnections},
            {name: this.resources.getString("SERVER_CONNECTIONS"), value: connections},
            {name: this.resources.getString("SERVER_MEMORY"), value: memory},
            {name: this.resources.getString("SERVER_THREADS"), value: threads}
        ];
        return {total: rows.length, rows: rows};
    },

    _countCurrentConnections: function(domainStatistics) {
        var connections = 0;
        for (var i = 0; i < domainStatistics.length; ++i) {
            connections += domainStatistics[i].currentSessions;
        }
        return connections;
    },

    _formatBindingHost: function(host) {
        return host ? host : this.resources.getString("SERVER_LICENSE_HOST_NON_BINDING");
    },

    _formatExpirationDate: function(date) {
        return date ? this._formatDate(date) : this.resources.getString("SERVER_LICENSE_EXPIRES_NON_EXPIRING");
    },

    _formatMemoryUsage: function(usedMemory, maxMemory) {
        return this._formatSize(usedMemory) + " of " + this._formatSize(maxMemory);
    },

    _formatUptime: function(value) {
        var uptime = new jscape.Hours(value);
        var pattern = [];
        if (uptime.hours) {
            pattern.push("{hours} hrs.");
        }
        if (uptime.minutes) {
            pattern.push("{minutes} min.");
        }
        if (uptime.seconds) {
            pattern.push("{seconds} sec.");
        }
        return pattern.join(" ").supplant(uptime);
    },

    _formatSize: function(value) {
        return new jscape.SizeFormat().format(value);
    }
});
/** @const */
jscape.ServerStatisticsView.LISTENER = {};


/** @class */
jscape.ServerDomainsView = Class.extend(
    /** @lends {jscape.ServerDomainsView.prototype} */
{
    /** @const */
    DOMAIN_STATE: {running: 0, paused: 1, stopped: 2},

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.domains = $("#server-domains");
        this.startButton = $("#domain-start-btn");
        this.stopButton = $("#domain-stop-btn");
        this.pauseButton = $("#domain-pause-btn");
        this.resumeButton = $("#domain-resume-btn");
        this.addButton = $("#domain-add-btn");
        this.editButton = $("#domain-edit-btn");
        this.deleteButton = $("#domain-delete-btn");
        this.statusButton = $("#domain-status-btn");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || new jscape.ServerDomainsView.LISTENER;
        this._initState();
    },

    showConfirmDeleteDialog: function (domainName) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("DOMAINS_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("DOMAINS_CONFIRM_DELETE_MESSAGE", {0: domainName}));
    },

    addDomain: function(domainStatistics) {
        this.domains.datagrid("appendRow", domainStatistics);
    },

    updateDomain: function(domainStatistics) {
        var index = this._rowIndexFor(domainStatistics.domainName);
        if (index != -1) {
            this.domains.datagrid("deleteRow", index);
            this.domains.datagrid("updateRow", {
                index: index,
                row: domainStatistics
            });
        }
    },

    removeDomain: function(domainName) {
        var index = this._rowIndexFor(domainName);
        if (index != -1) {
            this.domains.datagrid("deleteRow", index);
        }
    },

    setDomains: function(domainStatistics) {
        var rows = [];
        for (var i = 0; i < domainStatistics.length; i++) {
            var entry = domainStatistics[i];
            rows.push({
                "name": entry.domainName,
                "state": entry.isRunning() ? (entry.paused ? this.DOMAIN_STATE.paused : this.DOMAIN_STATE.running) : this.DOMAIN_STATE.stopped,
                "current_sessions": entry.currentSessions,
                "current_transfers": entry.currentTransfers,
                "total_sessions": entry.totalSessions,
                "uploaded_bytes": entry.bytesUploaded,
                "uploaded_files": entry.filesUploaded,
                "downloaded_bytes": entry.bytesDownloaded,
                "downloaded_files": entry.filesDownloaded,
                "quota_uploads": entry.uploadsQuota ? {used: entry.currentUploadedBytes, total: entry.uploadsQuota} : null,
                "quota_downloads": entry.downloadsQuota ? {used: entry.currentDownloadedBytes, total: entry.downloadsQuota} : null,
                "quota_transfers": entry.transfersQuota ? {used: entry.currentUploadedBytes + entry.currentDownloadedBytes, total: entry.transfersQuota} : null,
                "date_start": entry.startDate ? new Date(entry.startDate) : null,
                "date_stop": entry.stopDate ? new Date(entry.stopDate) : null
            });
        }

        this.domains.datagrid("loadData", rows);
        this._adjustButtons();
    },

    _init: function() {
        this._initComponents();
        this._initListeners();
    },

    _initComponents: function() {
        var sizeFormatter = $.proxy(this._formatSize, this);
        var quotaFormatter = $.proxy(this._formatQuota, this);
        var dateFormatter = $.proxy(this._formatDate, this);
        this.domains.datagrid({
            onSelect: $.proxy(this._onSelect$Row, this),
            singleSelect:true,
            fitColumns: true,
            remoteSort: false,
            idField: "name",
            columns: [[
                {field: "name", rowspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_NAME"), sortable: true, width: 175},
                {field: "state", rowspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_STATE"), formatter: $.proxy(this._formatState, this), styler: $.proxy(this._styleState, this), sortable: true, width: 90},
                {field: "current_sessions", rowspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_CURRENT_SESSIONS"), sortable: true, width: 110},
                {field: "current_transfers", rowspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_CURRENT_TRANSFERS"), sortable: true, width: 110},
                {field: "total_sessions", rowspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_TOTAL_SESSIONS"), sortable: true, width: 110},
                {colspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_UPLOADED")},
                {colspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_DOWNLOADED")},
                {colspan: 3, title: this.resources.getString("DOMAINS_TABLE_COLUMN_QUOTA")},
                {field: "date_start", rowspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_DATE_START"), formatter: dateFormatter, sortable: true, width: 150},
                {field: "date_stop", rowspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_DATE_STOP"), formatter: dateFormatter, sortable: true, width: 150}
            ],[
                {field: "uploaded_bytes", rowspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_UPLOADED_BYTES"), formatter: sizeFormatter, sortable: true, width: 110, align: "center"},
                {field: "uploaded_files", rowspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_UPLOADED_FILES"), sortable: true, width: 100, align: "center"},
                {field: "downloaded_bytes", rowspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_DOWNLOADED_BYTES"), formatter: sizeFormatter, sortable: true, width: 110, align: "center"},
                {field: "downloaded_files", rowspan: 2, title: this.resources.getString("DOMAINS_TABLE_COLUMN_DOWNLOADED_FILES"), sortable: true, width: 100, align: "center"},
                {field: "quota_uploads", title: this.resources.getString("DOMAINS_TABLE_COLUMN_QUOTA_UPLOADS"), formatter: quotaFormatter, sortable: true, width: 140, align: "center"},
                {field: "quota_downloads", title: this.resources.getString("DOMAINS_TABLE_COLUMN_QUOTA_DOWNLOADS"), formatter: quotaFormatter, sortable: true, width: 150, align: "center"},
                {field: "quota_transfers", title: this.resources.getString("DOMAINS_TABLE_COLUMN_QUOTA_TRANSFERS"), formatter: quotaFormatter, sortable: true, width: 150, align: "center"}
            ]]
        });

        this.startButton.linkbutton({disabled: true});
        this.stopButton.linkbutton({disabled: true});
        this.pauseButton.linkbutton({disabled: true});
        this.resumeButton.linkbutton({disabled: true});
        this.addButton.linkbutton({iconCls: "icon-add"});
        this.editButton.linkbutton({disabled: true, iconCls: "icon-edit"});
        this.statusButton.linkbutton({disabled: true});
        this.deleteButton.linkbutton({disabled: true, iconCls: "icon-remove"});
    },

    _initListeners: function() {
        var handler = $.proxy(this._handleActionPerformed, this);
        this.startButton.click($.proxy(this._onClick$Start, this));
        this.stopButton.click($.proxy(this._onClick$Stop, this));
        this.pauseButton.click($.proxy(this._onClick$Pause, this));
        this.resumeButton.click($.proxy(this._onClick$Resume, this));
        this.addButton.click($.proxy(this._onClick$Add, this));
        this.editButton.click($.proxy(this._onClick$Edit, this));
        this.statusButton.click($.proxy(this._onClick$Status, this));
        this.deleteButton.click($.proxy(this._onClick$Delete, this));
    },

    _rowIndexFor: function(domainName) {
        var data = this.domains.datagrid("getData");
        for (var i = 0; i < data.length; i++) {
            if (data[i].name = domainName) {
                return i;
            }
        }
        return -1;
    },

    _formatState: function(value) {
        switch (value) {
            case this.DOMAIN_STATE.running:
                return this.resources.getString("DOMAIN_STATE_RUNNING");
            case this.DOMAIN_STATE.paused:
                return this.resources.getString("DOMAIN_STATE_PAUSED");
            case this.DOMAIN_STATE.stopped:
                return this.resources.getString("DOMAIN_STATE_STOPPED");
            default:
                return "";
        }
    },

    _styleState: function(value) {
        switch (value) {
            case this.DOMAIN_STATE.running:
                return "color:green;font-weight:normal";
            case this.DOMAIN_STATE.paused:
                return "color:lightgreen;font-weight:bold";
            case this.DOMAIN_STATE.stopped:
                return "color:red;font-weight:normal";
            default:
                return "color:black;font-weight:normal";
        }
    },

    _formatDate: function(value) {
        return value ? new jscape.DateFormat().format(value) : "";
    },

    _formatQuota: function(quota) {
        return quota != null
               ? this.resources.getString("DOMAINS_TABLE_QUOTA_PATTERN", {0: this._formatSize(quota.used), 1: this._formatSize(quota.total)})
                : "";
    },

    _formatSize: function(value) {
        return new jscape.SizeFormat().format(value);
    },

    getSelectedDomain: function() {
        var domain = this.domains.datagrid("getSelected");
        return domain != null ? domain.name : null;
    },

    _onSelect$Row: function() {
        this._adjustButtons();
    },

    _onClick$Start: function() {
        if (this._buttonEnabled(this.startButton)) {
            this.listener.onStartDomain(this.getSelectedDomain());
        }
        return false;
    },

    _onClick$Stop: function() {
        if (this._buttonEnabled(this.stopButton)) {
            this.listener.onStopDomain(this.getSelectedDomain());
        }
        return false;
    },

    _onClick$Pause: function() {
        if (this._buttonEnabled(this.pauseButton)) {
            this.listener.onPauseDomain(this.getSelectedDomain());
        }
        return false;
    },

    _onClick$Resume: function() {
        if (this._buttonEnabled(this.resumeButton)) {
            this.listener.onResumeDomain(this.getSelectedDomain());
        }
        return false;
    },

    _onClick$Add: function() {
        this.listener.onAddDomain();
        return false;
    },

    _onClick$Edit: function() {
        if (this._buttonEnabled(this.editButton)) {
            this.listener.onResumeDomain(this.getSelectedDomain());
        }
        return false;
    },

    _onClick$Status: function() {
        if (this._buttonEnabled(this.statusButton)) {
            this.listener.onStatusDomain(this.getSelectedDomain());
        }
        return false;
    },

    _onClick$Delete: function() {
        if (this._buttonEnabled(this.deleteButton)) {
            this.listener.onDeleteDomain(this.getSelectedDomain());
        }
        return false;
    },

    _initState: function() {
        this._adjustButtons();
    },

    _adjustButtons: function() {
        var selections = $("#server-domains").datagrid("getSelections");
        var row = selections.length == 1 ? selections[0] : null;

        this._setButtonEnabled(this.startButton, row && row.state == this.DOMAIN_STATE.stopped);
        this._setButtonEnabled(this.stopButton, row && row.state != this.DOMAIN_STATE.stopped);
        this._setButtonEnabled(this.pauseButton, row && row.state == this.DOMAIN_STATE.running);
        this._setButtonEnabled(this.resumeButton, row && row.state == this.DOMAIN_STATE.paused);
        this._setButtonEnabled(this.editButton, false/*todo: row != null*/);
        this._setButtonEnabled(this.deleteButton, selections.length != 0);
        this._setButtonEnabled(this.statusButton, false/*todo: row != null*/);
    },

    _buttonEnabled: function(button) {
        return !button.linkbutton("options").disabled;
    },

    _setButtonEnabled: function(button, enabled) {
        button.linkbutton(enabled ? "enable" : "disable");
    }
});
/** @const */
jscape.ServerDomainsView.LISTENER = {
    onStartDomain: $.noop(),
    onStopDomain: $.noop(),
    onPauseDomain: $.noop(),
    onResumeDomain: $.noop(),
    onAddDomain: $.noop(),
    onEditDomain: $.noop(),
    onDeleteDomain: $.noop(),
    onStatusDomain: $.noop()
};

/** @class */
jscape.ServerInterfacesController = jscape.ApplicationModule.extend(
    /** @lends {jscape.ServerInterfacesController.prototype} */
{
    interfaces: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.ServerInterfacesView(resources);
    },

    start: function(listener, managementService, serverParameters) {
        this.interfaces = serverParameters.interfaces;

        this._setupView();

        this._super(listener, managementService);
    },

    _setupView: function() {
        this.view.show(this);
        this.view.setInterfaces(this.interfaces);
    }
});
/** @class */
jscape.ServerInterfacesView = Class.extend(
    /** @lends {jscape.ServerInterfacesView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.table = $("#server-interfaces");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.ServerInterfacesView.LISTENER;
    },

    setInterfaces: function(interfaces) {
        var rows = [];
        for (var i = 0; i < interfaces.length; i++) {
            var entry = interfaces[i];
            rows.push({
                "name": entry.name,
                "display_name": entry.displayName,
                "addresses": entry.addresses.join("")
            });
        }
        this.table.datagrid("loadData", rows);
    },

    _init: function() {
        this._initComponents();
    },

    _initComponents: function() {
        this.table.datagrid({
            onClickRow: $.proxy(this._onClick$Row, this),
            singleSelect:true,
            fitColumns: true,
            remoteSort: false,
            columns: [[
                {field: "name", title: this.resources.getString("INTERFACES_TABLE_COLUMN_NAME"), sortable: true, width: 20},
                {field: "display_name", title: this.resources.getString("INTERFACES_TABLE_COLUMN_DISPLAY_NAME"), sortable: true, width: 40},
                {field: "addresses", title: this.resources.getString("INTERFACES_TABLE_COLUMN_ADDRESSES"), sortable: true, width: 40}
            ]]
        });
    }
});
/** @const */
jscape.ServerInterfacesView.LISTENER = {};
