var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.DomainsAutoStart = Class.extend(
    /** @lends {jscape.DomainsAutoStart.prototype} */
{
    /** @constructor */
    initialize: function(autostart) {
        this.entries = autostart || [];
    },

    addAutoStart: function(domainName, autoStart) {
        this.entries.push({domain: String(domainName), startup: !!autoStart});
    },

    toJSON: function() {
        var domains = {};
        for (var i = 0; i < this.entries.length; i++) {
            var entry = this.entries[i];
            domains[entry.domain] = entry.startup;
        }
        return {
            domains: $.extend({"@class": "java.util.HashMap"}, domains)
        }
    }
});
jscape.DomainsAutoStart.fromJSON = function(data) {
    var domains = data.domains;
    var obj = new jscape.DomainsAutoStart();
    for (var key in domains) {
        if (domains.hasOwnProperty(key)) {
            if ("@class" != key) {
                obj.addAutoStart(key, domains[key]);
            }
        }
    }
    return obj;
};

/** @class */
jscape.settings.DomainsAutoStartController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.DomainsAutoStartController.prototype} */
{
    view: null,
    domains: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.DomainsAutoStartView(resources);
    },

    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setDomainsAutoStart(config);
        this.listener.onChangesSubmitted(this);
    },

    _initConfiguration: function() {
        this.domains = this.serverSettings.getDomainsAutoStart().entries;
    },

    _createConfiguration: function() {
        return new jscape.DomainsAutoStart(this.view.getAutoStart());
    },

    _setupView: function() {
        this.view.setAutoStart(this.domains);
        this.view.show(this);
    }
});

/** @class */
jscape.settings.DomainsAutoStartView = Class.extend(
    /** @lends {jscape.settings.DomainsAutoStartView.prototype} */
{
    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.table = $("#ssdas-data");
        this.setAutoStartButton = $("#ssdas-set");
        this.clearAutoStartButton = $("#ssdas-clear");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.DomainsAutoStartView.LISTENER;
        this._initState();
    },

    getAutoStart: function() {
        this.table.datagrid("acceptChanges");
        return this.table.datagrid("getRows");
    },

    setAutoStart: function(values) {
        this.table.datagrid("loading");
        this.table.datagrid("loadData", values);
        this.table.datagrid("loaded");
    },

    _init: function() {
        this._initComponents();
        this._initListeners();
    },

    _initComponents: function() {
        this.table.datagrid({
            width: 800,
            remoteSort: false,
            singleSelect: true,
            fitColumns: true,
            onSelect: $.proxy(this._onSelect$Row, this),
            onClickRow: $.proxy(this._onClick$Row, this),
            onSortColumn: $.proxy(this._onSort$Column, this),
            columns: [[
                {field: "domain", title: this.resources.getString("TABLE_COLUMN_DOMAINS_AUTOSTART_DOMAIN"), sortable: true, width: 80},
                {field: "startup", title: this.resources.getString("TABLE_COLUMN_DOMAINS_AUTOSTART_STARTUP"),
                    sortable: true, width: 20,
                    formatter: function(value) {return "<input type=\"checkbox\"{0}/>".supplant({0: value ? " checked" : ""}); },
                    editor: {type: "checkbox", options: {on: true, off: false}}, align: "center"}
            ]],
            editors: {
                checkbox: $.extend({}, $.fn.datagrid.defaults.editors.checkbox, {
                    getValue: function(target) {
                        var value = $(target).is(":checked") ? $(target).val() : $(target).attr("offval");
                        return (/^true$/i).test(value);
                    },
                    setValue:function(target, value) {
                        $(target)._propAttr("checked", (/^true$/i).test(value));
                    }
                })
            }
        });
        this.setAutoStartButton.linkbutton();
        this.clearAutoStartButton.linkbutton();
    },

    _initListeners: function() {
        this.setAutoStartButton.click($.proxy(this._onClick$SetAutoStart, this));
        this.clearAutoStartButton.click($.proxy(this._onClick$ClearAutoStart, this));
    },

    _initState: function() {
        this._adjustButtonState();
    },

    _onSelect$Row: function() {
        this._adjustButtonState();
    },

    _onSort$Column: function() {
        this._adjustButtonState();
    },

    _onClick$SetAutoStart: function() {
        this._setAutoStart(true);
        this._endEdit();
        return false;
    },

    _onClick$ClearAutoStart: function() {
        this._setAutoStart(false);
        this._endEdit();
        return false;
    },

    _onClick$Row: function(index) {
        this._beginEdit(index);
    },

    _beginEdit: function(index) {
        this._endEdit();
        this.table.datagrid("beginEdit", index);
        this.editIndex = index;
    },

    _endEdit: function() {
        if (this.editIndex != null) {
            var index = this.editIndex;
            this.editIndex = null;
            if (this.table.datagrid("validateRow", index)) {
                this.table.datagrid("endEdit", index);
            } else {
                this.table.datagrid("cancelEdit", index);
            }
        }
    },

    _setAutoStart: function(value) {
        var rows = this.table.datagrid("getSelections");

        while (rows.length != 0) {
            var row = rows.pop();
            var index = this.table.datagrid("getRowIndex", row);
            row.startup = value;

            this.table.datagrid("refreshRow", index);
            this.table.datagrid("selectRow", index);
        }
    },

    _adjustButtonState: function() {
        var enable = this.table.datagrid("getSelections").length != 0;
        this.setAutoStartButton.linkbutton(enable ? "enable" : "disable");
        this.clearAutoStartButton.linkbutton(enable ? "enable" : "disable");
    }
});
/** @const */
jscape.settings.DomainsAutoStartView.LISTENER = {};