var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.EmailServiceDescriptor = Class.extend(
    /** @lends {jscape.EmailServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(config, resources) {
        this.config = config;
        this.resources = resources || {};
    },

    toJSON: function() {
        return {
            configuration: this.config.toJSON(),
            properties: this.resources
        }
    }
});
jscape.EmailServiceDescriptor.fromJSON = function(data) {
    return new jscape.EmailServiceDescriptor(jscape.EmailServiceConfiguration.fromJSON(data.configuration),
                                             data.properties);
};
/** @class */
jscape.EmailServiceConfiguration = Class.extend(
    /** @lends {jscape.EmailServiceConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(enabled,
                         host,
                         port,
                         connectionType,
                         username,
                         password,
                         fromAddress,
                         replyToAddress,
                         debugFile,
                         encryptKey,
                         signKey,
                         enableAdHoc,
                         enableProtection,
                         protectionType,
                         linkExpiresRange,
                         maxDownloadsDefault,
                         allowPublicContactRecipient,
                         allowTopLevelDomainRecipient,
                         topLevelDomains) {
        this.enabled = enabled || false;
        this.host = host;
        this.port = port;
        this.connectionType = connectionType;
        this.username = username;
        this.password = password;
        this.fromAddress = fromAddress;
        this.replyToAddress = replyToAddress;
        this.debugFile = debugFile;
        this.encryptKey = encryptKey;
        this.signKey = signKey;
        this.enableAdHoc = enableAdHoc || false;
        this.enableProtection = enableProtection || false;
        this.protectionType = protectionType;
        this.linkExpiresRange = linkExpiresRange || {};
        this.maxDownloadsDefault = maxDownloadsDefault;
        this.publicContactRecipientsAllowed = allowPublicContactRecipient || false;
        this.topLevelDomainRecipientsAllowed = allowTopLevelDomainRecipient || false;
        this.topLevelDomains = topLevelDomains || [];
    },

    toJSON: function() {
        return {
            enabled: this.enabled,
            host: this.host,
            port: this.port,
            connectionType: this.connectionType,
            username: this.username,
            password: this.password,
            encryptionKeyAlias: this.encryptKey,
            signingKeyAlias: this.signKey,
            sender: this.fromAddress,
            replyTo: this.replyToAddress,
            debugFile: this.debugFile,
            enableAdHocTransfer: this.enableAdHoc,
            enablePasswordProtection: this.enableProtection,
            passwordProtectionType: this.protectionType,
            linkExpiresRange: this.linkExpiresRange,
            maxDownloadsDefault: this.maxDownloadsDefault,
            publicContactRecipientsAllowed: this.publicContactRecipientsAllowed,
            topLevelDomainRecipientsAllowed: this.topLevelDomainRecipientsAllowed,
            allowedTopLevelDomains: ["java.util.LinkedList", this.topLevelDomains]
        }
    }
});
jscape.EmailServiceConfiguration.fromJSON = function(data) {
    return new jscape.EmailServiceConfiguration(data.enabled,
                                                data.host,
                                                data.port,
                                                data.connectionType,
                                                data.username,
                                                data.password,
                                                data.sender,
                                                data.replyTo,
                                                data.debugFile,
                                                data.encryptionKeyAlias,
                                                data.signingKeyAlias,
                                                data.enableAdHocTransfer,
                                                data.enablePasswordProtection,
                                                data.passwordProtectionType,
                                                data.linkExpiresRange,
                                                data.maxDownloadsDefault,
                                                data.publicContactRecipientsAllowed,
                                                data.topLevelDomainRecipientsAllowed,
                                                data.allowedTopLevelDomains && data.allowedTopLevelDomains[1]);
};

/** @class */
jscape.LinkExpiresRange = Class.extend(
    /** @lends {jscape.LinkExpiresRange.prototype} */
{
    /** @constructor */
    initialize: function(lowerBound, upperBound, step) {
        this.lowerBound = lowerBound;
        this.upperBound = upperBound;
        this.step = step;
    }
});

/** @class */
jscape.settings.EmailController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.EmailController.prototype} */
{
    /** @const */
    VARIABLES: [
        "username", "URL", "date", "message", "password",
        "transferEntries", "entry.filename", "entry.filesize", "entry.downloadUrl", "entry.expirationDate", "entry.maxDownloads"
    ],

    descriptor: null,
    domains: null,
    variables: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.EmailView(resources);
        this.miscView = new jscape.settings.EmailMiscView(resources);
        this.adHocView = new jscape.settings.AdHocTransferView(resources);
        this.resourcesView = new jscape.settings.EmailResourcesView(resources);
        this.variables = this.VARIABLES;

        this._initValidators(resources);
    },

    submitChanges: function() {
        var descriptor = this._createDescriptor();
        this.serverSettings.setEmailServiceDescriptor(descriptor);
        this.listener.onChangesSubmitted(this);
    },

    onTestServer: function() {
        this.view.showTestServerDialog({
            onSubmit: $.proxy(this._onSubmit$TestServerDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onAddDomain: function() {
        this.adHocView.showAddDomainDialog({
            onSubmit: $.proxy(this._onSubmit$AddDomainDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        })
    },

    onEditDomain: function(domain) {
        var index = $.inArray(domain, this.domains);
        this.adHocView.showEditDomainDialog({
            onSubmit: $.proxy(this._onSubmit$EditDomainDialog, this, index),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, domain);
    },

    onDeleteDomain: function(domain) {
        this.adHocView.showConfirmDelete(domain).done($.proxy(this._onConfirm$DeleteDomain, this, domain));
    },

    onAddVariable: function() {
        this.resourcesView.showAddVariableDialog(this.variables, {
            onSubmit: $.proxy(this._onSubmit$AddVariableDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    _onSubmit$TestServerDialog: function(dialog) {
        var recipient = dialog.getRecipient();
        dialog.hide();

        var descriptor = this._createDescriptor();
        this.managementService.testEmailServer(descriptor, recipient)
                .done($.proxy(this._onSuccess$TestEmailService, this))
                .fail($.proxy(this._onError$TestEmailService, this));
    },

    _onSuccess$TestEmailService: function() {
        this.view.showTestSuccessMessage();
    },

    _onError$TestEmailService: function() {
        // ok
    },

    _onSubmit$AddDomainDialog: function(dialog) {
        var domain = dialog.getDomain();
        dialog.hide();

        this.domains.push(domain);
        this.adHocView.setTopLevelDomains(this.domains, $.inArray(domain, this.domains));
    },

    _onSubmit$EditDomainDialog: function(index, dialog) {
        var domain = dialog.getDomain();
        dialog.hide();

        this.domains[index] = domain;
        this.adHocView.setTopLevelDomains(this.domains, index);
    },

    _onConfirm$DeleteDomain: function(domain) {
        var index = $.inArray(domain, this.domains);
        if (index != -1) {
            this.domains.splice(index, 1);
            this.adHocView.setTopLevelDomains(this.domains, index);
        }
    },

    _onSubmit$AddVariableDialog: function(dialog) {
        var variable = dialog.getVariable();
        dialog.hide();

        this.resourcesView.insertVariable(this._decorateVariable(variable));
    },

    _onCancel$Dialog: function(dialog) {
        dialog.hide();
    },

    _decorateVariable: function(name) {
        return "${" + name + "}";
    },

    _initConfiguration: function() {
        this.descriptor = this.serverSettings.getEmailServiceDescriptor();
        this.domains = [].concat(this.descriptor.config.topLevelDomains);
    },

    _createDescriptor: function() {
        return new jscape.EmailServiceDescriptor(this._createConfiguration(),
                                                 this.resourcesView.getResources());
    },

    _createConfiguration: function() {
        return new jscape.EmailServiceConfiguration(this.view.getEnabled(),
                                                    this.view.getHost(),
                                                    this.view.getPort(),
                                                    this.view.getConnectionType(),
                                                    this.view.getUsername(),
                                                    this.view.getPassword(),
                                                    this.view.getFromAddress(),
                                                    this.view.getReplyToAddress(),
                                                    this.view.getDebugFile(),
                                                    this.miscView.getEncryptionKey(),
                                                    this.miscView.getSigningKey(),
                                                    this.adHocView.getEnabled(),
                                                    this.adHocView.getProtectionEnabled(),
                                                    this.adHocView.getProtectionType(),
                                                    new jscape.LinkExpiresRange(this.adHocView.getLinkExpiresRangeBegin(),
                                                                                this.adHocView.getLinkExpiresRangeEnd(),
                                                                                this.adHocView.getLinkExpiresRangeStep()),
                                                    this.adHocView.getMaxDownloadsDefault(),
                                                    this.adHocView.getRecipientsPublicContactAllowed(),
                                                    this.adHocView.getRecipientsTopLevelDomainAllowed(),
                                                    this.adHocView.getTopLevelDomains()
        );
    },

    _setupView: function() {
        var config = this.descriptor.config;

        this.view.setHost(config.host);
        this.view.setPort(config.port);
        this.view.setConnectionType(config.connectionType);
        this.view.setUsername(config.username);
        this.view.setPassword(config.password);
        this.view.setFromAddress(config.fromAddress);
        this.view.setReplyToAddress(config.replyToAddress);
        this.view.setDebugFile(config.debugFile);
        this.view.setEnabled(config.enabled);
        this.view.show(this);

        this.miscView.setEncryptionKeys(this._getEncryptionKeys());
        this.miscView.setEncryptionKey(config.encryptKey);
        this.miscView.setSigningKeys(this._getSigningKeys());
        this.miscView.setSigningKey(config.signKey);
        this.miscView.show(this);

        this.adHocView.setEnabled(config.enableAdHoc);
        this.adHocView.setProtectionEnabled(config.enableProtection);
        this.adHocView.setProtectionType(config.protectionType);
        this.adHocView.setLinkExpiresRangeBegin(config.linkExpiresRange.lowerBound);
        this.adHocView.setLinkExpiresRangeEnd(config.linkExpiresRange.upperBound);
        this.adHocView.setLinkExpiresRangeStep(config.linkExpiresRange.step);
        this.adHocView.setMaxDownloadsDefault(config.maxDownloadsDefault);
        this.adHocView.setRecipientsPublicContactAllowed(config.publicContactRecipientsAllowed);
        this.adHocView.setRecipientsTopLevelDomainAllowed(config.topLevelDomainRecipientsAllowed);
        this.adHocView.setTopLevelDomains(this.domains);
        this.adHocView.show(this);

        this.resourcesView.setResources(this.descriptor.resources);
        this.resourcesView.show(this);
    },

    _getEncryptionKeys: function() {
        var keys = this.serverSettings.getKeys();
        return this._getKeysAliases(keys.getPgpPublicKeys().concat(keys.getPgpSecretKeys()));
    },

    _getSigningKeys: function() {
        return this._getKeysAliases(this.serverSettings.getKeys().getPgpSecretKeys());
    },

    _getKeysAliases: function(keys) {
        var aliases = [];
        for (var i = 0; i < keys.length; i++) {
            aliases.push(keys[i].alias)
        }
        return aliases;
    },

    _initValidators: function(resources) {
        $.extend($.fn.validatebox.defaults.rules, {
            'smtp_host_nonempty': {
                validator: function(value) { return value && value.trim().length != 0; },
                message: resources.getString("EMAIL_SERVICE_ERROR_BAD_HOST")
            },
            'adhoc_tld_nonempty': {
                validator: function(value) { return value && value.trim().length != 0; },
                message: resources.getString("ADHOC_TRANSFER_ERROR_BAD_DOMAIN")
            },
            'adhoc_tld_notexists': {
                validator: $.proxy(this._validatorDomainExists, this),
                message: resources.getString("ADHOC_TRANSFER_ERROR_DOMAIN_EXISTS")
            }
        });
    },

    _validatorDomainExists: function(value, excludes) {
        return $.inArray(value, excludes) != -1 || $.inArray(value, this.domains) == -1;
    }
});

/** @class */
jscape.settings.TestServerDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.settings.TestServerDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#sses-dialog-test"), resources);
        this.recipient = $("#sses-test-to");

        this._initComponents();
    },

    getRecipient: function() {
        return this.recipient.val();
    },

    /** @override */
    _valid: function() {
        return this.recipient.validatebox("isValid");
    },

    _initComponents: function() {
        this.recipient.validatebox({
            required: true,
            validType: "email",
            missingMessage: this.resources.getString("EMAIL_SERVICE_ERROR_BAD_RECIPIENT"),
            invalidMessage: this.resources.getString("EMAIL_SERVICE_ERROR_BAD_RECIPIENT")});
    }
});

/** @class */
jscape.settings.EmailView = Class.extend(
    /** @lends {jscape.settings.EmailView.prototype} */
{
    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset#sses-fields");
        this.enabled = $("#sses-enable");
        this.host = $("#sses-host");
        this.port = $("#sses-port");
        this.connectionType = $("#sses-conntype");
        this.username = $("#sses-username");
        this.password = $("#sses-pwd");
        this.debugFile = $("#sses-debugfile");
        this.fromAddress = $("#sses-from");
        this.replyToAddress = $("#sses-replyto");
        this.testButton = $("#sses-test");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.EmailView.LISTENER;
        this._initState();
    },

    showTestSuccessMessage: function() {
        $.messager.alert(this.resources.getString("EMAIL_SERVICE_TEST_SUCCESS_TITLE"),
                         this.resources.getString("EMAIL_SERVICE_TEST_SUCCESS_MESSAGE"),
                         "info");
    },

    showTestServerDialog: function(listener) {
        var dialog = new jscape.settings.TestServerDialog(this.resources);
        dialog.show(listener);
    },

    getEnabled: function() {
        return this.enabled.prop("checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", value);
        this._adjustFieldState();
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value);
    },

    getConnectionType: function() {
        return this.connectionType.combobox("getValue");
    },

    setConnectionType: function(value) {
        this.connectionType.combobox("setValue", value);
    },

    getUsername: function() {
        return this.username.val();
    },

    setUsername: function(value) {
        this.username.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getDebugFile: function() {
        return this.debugFile.val();
    },

    setDebugFile: function(value) {
        this.debugFile.val(value);
    },

    getFromAddress: function() {
        return this.fromAddress.val();
    },

    setFromAddress: function(value) {
        this.fromAddress.val(value);
    },

    getReplyToAddress: function() {
        return this.replyToAddress.val();
    },

    setReplyToAddress: function(value) {
        this.replyToAddress.val(value);
    },

    _init: function() {
        this.enabled.click($.proxy(this._onClick$Enabled, this));

        this.host.validatebox({
            required: true,
            validType: "smtp_host_nonempty",
            missingMessage: this.resources.getString("EMAIL_SERVICE_ERROR_BAD_HOST")
        });

        this.port.numberspinner({min: 1, max: 65535, required: true});
        this.connectionType.combobox({required: true, editable: false, multiple: false});
        this.fromAddress.validatebox({required: true, validType: "email", missingMessage: this.resources.getString("EMAIL_SERVICE_ERROR_BAD_EMAIL")});
        this.replyToAddress.validatebox({required: false, validType: "email", invalidMessage: this.resources.getString("EMAIL_SERVICE_ERROR_BAD_EMAIL")});

        this.testButton.linkbutton();
        this.testButton.click($.proxy(this._onClick$Test, this));
    },

    _initState: function() {
        this._adjustFieldState();
    },

    _valid: function() {
        return this.host.validatebox("isValid")
               && this.port.numberspinner("isValid")
               && this.connectionType.combobox("isValid")
               && this.fromAddress.validatebox("isValid")
               && this.replyToAddress.validatebox("isValid")
    },

    _onClick$Enabled: function() {
        this._adjustFieldState();
    },

    _onClick$Test: function() {
        if (this._buttonEnabled(this.testButton) && this._valid()) {
            this.listener.onTestServer();
        }
        return false;
    },

    _adjustFieldState: function() {
        var enabled = this.getEnabled();
        this.host.prop("disabled", !enabled);
        this.host.validatebox(enabled ? "enableValidation" : "disableValidation");
        this.port.numberspinner(enabled ? "enable" : "disable");
        this.port.numberspinner(enabled ? "enableValidation" : "disableValidation");
        this.connectionType.combobox(enabled ? "enable" : "disable");
        this.connectionType.combobox(enabled ? "enableValidation" : "disableValidation");
        this.username.prop("disabled", !enabled);
        this.password.prop("disabled", !enabled);
        this.debugFile.prop("disabled", !enabled);
        this.fromAddress.prop("disabled", !enabled);
        this.fromAddress.validatebox(enabled ? "enableValidation" : "disableValidation");
        this.replyToAddress.prop("disabled", !enabled);
        this.replyToAddress.validatebox(enabled ? "enableValidation" : "disableValidation");
        this.testButton.linkbutton(enabled ? "enable" : "disable");

        this.fieldset.form("validate");
    },

    _buttonEnabled: function(button) {
        return !button.linkbutton("options").disabled;
    }
});
/** @const */
jscape.settings.EmailView.LISTENER = {
    onTestServer: $.noop()
};

/** @class */
jscape.settings.EmailMiscView = Class.extend(
    /** @lends {jscape.settings.EmailMiscView.prototype} */
{
    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.encrypt = $("#ssesmisc-encrypt");
        this.encryptKey = $("#ssesmisc-encryptkey");
        this.sign = $("#ssesmisc-sign");
        this.signKey = $("#ssesmisc-signkey");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.EmailMiscView.LISTENER;
        this._initState();
    },

    getEncryptionKey: function() {
        return this.encrypt.prop("checked") ? this.encryptKey.combobox("getValue") : "";
    },

    setEncryptionKey: function(value) {
        if (value) {
            this.encrypt.prop("checked", true);
            this.encryptKey.combobox("setValue" , value);
            this.encryptKey.combobox("select", value);
        } else {
            this.encrypt.prop("checked", false);
        }
    },

    setEncryptionKeys: function(values) {
        this._setComboboxValues(this.encryptKey, values);
        if (values.length != 0) {
            this.encryptKey.combobox("select", values[0]);
        }
    },

    getSigningKey: function() {
        return this.sign.prop("checked") ? this.signKey.combobox("getValue") : "";
    },

    setSigningKey: function(value) {
        if (value) {
            this.sign.prop("checked", true);
            this.signKey.combobox("setValue", value);
            this.signKey.combobox("select", value);
        } else {
            this.sign.prop("checked", false);
        }
    },

    setSigningKeys: function(values) {
        this._setComboboxValues(this.signKey, values);
        if (values.length != 0) {
            this.signKey.combobox("select", values[0]);
        }
    },

    _init: function() {
        this.encrypt.click($.proxy(this._onClick$Encrypt, this));
        this.encryptKey.combobox({editable: false, disabled: true});

        this.sign.click($.proxy(this._onClick$Sign, this));
        this.signKey.combobox({editable: false, disabled: true});
    },

    _initState: function() {
        this._adjustEncryptKeyFieldState();
        this._adjustSignKeyFieldState();
    },

    _onClick$Encrypt: function() {
        this._adjustEncryptKeyFieldState();
    },

    _onClick$Sign: function() {
        this._adjustSignKeyFieldState();
    },

    _adjustEncryptKeyFieldState: function() {
        this.encryptKey.combobox(this.encrypt.prop("checked") ? "enable" : "disable");
    },

    _adjustSignKeyFieldState: function() {
        this.signKey.combobox(this.sign.prop("checked") ? "enable" : "disable");
    },

    _setComboboxValues: function(box, values) {
        var data = [];
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            data.push({text: value, value: value});
        }
        box.combobox("loadData", data);
    }
});

/** @class */
jscape.settings.TopLevelDomainDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.settings.TopLevelDomainDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#sseft-dialog-tld"), resources);
        this.domain = $("#sseft-tld-domain");

        this._initComponents();
    },

    getDomain: function() {
        return this.domain.val();
    },

    setDomain: function(value) {
        this.domain.val(value || "");
        this._initDomainValidator(value);
    },

    /** @override */
    _valid: function() {
        return this.domain.validatebox("isValid");
    },

    _initComponents: function() {
        this._initDomainValidator();
    },

    _initDomainValidator: function(domain) {
        this.domain.validatebox({
            required: true,
            validType: ["adhoc_tld_nonempty", "adhoc_tld_notexists" + (domain ? "['" + domain + "']" : "")],
            missingMessage: this.resources.getString("ADHOC_TRANSFER_ERROR_BAD_DOMAIN")
        });
    }
});
/** @class */
jscape.settings.AdHocTransferView = Class.extend(
    /** @lends {jscape.settings.AdHocTransferView.prototype} */
{
    /** @const */
    DEFAULT_EXPIRES_BEGIN: 5,
    /** @const */
    DEFAULT_EXPIRES_END: 90,
    /** @const */
    DEFAULT_EXPIRES_STEP: 5,
    /** @const */
    DEFAULT_MAX_DOWNLOADS: 5,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.enable = $("#sseft-enable");
        this.linkExpiresRangeBegin = $("#sseft-expires-begin");
        this.linkExpiresRangeEnd = $("#sseft-expires-end");
        this.linkExpiresRangeStep = $("#sseft-expires-step");
        this.maxDownloads = $("#sseft-maxdloads");
        this.protect = $("#sseft-protect");
        this.protectType = $("#sseft-protecttype");
        this.recipientsPublicContactAllowed = $("#sseft-allowrecepientpub");
        this.recipientsTopLevelDomainAllowed = $("#sseft-allowrecepienttld");
        this.topLevelDomains = $("#sseft-tld-data");
        this.addButton = $("#sseft-tldadd");
        this.editButton = $("#sseft-tldedit");
        this.deleteButton = $("#sseft-tlddel");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.AdHocTransferView.LISTENER;
        this._initState();
    },

    showAddDomainDialog: function(listener) {
        var dialog = new jscape.settings.TopLevelDomainDialog(this.resources);
        dialog.setTitle(this.resources.getString("ADHOC_TRANSFER_ADD_TITLE"));
        dialog.setDomain(null);
        dialog.show(listener);
    },

    showEditDomainDialog: function(listener, domain) {
        var dialog = new jscape.settings.TopLevelDomainDialog(this.resources);
        dialog.setTitle(this.resources.getString("ADHOC_TRANSFER_EDIT_TITLE"));
        dialog.setDomain(domain);
        dialog.show(listener);
    },

    showConfirmDelete: function(domain) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("ADHOC_TRANSFER_DELETE_TITLE"),
                                            this.resources.getString("ADHOC_TRANSFER_DELETE_MESSAGE", {0: domain}));
    },

    getEnabled: function() {
        return this.enable.prop("checked");
    },

    setEnabled: function(value) {
        this.enable.prop("checked", value);
    },

    getLinkExpiresRangeBegin: function() {
        return this.linkExpiresRangeBegin.numberspinner("getValue");
    },

    setLinkExpiresRangeBegin: function(value) {
        this.linkExpiresRangeBegin.numberspinner("setValue", value || this.DEFAULT_EXPIRES_BEGIN);
    },

    getLinkExpiresRangeEnd: function() {
        return this.linkExpiresRangeEnd.numberspinner("getValue");
    },

    setLinkExpiresRangeEnd: function(value) {
        this.linkExpiresRangeEnd.numberspinner("setValue", value || this.DEFAULT_EXPIRES_END);
    },

    getLinkExpiresRangeStep: function() {
        return this.linkExpiresRangeStep.numberspinner("getValue");
    },

    setLinkExpiresRangeStep: function(value) {
        this.linkExpiresRangeStep.numberspinner("setValue", value || this.DEFAULT_EXPIRES_STEP);
    },

    getMaxDownloadsDefault: function() {
        return this.maxDownloads.numberspinner("getValue");
    },

    setMaxDownloadsDefault: function(value) {
        this.maxDownloads.numberspinner("setValue", value || this.DEFAULT_MAX_DOWNLOADS);
    },

    getProtectionEnabled: function() {
        return this.protect.prop("checked");
    },

    setProtectionEnabled: function(value) {
        this.protect.prop("checked", value);
    },

    getProtectionType: function() {
        return this.protectType.combobox("getValue");
    },

    setProtectionType: function(value) {
        this.protectType.combobox("select", value);
    },

    getRecipientsPublicContactAllowed: function() {
        return this.recipientsPublicContactAllowed.prop("checked");
    },

    setRecipientsPublicContactAllowed: function(value) {
        this.recipientsPublicContactAllowed.prop("checked", value);
    },

    getRecipientsTopLevelDomainAllowed: function() {
        return this.recipientsTopLevelDomainAllowed.prop("checked");
    },

    setRecipientsTopLevelDomainAllowed: function(value) {
        this.recipientsTopLevelDomainAllowed.prop("checked", value);
    },

    getTopLevelDomains: function() {
        var rows = this.topLevelDomains.datagrid("getRows");
        var names = new Array(rows.length);
        for (var i = 0; i < rows.length; i++) {
            names[i] = rows[i].name;
        }
        return names;
    },

    setTopLevelDomains: function(domains, selectIndex) {
        var data = new Array(domains.length);
        for (var i = 0; i < domains.length; i++) {
            data[i] = {name: domains[i]};
        }
        this.topLevelDomains.datagrid("loading");
        this.topLevelDomains.datagrid("loadData", data);
        this.topLevelDomains.datagrid("loaded");

        if (selectIndex != undefined) {
            var index = Math.max(0, Math.min(selectIndex, domains.length - 1));
            this.topLevelDomains.datagrid("selectRow", index);
        }
    },

    _init: function() {
        this.enable.click($.proxy(this._onClick$Enable, this));

        this.topLevelDomains.datagrid({
            singleSelect: true,
            remoteSort: false,
            fitColumns: true,
            columns: [[{
                field: "name",
                title: this.resources.getString("ADHOC_TRANSFER_TABLE_COLUMNN_TLD"),
                resizable: true,
                sortable: true,
                width: 100
            }]],
            onSelect: $.proxy(this._onSelect$Row, this),
            onSortColumn: $.proxy(this._onSort$Column, this)
        });

        var handler = $.proxy(this._onChange$LinkExpiresRange, this);
        this.linkExpiresRangeBegin.numberspinner({min: 1, max: 90, onChange: handler, disabled: true});
        this.linkExpiresRangeEnd.numberspinner({min: 1, max: 90, onChange: handler, disabled: true});
        this.linkExpiresRangeStep.numberspinner({min: 1, max: 90, onChange: handler, disabled: true});
        this.maxDownloads.numberspinner({min: 1, max: 10000, disabled: true});

        this.protect.click($.proxy(this._onClick$Protect, this));
        this.protectType.combobox({editable: false, disabled: true});

        this.addButton.linkbutton();
        this.addButton.click($.proxy(this._onClick$Add, this));

        this.editButton.linkbutton();
        this.editButton.click($.proxy(this._onClick$Edit, this));

        this.deleteButton.linkbutton();
        this.deleteButton.click($.proxy(this._onClick$Delete, this));
    },

    _initState: function() {
        this._adjustFieldState();
        this._adjustButtonState();
        this._adjustLinkExpiresRange();
    },

    _onSelect$Row: function() {
        this._adjustButtonState();
    },

    _onSort$Column: function() {
        this._adjustButtonState();
    },

    _onClick$Enable: function() {
        this._adjustFieldState();
    },

    _onClick$Protect: function() {
        this.protectType.combobox(this.getProtectionEnabled() ? "enable" : "disable");
    },

    _onChange$LinkExpiresRange: function() {
        this._adjustLinkExpiresRange();
    },

    _onClick$Add: function() {
        this.listener.onAddDomain();
        return false;
    },

    _onClick$Edit: function() {
        if (this._buttonEnabled(this.editButton)) {
            var domain = this.topLevelDomains.datagrid("getSelected").name;
            this.listener.onEditDomain(domain);
        }
        return false;
    },

    _onClick$Delete: function() {
        if (this._buttonEnabled(this.deleteButton)) {
            var domain = this.topLevelDomains.datagrid("getSelected").name;
            this.listener.onDeleteDomain(domain);
        }
        return false;
    },

    _adjustFieldState: function() {
        var enabled = this.getEnabled();
        this.linkExpiresRangeBegin.numberspinner(enabled ? "enable" : "disable");
        this.linkExpiresRangeEnd.numberspinner(enabled ? "enable" : "disable");
        this.linkExpiresRangeStep.numberspinner(enabled ? "enable" : "disable");
        this.maxDownloads.numberspinner(enabled ? "enable" : "disable");
        this.protect.prop("disabled", !enabled);
        this.protectType.combobox(enabled && this.protect.prop("checked") ? "enable" : "disable");
    },

    _adjustLinkExpiresRange: function() {
        var begin = this.getLinkExpiresRangeBegin();
        var end = this.getLinkExpiresRangeEnd();
        var step = this.getLinkExpiresRangeStep();

        this._setMaxValue(this.linkExpiresRangeBegin, end);

        if (begin > end) {
            this.setLinkExpiresRangeBegin(end);
            this._setMaxValue(this.linkExpiresRangeStep, 1);
        } else {
            var stepMax = end - begin;
            this._setMaxValue(this.linkExpiresRangeStep, stepMax);
            if (step > stepMax) {
                this.setLinkExpiresRangeStep(stepMax);
            }
        }
    },

    _setMaxValue: function(spinner, value) {
        spinner.numberspinner("options").max = value;
    },

    _adjustButtonState: function() {
        var enabled = this.topLevelDomains.datagrid("getSelected") != null;
        this._setButtonEnabled(this.editButton, enabled);
        this._setButtonEnabled(this.deleteButton, enabled);
    },

    _buttonEnabled: function(button) {
        return !button.linkbutton("options").disabled;
    },

    _setButtonEnabled: function(button, enable) {
        button.linkbutton(enable ? "enable" : "disable");
    }

});
/** @const */
jscape.settings.AdHocTransferView.LISTENER = {
    onAddDomain: $.noop(),
    onEditDomain: $.noop(),
    onDeleteDomain: $.noop()
};

/** @class */
jscape.settings.VariableDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.settings.VariableDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#sseres-dialog-addvariable"), resources);
        this.variables = $("#sseres-variables");

        this._init();
    },

    getVariable: function() {
        return this.variables.datagrid("getSelected").name;
    },

    setVariables: function(variables) {
        var data = [];
        for (var i = 0; i < variables.length; i++) {
            var variable = variables[i];
            data.push({name: variable});
        }
        this.variables.propertygrid("loadData", data);
    },

    _valid: function() {
        return this.variables.datagrid("getSelected") != null;
    },

    _init: function() {
        this.variables.datagrid({
            singleSelect: true,
            showHeader: false,
            remoteSort: false,
            fitColumns: true,
            loadMsg: "",
            columns:[[{
                field: "name", width: 75
            }]],
            onDblClickRow: $.proxy(function() {this._handleSubmit(); }, this)
        });
    }
});

/** @class */
jscape.settings.EmailResourcesView = Class.extend(
    /** @lends {jscape.settings.EmailResourcesView.prototype} */
{
    rowIndex: -1,
    caretPosition: -1,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.table = $("#sseres-data");
        this.addVariableButton = $("#sseres-addvar");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.EmailResourcesView.LISTENER;
        this._initState();
    },

    showAddVariableDialog: function(variables, listener) {
        var dialog = new jscape.settings.VariableDialog(this.resources);
        dialog.setTitle(this.resources.getString("VARIABLE_DIALOG_ADD_TITLE"));
        dialog.setVariables(variables);
        dialog.show(listener);
    },

    getResources: function() {
        var resources = {};
        var rows = this.table.propertygrid("getRows");
        for (var i = 0; i < rows.length; i++) {
            var row = rows[i];
            resources[row.name] = row.value;
        }
        return resources;
    },

    setResources: function(resources) {
        var data = [];
        for (var key in resources) {
            if (resources.hasOwnProperty(key)) {
                data.push({name: key, value: resources[key], editor: "text"});
            }
        }
        this.table.propertygrid("loadData", data);
    },

    insertVariable: function(variable) {
        var index = this.rowIndex;
        var rows = this.table.propertygrid("getRows");

        if (index != -1 && index < rows.length) {
            this.table.propertygrid("selectRow", index);
            this.table.propertygrid("beginEdit", index);
            this.table.propertygrid("options").editIndex = index;
            var editor = this.table.propertygrid("getEditor", {index: index, field: "value"});
            var value = rows[index].value + variable;
            $(editor.target).val(value);
        }
        // todo: edit field carret position
        this.caretPosition = -1;
    },

    _init: function() {
        this.table.propertygrid({
            showGroup: false,
            scrollbarSize: 0,
            sortName: "name",
            sortOrder: "asc",
            onClickRow: $.proxy(this._onClick$Row, this),
            onBeforeEdit: $.proxy(this._onEdit$Before, this),
            onCancelEdit: $.proxy(this._onEdit$Cancel, this)
        });

        this.addVariableButton.linkbutton();
        this.addVariableButton.click($.proxy(this._onClick$AddVariable, this));
    },

    _initState: function() {
        this._adjustButtonsState();
    },

    _onClick$Row: function() {
        this._adjustButtonsState();
    },

    _onEdit$Before: function(index) {
        this.rowIndex = index;
    },

    _onEdit$Cancel: function() {
        this.rowIndex = -1;
    },

    _onClick$AddVariable: function() {
        if (this._buttonEnabled(this.addVariableButton)) {
            this.listener.onAddVariable();
        }
        return false;
    },

    _adjustButtonsState: function() {
        this._setButtonEnabled(this.addVariableButton, this.table.propertygrid("getSelected") != null);
    },

    _buttonEnabled: function(button) {
        return !button.linkbutton("options").disabled;
    },

    _setButtonEnabled: function(button, enabled) {
        button.linkbutton(enabled ? "enable" : "disable");
    }
});
/** @const */
jscape.settings.EmailResourcesView.LISTENER = {
    onAddVariable: $.noop()
};