var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.ManagerConfiguration = Class.extend(
    /** @lends {jscape.ManagerConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(host, port, timeout, username, password) {
        this.host = host;
        this.port = port;
        this.timeout = timeout;
        this.username = username;
        this.password = password;
    },

    toJSON: function() {
        return {
            host: this.host,
            port: this.port,
            connectionTimeout: this.timeout,
            username: this.username,
            password: this.password
        }
    }
});
jscape.ManagerConfiguration.fromJSON = function(data) {
    return new jscape.ManagerConfiguration(data.host,
                                           data.port,
                                           data.connectionTimeout,
                                           data.username,
                                           data.password);
};

/** @class */
jscape.IpSubstitutor = Class.extend(
    /** @lends {jscape.IpSubstitutor.prototype} */
{
    /** @constructor */
    initialize: function(rules) {
        this.rules = rules || [];
    },

    toJSON: function() {
        var entries = [];
        for (var i = 0; i < this.rules.length; i++) {
            entries.push(this.rules[i].toJSON());
        }
        return {
            '@class': "com.jscape.inet.mft.IPRuleSubstitutor",
            rules: ["java.util.LinkedList", entries]
        };
    }
});
jscape.IpSubstitutor.fromJSON = function(data) {
    var entries = [];
    if (data.rules && data.rules[1]) {
        var rules = data.rules[1];
        for (var i = 0; i < rules.length; i++) {
            entries.push(jscape.IpSubstitutor.Rule.fromJSON(rules[i]));
        }
    }
    return new jscape.IpSubstitutor(entries);
};

/** @class */
jscape.IpSubstitutor.Rule = Class.extend(
    /** @lends {jscape.IpSubstitutor.Rule.prototype} */
{
    /** @constructor */
    initialize: function(ip, substitution) {
        this.ip = ip;
        this.substitution = substitution;
    },

    toJSON: function() {
        return {
            '@class': "com.jscape.inet.mft.IPRuleSubstitutor$ExactSubstitutionRule",
            ip: this.ip,
            substitution: this.substitution
        };
    }
});
jscape.IpSubstitutor.Rule.fromJSON = function(data) {
    return new jscape.IpSubstitutor.Rule(data.ip, data.substitution);
};

/** @class */
jscape.FailoverConfiguration = Class.extend(
    /** @lends {jscape.FailoverConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(manager, substitutor, enabled, forceStart) {
        this.manager = manager;
        this.substitutor = substitutor;
        this.enabled = enabled || false;
        this.forceStart = forceStart || false;
    },

    toJSON: function() {
        return {
            managerConfiguration: this.manager.toJSON(),
            ipSubstitutor: this.substitutor,
            enabled: this.enabled,
            forceStart: this.forceStart
        }
    }
});
jscape.FailoverConfiguration.fromJSON = function(data) {
    return new jscape.FailoverConfiguration(jscape.ManagerConfiguration.fromJSON(data.managerConfiguration),
                                            jscape.IpSubstitutor.fromJSON(data.ipSubstitutor),
                                            data.enabled,
                                            data.forceStart);
};

/** @class */
jscape.FailoverSyncParameters = Class.extend(
    /** @lends {jscape.FailoverSyncParameters.prototype} */
{
    /** @constructor */
    initialize: function(host, port, timeout, username, password, substitutor, forceStart) {
        this.host = host;
        this.port = port;
        this.timeout = timeout;
        this.username = username;
        this.password = password;
        this.substitutor = substitutor;
        this.forceStart = forceStart;
    },

    toJSON: function() {
        return {
            host: this.host,
            port: this.port,
            timeoutMillis: this.timeout,
            username: this.username,
            password: this.password,
            substitutor: this.substitutor.toJSON(),
            forceStart: this.forceStart
        }
    }
});

/** @class */
jscape.settings.FailoverController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.FailoverController.prototype} */
{
    /** @const */
    SYNC_STATE_PERIOD: 3 * 1000,

    ipSubstitutorController: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.FailoverView(resources);
        this.ipSubstitutorController = new jscape.settings.IpSubstitutorController(resources);
    },

    /** @override */
    start: function(listener, managementService, serverSettings) {
        this._super(listener, managementService, serverSettings);

        this._startIPSubstitutionController();
    },

    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setFailoverServiceConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    onSyncServerState: function() {
        var config = this._createSyncConfiguration();
        this.managementService.syncFailoverServerState(config)
                .done($.proxy(this._onSuccess$SyncServer, this))
                .fail($.proxy(this._onError$SyncServer, this));
    },

    _onSuccess$SyncServer: function() {
        this._failoverSyncStatePoll();
    },

    _onError$SyncServer: function() {
        // ok
    },

    _failoverSyncStatePoll: function() {
        setTimeout($.proxy(function() {
            this.managementService.failoverSyncComplete().done(function(complete) {
                if (!complete) {
                    this._failoverSyncStatePoll();
                }
            });
        }, this), this.SYNC_STATE_PERIOD);
    },

    _initConfiguration: function() {
        this.config = this.serverSettings.getFailoverServiceConfiguration();
    },

    _createConfiguration: function() {
        return new jscape.FailoverConfiguration(
                new jscape.ManagerConfiguration(this.view.getHost(),
                                                this.view.getPort(),
                                                this.view.getTimeout(),
                                                this.view.getUsername(),
                                                this.view.getPassword()),
                this.ipSubstitutorController.getSubstitutor(),
                this.view.getEnabled(),
                this.view.getForceStart());
    },

    _createSyncConfiguration: function() {
        return new jscape.FailoverSyncParameters(this.view.getHost(),
                                                this.view.getPort(),
                                                this.view.getTimeout(),
                                                this.view.getUsername(),
                                                this.view.getPassword(),
                                                this.ipSubstitutorController.getSubstitutor(),
                                                this.view.getForceStart());
    },

    _setupView: function() {
        var config = this.config;
        this.view.setHost(config.manager.host);
        this.view.setPort(config.manager.port);
        this.view.setTimeout(config.manager.timeout);
        this.view.setUsername(config.manager.username);
        this.view.setPassword(config.manager.password);
        this.view.setEnabled(config.enabled);
        this.view.setForceStart(config.forceStart);
        this.view.show(this);
    },

    _startIPSubstitutionController: function() {
        this.ipSubstitutorController.start(this.config.substitutor);
    }
});

/** @class */
jscape.settings.FailoverView = Class.extend(
    /** @lends {jscape.settings.FailoverView.prototype} */
{
    /** @const */
    DEFAULT_HOST: "localhost",
    /** @const */
    DEFAULT_PORT: 10880,
    /** @const */
    DEFAULT_TIMEOUT: 10000,
    /** @const */
    TIMEOUT_FACTOR: 1000,

    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.form = $("fieldset#ssfs-fields");
        this.enabled = $("#ssfs-enabled");
        this.host = $("#ssfs-host");
        this.port = $("#ssfs-port");
        this.timeout = $("#ssfs-timeout");
        this.username = $("#ssfs-user");
        this.password = $("#ssfs-pwd");
        this.forceStart = $("#ssfs-forcestart");
        this.syncButton = $("#ssfs-synchronize");

        this._initComponents();
        this._initListeners();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.FailoverView.LISTENER;
        this._initState();
    },

    getEnabled: function () {
        return this.enabled.prop("checked");
    },

    setEnabled: function (value) {
        this.enabled.prop("checked", value);
        this._adjustFieldState();
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value ? value : this.DEFAULT_HOST);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value ? value : this.DEFAULT_PORT);
    },

    getTimeout: function() {
        return this.timeout.numberspinner("getValue") * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", value ? value / this.TIMEOUT_FACTOR : this.DEFAULT_TIMEOUT);
    },

    getUsername: function() {
        return this.username.val();
    },

    setUsername: function(value) {
        this.username.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getForceStart: function() {
        return this.forceStart.prop("checked");
    },

    setForceStart: function(value) {
        this.forceStart.prop("checked", value);
    },

    _initComponents: function() {
        this.enabled.click($.proxy(this._onClick$Enabled, this));
        this.host.validatebox({required: true, missingMessage: this.resources.getString("FAILOVER_ERROR_BAD_HOST")});
        this.port.numberspinner({min: 1, max: 65535, required: true});
        this.timeout.numberspinner({min: 1, max: 999});
        this.username.validatebox({required: true, missingMessage: this.resources.getString("FAILOVER_ERROR_BAD_USERNAME")});
        this.password.validatebox({required: true, missingMessage: this.resources.getString("FAILOVER_ERROR_BAD_PASSWORD")});
        this.syncButton.linkbutton();
    },

    _initListeners: function() {
        this.syncButton.click($.proxy(this._onClick$SyncButton, this));
    },

    _initState: function() {
        this._adjustFieldState();
    },

    _onClick$Enabled: function () {
        this._adjustFieldState();
    },

    _onClick$SyncButton: function() {
        this.listener.onSyncServerState();
        return false;
    },

    _adjustFieldState: function () {
        var enabled = this.getEnabled();
        this.host.validatebox(enabled ? "enableValidation" : "disableValidation");
        this.port.numberspinner(enabled ? "enableValidation" : "disableValidation");
        this.username.validatebox(enabled ? "enableValidation" : "disableValidation");
        this.password.validatebox(enabled ? "enableValidation" : "disableValidation");

        this.form.form("validate");
    }
});

/** @const */
jscape.settings.FailoverView.LISTENER = {
    onSyncServerState: $.noop()
};


/** @class */
jscape.settings.IpSubstitutorController = Class.extend(
    /** @lends {jscape.settings.IpSubstitutorController.prototype} */
{
    view: null,
    rules: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.view = new jscape.settings.IpSubstitutorView(resources);

        this._initValidators(resources);
    },

    start: function(substitutor) {
        this.rules = substitutor.rules;
        this.view.setRules(this.rules);
        this.view.show(this);
    },

    getSubstitutor: function() {
        var rules = this.view.getRules();
        return new jscape.IpSubstitutor(rules);
    },

    onAddRule: function() {
        this.view.showAddDialog({
            onSubmit: $.proxy(this._onSubmit$AddDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onEditRule: function(rule) {
        var index = $.inArray(rule, this.rules);
        this.view.showEditDialog(rule.ip, rule.substitution, {
            onSubmit: $.proxy(this._onSubmit$EditDialog, this, index),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onDeleteRule: function(rule) {
        this.view.showConfirmDeleteDialog(rule.ip).done($.proxy(this._onConfirm$DeleteDialog, this, rule));
    },

    _onSubmit$AddDialog: function(dialog) {
        var rule = new jscape.IpSubstitutor.Rule(dialog.getIp(), dialog.getSubstitution());
        dialog.hide();

        this.rules.push(rule);
        this.view.setRules(this.rules, $.inArray(rule, this.rules));
    },

    _onSubmit$EditDialog: function(index, dialog) {
        var rule = new jscape.IpSubstitutor.Rule(dialog.getIp(), dialog.getSubstitution());
        dialog.hide();

        this.rules[index] = rule;
        this.view.setRules(this.rules, index);
    },

    _onConfirm$DeleteDialog: function(rule) {
        var index = $.inArray(rule, this.rules);
        if (index != -1) {
            this.rules.splice(index, 1);
            this.view.setRules(this.rules, index);
        }
    },

    _onCancel$Dialog: function(dialog) {
        dialog.hide();
    },

    _initValidators: function(resources) {
        $.extend($.fn.validatebox.defaults.rules, {
            'ip_nonempty': {
                validator: function(value) { return value && value.trim().length != 0; },
                message: resources.getString("IP_SUBSTITUTION_ERROR_BAD_IP")
            },
            'substitution_nonempty': {
                validator: function(value) { return value && value.trim().length != 0; },
                message: resources.getString("IP_SUBSTITUTION_ERROR_BAD_SUBSTITUTION")
            }
        });
    }
});

/** @class */
jscape.settings.IpSubstitutorDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.settings.IpSubstitutorDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#ssfsip-dialog-rule"), resources);

        this.ip = $("#ssfsip-ip");
        this.substitution = $("#ssfsip-substitution");

        this._initComponents();
    },

    reset: function() {
        this.ip.val("");
        this.substitution.val("");
    },

    getIp: function() {
        return this.ip.val();
    },

    setIp: function(value) {
        this.ip.val(value || "");
    },

    getSubstitution: function() {
        return this.substitution.val();
    },

    setSubstitution: function(value) {
        this.substitution.val(value || "");
    },

    /** @override */
    _valid: function() {
        return this.ip.validatebox("isValid") && this.substitution.validatebox("isValid");
    },

    _initComponents: function() {
        this.ip.validatebox({
            required: true,
            validType: ["ip_nonempty"],
            missingMessage: this.resources.getString("IP_SUBSTITUTION_ERROR_BAD_IP")
        });
        this.substitution.validatebox({
            required: true,
            validType: ["substitution_nonempty"],
            missingMessage: this.resources.getString("IP_SUBSTITUTION_ERROR_BAD_SUBSTITUTION")
        });
    }
});

/** @class */
jscape.settings.IpSubstitutorView = Class.extend(
    /** @lends {jscape.settings.IpSubstitutorView.prototype} */
{
    resources: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.table = $("#ssfsip-data");
        this.addButton = $("#ssfsip-add");
        this.editButton = $("#ssfsip-edit");
        this.deleteButton = $("#ssfsip-delete");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.IpSubstitutorView.LISTENER;
        this._initState();
    },

    showAddDialog: function(listener) {
        var dialog = new jscape.settings.IpSubstitutorDialog(this.resources);
        dialog.setTitle(this.resources.getString("IP_SUBSTITUTION_ADD_TITLE"));
        dialog.reset();
        dialog.show(listener);
    },

    showEditDialog: function(ip, substitution, listener) {
        var dialog = new jscape.settings.IpSubstitutorDialog(this.resources);
        dialog.setTitle(this.resources.getString("IP_SUBSTITUTION_EDIT_TITLE"));
        dialog.reset();
        dialog.setIp(ip);
        dialog.setSubstitution(substitution);
        dialog.show(listener);
    },

    showConfirmDeleteDialog: function(ip) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("IP_SUBSTITUTION_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("IP_SUBSTITUTION_CONFIRM_DELETE_MESSAGE", {0: ip}));
    },

    getRules: function() {
        return this.table.datagrid("getRows");
    },

    setRules: function(rules, selectIndex) {
        this.table.datagrid("loading");
        this.table.datagrid("loadData", rules);
        this.table.datagrid("loaded");

        if (selectIndex !== undefined) {
            var index = Math.max(0, Math.min(selectIndex, rules.length - 1));
            this.table.datagrid("selectRow", index);
        }
    },

    _init: function() {
        this._initComponents();
        this._initListeners();
    },

    _initComponents: function() {
        this.table.datagrid({
            singleSelect: true,
            remoteSort: false,
            fitColumns: true,
            onSelect: $.proxy(this._onSelect$Row, this),
            onSortColumn: $.proxy(this._onSort$Column, this),
            onDblClickRow: $.proxy(this._onClick$Edit, this),
            columns: [[
                    {field: "ip", title: this.resources.getString("TABLE_COLUMN_IP"), sortable: true, width: 100},
                    {field: "substitution", title: this.resources.getString("TABLE_COLUMN_SUBSTITUTION"), sortable: true, resizable: true, width: 100}
                ]]
        });
        this.addButton.linkbutton();
        this.editButton.linkbutton();
        this.deleteButton.linkbutton();
    },

    _initListeners: function() {
        this.addButton.click($.proxy(this._onClick$Add, this));
        this.editButton.click($.proxy(this._onClick$Edit, this));
        this.deleteButton.click($.proxy(this._onClick$Delete, this));
    },

    _initState: function() {
        this._adjustButtonState();
    },

    _onSelect$Row: function () {
        this._adjustButtonState();
    },

    _onSort$Column: function() {
        this._adjustButtonState();
    },

    _onClick$Add: function() {
        this.listener.onAddRule();
        return false;
    },

    _onClick$Edit: function() {
        if (!this.editButton.linkbutton("options").disabled) {
            this.listener.onEditRule(this.table.datagrid("getSelected"));
        }
        return false;
    },

    _onClick$Delete: function() {
        if (!this.deleteButton.linkbutton("options").disabled) {
            this.listener.onDeleteRule(this.table.datagrid("getSelected"));
        }
        return false;
    },

    _adjustButtonState: function() {
        var rule = this.table.datagrid("getSelected");
        this.editButton.linkbutton(rule != null ? "enable" : "disable");
        this.deleteButton.linkbutton(rule != null ? "enable" : "disable");
    }
});

/** @const */
jscape.settings.IpSubstitutorView.LISTENER = {
    onAddRule: $.noop(),
    onEditRule: $.noop(),
    onDeleteRule: $.noop()
};