var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.ManagementAccessConfiguration = Class.extend(
    /** @lends {jscape.ManagementAccessConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e) {
        this.rules = a || [];
        this.timeout = b !== undefined ? b : null;
        this.authAttempts = c !== undefined ? c : null;
        this.authTimeout = d !== undefined ? d : null;
        this.restoringTimeout = e !== undefined ? e : null;
    },

    toJSON: function() {
        return {
            rules: this.rules,
            authenticationTimeout: this.timeout,
            maxAuthenticationAttempts: this.authAttempts,
            maxAuthenticationTimeout: this.authTimeout,
            restoringTimeout: this.restoringTimeout
        };
    }
});
jscape.ManagementAccessConfiguration.fromJSON = function(data) {
    var rules = [];
    for (var i = 0, size = data.rules.length; i < size; ++i) {
        rules.push(jscape.IPAccessRule.fromJSON(data.rules[i]));
    }
    return new jscape.ManagementAccessConfiguration(rules,
                                                    data.authenticationTimeout,
                                                    data.maxAuthenticationAttempts,
                                                    data.maxAuthenticationTimeout,
                                                    data.restoringTimeout);
};

/** @class */
jscape.IPAccessRule = Class.extend(
    /** @lends {jscape.IPAccessRule.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.mask = a;
        this.reason = b || "";
        this.allowed = c || false;
    },

    toJSON: function() {
        return {
            addressMask: this.mask,
            reason: this.reason,
            allowed: this.allowed
        };
    }
});
jscape.IPAccessRule.fromJSON = function(data) {
    return new jscape.IPAccessRule(data.addressMask, data.reason, data.allowed);
};

/** @class */
jscape.settings.ManagerAccessController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.ManagerAccessController.prototype} */
{
    view: null,
    config: null,
    rules: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.ManagerAccessView(resources);

        this._initValidators(resources);
    },

    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setManagementAccessServiceConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    onAddRule: function() {
        this.view.showAddDialog({
            onSubmit: $.proxy(this._onSubmit$AddDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onEditRule: function(rule) {
        var index = $.inArray(rule, this.rules);
        this.view.showEditDialog({
            onSubmit: $.proxy(this._onSubmit$EditDialog, this, index),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, rule.mask, rule.reason, rule.allowed);
    },

    onDeleteRule: function(rule) {
        this.view.showConfirmDeleteDialog(rule.mask).done($.proxy(this._onConfirm$DeleteDialog, this, rule));
    },

    onRuleUp: function(rule) {
        var index = $.inArray(rule, this.rules);
        this.rules.splice(index - 1, 2, this.rules[index], this.rules[index - 1]);
        this.view.setAccessRules(this.rules, index - 1);
    },

    onRuleDown: function(rule) {
        var index = $.inArray(rule, this.rules);
        this.rules.splice(index, 2, this.rules[index + 1], this.rules[index]);
        this.view.setAccessRules(this.rules, index + 1);
    },

    _onSubmit$AddDialog: function(dialog) {
        var rule = new jscape.IPAccessRule(dialog.getMask(), dialog.getReason(), dialog.getAllowed());
        dialog.hide();

        this.rules.push(rule);
        this.view.setAccessRules(this.rules, $.inArray(rule, this.rules));
    },

    _onSubmit$EditDialog: function(index, dialog) {
        var rule = new jscape.IPAccessRule(dialog.getMask(), dialog.getReason(), dialog.getAllowed());
        dialog.hide();

        this.rules[index] = rule;
        this.view.setAccessRules(this.rules, index);
    },

    _onConfirm$DeleteDialog: function(rule) {
        var index = $.inArray(rule, this.rules);
        if (index != -1) {
            this.rules.splice(index, 1);
            this.view.setAccessRules(this.rules, index);
        }
    },

    _onCancel$Dialog: function(dialog) {
        dialog.hide();
    },

    _initConfiguration: function() {
        this.config = this.serverSettings.getManagementAccessServiceConfiguration();
        this.rules = [].concat(this.config.rules);
    },

    _createConfiguration: function() {
        return new jscape.ManagementAccessConfiguration(this.view.getAccessRules(),
                                                        this.view.getTimeout(),
                                                        this.view.getAuthAttempts(),
                                                        this.view.getAuthTimeout(),
                                                        null);
    },

    _setupView: function() {
        this.view.setAccessRules(this.rules);
        this.view.setTimeout(this.config.timeout);
        this.view.setAuthAttempts(this.config.authAttempts);
        this.view.setAuthTimeout(this.config.authTimeout);
        this.view.show(this);
    },

    _initValidators: function(resources) {
        $.extend($.fn.validatebox.defaults.rules, {
            'ipaccess_nonempty': {
                validator: function(value) { return value && value.trim().length != 0; },
                message: resources.getString("MANAGER_ACCESS_ERROR_BAD_MASK")
            },
            'ipaccess_notexists': {
                validator: $.proxy(this._validatorRuleExists, this),
                message: resources.getString("MANAGER_ACCESS_ERROR_RULE_EXISTS")
            }
        });
    },

    _validatorRuleExists: function(value, excludes) {
        for (var i = 0; i < this.rules.length; i++) {
            var mask = this.rules[i].mask;
            if (excludes && $.inArray(mask, excludes) != -1) {
                continue;
            }
            if (mask == value) {
                return false;
            }
        }
        return true;
    }
});

/** @class */
jscape.settings.IpAccessDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.settings.IpAccessDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#ssma-dialog-ipaccess"), resources);
        this.mask = $("#ssma-ipaccessmask");
        this.reason = $("#ssma-ipaccessreason");
        this.allowed = $("input:radio[name=ssma-ipaccessallowed]");

        this._initMaskValidator();
    },

    reset: function() {
        this.setMask("");
        this.setReason("");
        this.setAllowed(true);
    },

    getMask: function() {
        return this.mask.val();
    },

    setMask: function(value) {
        this.mask.val(value);
        this._initMaskValidator(value);
    },

    getReason: function() {
        return this.reason.val();
    },

    setReason: function(value) {
        this.reason.val(value);
    },

    getAllowed: function() {
        return (/^true$/i).test(this.allowed.filter(":checked").val());
    },

    setAllowed: function(value) {
        this.allowed.filter("[value=" + Boolean(value) + "]").prop("checked", true);
    },

    /** @override */
    _valid: function() {
        return this.mask.validatebox("isValid");
    },

    _initMaskValidator: function(mask) {
        this.mask.validatebox({
            required: true,
            validType: ["ipaccess_nonempty", "ipaccess_notexists" + (mask ? "['" + mask + "']" : "")],
            missingMessage: this.resources.getString("MANAGER_ACCESS_ERROR_BAD_MASK")
        });
    }
});

/** @class */
jscape.settings.ManagerAccessView = Class.extend(
    /** @lends {jscape.settings.ManagerAccessView.prototype} */
{
    /** @const */
    DEFAULT_TIMEOUT: 10,
    /** @const */
    DEFAULT_AUTH_ATTEMPTS: 3,
    /** @const */
    DEFAULT_AUTH_TIMEOUT: 1,
    /** @const */
    TIMEOUT_FACTOR: 1000,
    /** @const */
    AUTH_TIMEOUT_FACTOR: 1000 * 60,

    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.timeoutEnabled = $("#ssma-timeoutbox");
        this.timeout = $("#ssma-timeout");
        this.authEnabled = $("#ssma-auth");
        this.authAttempts = $("#ssma-authattempt");
        this.authTimeout = $("#ssma-authtimeout");
        this.table = $("#ssma-data");
        this.moveUpButton = $("#ssma-up");
        this.moveDownButton = $("#ssma-down");
        this.addButton = $("#ssma-add");
        this.editButton = $("#ssma-edit");
        this.deleteButton = $("#ssma-delete");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.ManagerAccessView.LISTENER;
        this._initState();
    },

    showAddDialog: function(listener) {
        var dialog = new jscape.settings.IpAccessDialog(this.resources);
        dialog.setTitle(this.resources.getString("MANAGER_ACCESS_ADD_TITLE"));
        dialog.reset();
        dialog.show(listener);
    },

    showEditDialog: function(listener, mask, reason, allowed) {
        var dialog = new jscape.settings.IpAccessDialog(this.resources);
        dialog.setTitle(this.resources.getString("MANAGER_ACCESS_EDIT_TITLE"));
        dialog.setMask(mask);
        dialog.setReason(reason);
        dialog.setAllowed(allowed);
        dialog.show(listener);
    },

    showConfirmDeleteDialog: function (mask) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("MANAGER_ACCESS_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("MANAGER_ACCESS_CONFIRM_DELETE_MESSAGE", {0: mask}));
    },

    getAccessRules: function() {
        return this.table.datagrid("getRows");
    },

    setAccessRules: function(rules, selectIndex) {
        this.table.datagrid("loading");
        this.table.datagrid("loadData", rules);
        this.table.datagrid("loaded");

        if (selectIndex !== undefined) {
            var index = Math.max(0, Math.min(selectIndex, rules.length - 1));
            this.table.datagrid("selectRow", index);
        }
    },

    getTimeout: function() {
        return this.timeoutEnabled.prop("checked")
                ? this.timeout.numberspinner("getValue") * this.TIMEOUT_FACTOR
                : null;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", value ? value / this.TIMEOUT_FACTOR : this.DEFAULT_TIMEOUT);
        this.timeoutEnabled.prop("checked", !!value);
        this._adjustFieldState();
    },

    getAuthAttempts: function() {
        return this.authEnabled.prop("checked")
                ? this.authAttempts.numberspinner("getValue")
                : null;
    },

    setAuthAttempts: function(value) {
        this.authAttempts.numberspinner("setValue", value ? value : this.DEFAULT_AUTH_ATTEMPTS);
        this.authEnabled.prop("checked", !!value);
        this._adjustFieldState();
    },

    getAuthTimeout: function() {
        return this.authEnabled.prop("checked")
                ? this.authTimeout.numberspinner("getValue") * this.AUTH_TIMEOUT_FACTOR
                : null;
    },

    setAuthTimeout: function(value) {
        value = value ? value / this.AUTH_TIMEOUT_FACTOR : this.DEFAULT_AUTH_TIMEOUT;
        this.authTimeout.numberspinner("setValue", value);
    },

    _init: function () {
        this.timeoutEnabled.click($.proxy(this._onClick$Timeout, this));
        this.timeout.numberspinner({min: 1, max: 10, disabled: true});

        this.authEnabled.click($.proxy(this._onClick$AuthEnabled, this));
        this.authAttempts.numberspinner({min: 1, max: 10, disabled: true});
        this.authTimeout.numberspinner({min: 1, max: 10, disabled: true});

        this.table.datagrid({
            singleSelect: true,
            remoteSort: false,
            fitColumns: true,
            onSelect: $.proxy(this._onSelect$Row, this),
            onSortColumn: $.proxy(this._onSort$Column, this),
            columns: [[
                    {field: "mask", title: this.resources.getString("TABLE_COLUMN_IPACCESS_MASK"), sortable: true, width: 60},
                    {field: "allowed", title: this.resources.getString("TABLE_COLUMN_IPACCESS_ALLOWED"), formatter: function (value) {return value ? "allowed" : "denied"}, styler: function (value) {return value ? "color:green" : "color:red"}, sortable: true, width: 15},
                    {field: "reason", title: this.resources.getString("TABLE_COLUMN_IPACCESS_REASON"), sortable: true, width: 25}
                ]]
        });

        this.moveUpButton.linkbutton({disabled: true});
        this.moveUpButton.click($.proxy(this._onClick$MoveUp, this));

        this.moveDownButton.linkbutton({disabled: true});
        this.moveDownButton.click($.proxy(this._onClick$MoveDown, this));

        this.addButton.linkbutton();
        this.addButton.click($.proxy(this._onClick$Add, this));

        this.editButton.linkbutton({disabled: true});
        this.editButton.click($.proxy(this._onClick$Edit, this));

        this.deleteButton.linkbutton({disabled: true});
        this.deleteButton.click($.proxy(this._onClick$Delete, this));
    },

    _initState: function() {
        this._adjustFieldState();
        this._adjustButtonState();
    },

    _onClick$Timeout: function() {
        this._adjustFieldState();
    },

    _onClick$AuthEnabled: function() {
        this._adjustFieldState();
    },

    _onSelect$Row: function() {
        this._adjustButtonState();
    },

    _onSort$Column: function() {
        this._adjustButtonState();
    },

    _onClick$MoveUp: function() {
        if (this._buttonEnabled(this.moveUpButton)) {
            this.listener.onRuleUp(this.table.datagrid("getSelected"));
        }
        return false;
    },

    _onClick$MoveDown: function() {
        if (this._buttonEnabled(this.moveDownButton)) {
            this.listener.onRuleDown(this.table.datagrid("getSelected"));
        }
        return false;
    },

    _onClick$Add: function() {
        this.listener.onAddRule();
        return false;
    },

    _onClick$Edit: function() {
        if (this._buttonEnabled(this.editButton)) {
            this.listener.onEditRule(this.table.datagrid("getSelected"));
        }
        return false;
    },

    _onClick$Delete: function() {
        if (this._buttonEnabled(this.deleteButton)) {
            this.listener.onDeleteRule(this.table.datagrid("getSelected"));
        }
        return false;
    },

    _adjustFieldState: function() {
        this.timeout.numberspinner(this.timeoutEnabled.prop("checked") ? "enable" : "disable");
        var enable = this.authEnabled.prop("checked") ? "enable" : "disable";
        this.authAttempts.numberspinner(enable);
        this.authTimeout.numberspinner(enable);
    },

    _adjustButtonState: function() {
        var rule = this.table.datagrid("getSelected");
        var index = this.table.datagrid("getRowIndex", rule);
        var count = this.table.datagrid("getRows").length;

        this._setButtonEnabled(this.moveUpButton, index > 0 && index <= count - 1);
        this._setButtonEnabled(this.moveDownButton, index >= 0 && index < count - 1);
        this._setButtonEnabled(this.editButton, rule != null);
        this._setButtonEnabled(this.deleteButton, rule != null);
    },

    _buttonEnabled: function(button) {
        return !button.linkbutton("options").disabled;
    },

    _setButtonEnabled: function(button, enabled) {
        button.linkbutton(enabled ? "enable" : "disable");
    }
});
/** @const */
jscape.settings.ManagerAccessView.LISTENER = {
    onAddRule: $.noop(),
    onEditRule: $.noop(),
    onDeleteRule: $.noop(),
    onRuleUp: $.noop(),
    onRuleDown: $.noop()
};