var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.Administrator = Class.extend(
    /** @lends {jscape.Administrator.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.name = a;
        this.login = b;
        this.enabled = c || false;
        this.passwordHash = d || "";
    },

    toJSON: function() {
        return this.password !== undefined
                ? {name: this.name, username: this.login, enabled: this.enabled, password: this.password}
                : {name: this.name, username: this.login, enabled: this.enabled, passwordHash: this.passwordHash};
    }
});
jscape.Administrator.fromJSON = function(data) {
    return new jscape.Administrator(data.name, data.username, data.enabled, data.passwordHash);
};

/** @class */
jscape.AdministratorList = Class.extend(
    /** @lends {jscape.AdministratorList.prototype} */
{
    /** @constructor */
    initialize: function(entries) {
        this.entries = [].concat(entries || []);
    },

    toJSON: function() {
        var values = [];
        for (var i = 0; i < this.entries.length; i++) {
            values.push(this.entries[i].toJSON());
        }
        return ["java.util.HashSet", values];
    }
});
jscape.AdministratorList.fromJSON = function(data) {
    var entries = [];
    if (data && data[1]) {
        var values = data[1];
        for (var i = 0; i < values.length; i++) {
            entries.push(jscape.Administrator.fromJSON(values[i]));
        }
    }
    return new jscape.AdministratorList(entries);
};

/** @class */
jscape.settings.AdministratorsController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.AdministratorsController.prototype} */
{
    view: null,
    administrators: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.AdministratorsView(resources);

        this._initValidators(resources);
    },

    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setManagementAccessServiceConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    onAdd: function() {
        this.view.showAddDialog({
            onSubmit: $.proxy(this._onSubmit$AddDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onEdit: function(admin) {
        var index = $.inArray(admin, this.administrators);
        this.view.showEditDialog({
            onSubmit: $.proxy(this._onSubmit$EditDialog, this, index),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, admin);
    },

    onDelete: function(admin) {
        this.view.showConfirmDeleteDialog(admin.name).done($.proxy(this._onConfirm$DeleteDialog, this, admin));
    },

    _onSubmit$AddDialog: function(dialog) {
        var admin = new jscape.Administrator(dialog.getName(), dialog.getLogin(), dialog.getEnabled());
        admin.password = dialog.getPassword();
        dialog.hide();

        this.administrators.push(admin);
        this.view.setAdministrators(this.administrators, $.inArray(admin, this.administrators));
    },

    _onSubmit$EditDialog: function(index, dialog) {
        var admin = new jscape.Administrator(dialog.getName(), dialog.getLogin(), dialog.getEnabled());
        admin.password = dialog.getNewPassword();
        dialog.hide();

        this.administrators[index] = admin;
        this.view.setAdministrators(this.administrators, index);
    },

    _onConfirm$DeleteDialog: function (admin) {
        var index = $.inArray(admin, this.administrators);
        if (index != -1) {
            this.administrators.splice(index, 1);
            this.view.setAdministrators(this.administrators, index);
        }
    },

    _onCancel$Dialog: function(dialog) {
        dialog.hide();
    },

    _initConfiguration: function() {
        this.administrators = this.serverSettings.getAdministrators().entries;
    },

    _createConfiguration: function() {
        return new jscape.AdministratorList(this.view.getAdministrators());
    },

    _setupView: function() {
        this.view.show(this);
        this.view.setAdministrators(this.administrators);
    },

    _initValidators: function(resources) {
        $.extend($.fn.validatebox.defaults.rules, {
            'login_nonempty': {
                validator: function(value) { return value && value.trim().length != 0; },
                message: this.resources.getString("ADMINISTRATORS_ERROR_BAD_LOGIN")
            },
            'login_notexists': {
                validator: $.proxy(this._validatorLoginExists, this),
                message: this.resources.getString("ADMINISTRATORS_ERROR_LOGIN_EXISTS")
            },
            'password_confirm': {
                validator: $.proxy(this._validatorConfirmPassword, this),
                message: resources.getString("ADMINISTRATORS_ERROR_PASSWORDS_NOT_MATCH")
            }
        });
    },

    _validatorLoginExists: function(value) {
        for (var i = 0; i < this.administrators.length; i++) {
            if (this.administrators[i].login == value) {
                return false;
            }
        }
        return true;
    },

    _validatorConfirmPassword: function(value, params) {
        return value == $(params[0]).val();
    }
});

/** @class */
jscape.settings.AddAdministratorDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.settings.AddAdministratorDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#ssmsadmin-dialog-add"), resources);

        this.name = $("#ssmsadminadd-name");
        this.login = $("#ssmsadminadd-login");
        this.password = $("#ssmsadminadd-pwd");
        this.confirmPassword = $("#ssmsadminadd-confirmpwd");
        this.enabled = $("#ssmsadminadd-enabled");

        this._init();
    },

    reset: function() {
        this.name.val("");
        this.login.val("");
        this.password.val("");
        this.confirmPassword.val("");
        this.enabled.prop("checked", false);
    },

    getName: function() {
        return this.name.val();
    },

    getLogin: function() {
        return this.login.val();
    },

    getPassword: function() {
        return this.password.val();
    },

    getEnabled: function() {
        return this.enabled.prop("checked");
    },

    /** @override */
    _valid: function() {
        return this.login.validatebox("isValid")
               && this.password.validatebox("isValid")
               && this.confirmPassword.validatebox("isValid");
    },

    _init: function() {
        this.login.validatebox({
            required: true,
            validType: ["login_nonempty", "login_notexists"],
            missingMessage: this.resources.getString("ADMINISTRATORS_ERROR_BAD_LOGIN")
        });
        this.password.validatebox({
            required: true,
            missingMessage: this.resources.getString("ADMINISTRATORS_ERROR_BAD_PASSWORD")
        });
        this.confirmPassword.validatebox({
            required: true,
            validType: "password_confirm['#" + this.password.attr("id") + "']",
            missingMessage: this.resources.getString("ADMINISTRATORS_ERROR_BAD_PASSWORD")
        });
    }
});

/** @class */
jscape.settings.EditAdministratorDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.settings.EditAdministratorDialog.prototype} */
{
    newPassword: undefined,

    /** @constructor */
    initialize: function(resources) {
        this._super($("#ssmsadmin-dialog-edit"), resources);

        this.name = $("#ssmsadminedit-name");
        this.login = $("#ssmsadminedit-login");
        this.enabled = $("#ssmsadminedit-enabled");
        this.changePassword = $("#ssmsadminedit-changepwdbtn");

        this._initComponents();
    },

    getName: function() {
        return this.name.val();
    },

    setName: function(value) {
        this.name.val(value);
    },

    getLogin: function() {
        return this.login.val();
    },

    setLogin: function(value) {
        this.login.val(value);
    },

    getNewPassword: function() {
        return this.newPassword;
    },

    getEnabled: function() {
        return this.enabled.prop("checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", value);
    },

    setEnabledChangeAllowed: function(value) {
        if (!value) {
            this.enabled.prop("readonly", true);
            this.enabled.click(function() { return false; });
        }
    },

    /** @override */
    _valid: function() {
        return true;
    },

    _initComponents: function() {
        this.changePassword.linkbutton();
        this.changePassword.click($.proxy(this._onClick$ChangePassword, this));
    },

    _onClick$ChangePassword: function() {
        var dialog = new jscape.settings.ChangePasswordDialog(this.resources);
        dialog.show({
            onSubmit: $.proxy(function(d) {d.hide(); this.newPassword = d.getPassword();}, this),
            onCancel: $.proxy(function(d) {d.hide(); this.newPassword = undefined;}, this)
        });
        return false;
    }
});

/** @class */
jscape.settings.ChangePasswordDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.settings.ChangePasswordDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#ssmsadmin-dialog-changepwd"), resources);

        this.password = $("#ssmsadminedit-pwd");
        this.confirmPassword = $("#ssmsadminedit-confirmpwd");

        this._initComponents();
    },

    /** @override */
    show: function(listener) {
        this._super(listener);
        this.password.val("");
        this.confirmPassword.val();
    },

    getPassword: function() {
        return this.password.val();
    },

    _initComponents: function() {
        this.password.validatebox({});
        this.confirmPassword.validatebox({});
        this.password.validatebox({
            required: true,
            missingMessage: this.resources.getString("ADMINISTRATORS_ERROR_BAD_PASSWORD")
        });
        this.confirmPassword.validatebox({
            required: true,
            validType: "password_confirm['#" + this.password.attr("id") + "']",
            missingMessage: this.resources.getString("ADMINISTRATORS_ERROR_BAD_PASSWORD"),
            invalidMessage: this.resources.getString("ADMINISTRATORS_ERROR_PASSWORDS_NOT_MATCH")
        });
    },

    /** @override */
    _valid: function() {
        return this.password.validatebox("isValid") && this.confirmPassword.validatebox("isValid");
    }
});

/** @class */
jscape.settings.AdministratorsView = Class.extend(
    /** @lends {jscape.settings.AdministratorsView.prototype} */
{
    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.table = $("#ssmsadmin-data");
        this.enableButton = $("#ssmsadmin-enalbe");
        this.disableButton = $("#ssmsadmin-disable");
        this.addButton = $("#ssmsadmin-add");
        this.editButton = $("#ssmsadmin-edit");
        this.deleteButton = $("#ssmsadmin-delete");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.AdministratorsView.LISTENER;
    },

    showAddDialog: function(listener) {
        var dialog = new jscape.settings.AddAdministratorDialog(this.resources);
        dialog.reset();
        dialog.show(listener);
    },

    showEditDialog: function(listener, admin) {
        var dialog = new jscape.settings.EditAdministratorDialog(this.resources);
        dialog.setName(admin.name);
        dialog.setLogin(admin.login);
        dialog.setEnabled(admin.enabled);
        dialog.setEnabledChangeAllowed(this._hasEnabledAdministrators(admin));
        dialog.show(listener);
    },

    showConfirmDeleteDialog: function(name) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("ADMINISTRATORS_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("ADMINISTRATORS_CONFIRM_DELETE_MESSAGE", {0: name}));
    },

    getAdministrators: function() {
        return this.table.datagrid("getRows");
    },

    setAdministrators: function(items, selectIndex) {
        this.table.datagrid("loading");
        this.table.datagrid("loadData", items);
        this.table.datagrid("loaded");

        if (selectIndex !== undefined) {
            var index = Math.max(0, Math.min(selectIndex, items.length - 1));
            this.table.datagrid("selectRow", index);
        }
    },

    _init: function() {
        this._initComponents();
        this._initListeners();
    },

    _initComponents: function () {
        this.table.datagrid({
            singleSelect: true,
            remoteSort: false,
            fitColumns: true,
            onSelect: $.proxy(this._onSelect$Row, this),
            onSortColumn: $.proxy(this._onSort$Column, this),
            columns: [[
                    {field: "name", title: this.resources.getString("TABLE_COLUMN_ADMINS_NAME"), sortable: true, width: 40},
                    {field: "login", title: this.resources.getString("TABLE_COLUMN_ADMINS_LOGIN"), sortable: true, width: 40},
                    {field: "enabled", title: this.resources.getString("TABLE_COLUMN_ADMINS_ENABLED"), formatter: function (value) {return value ? "enabled" : "disabled"}, styler: function (value) {return value ? "color:green" : "color:grey"}, sortable: true, width: 20}
                ]]
        });

        this.enableButton.linkbutton({disabled: true});
        this.disableButton.linkbutton({disabled: true});
        this.addButton.linkbutton();
        this.editButton.linkbutton({disabled: true});
        this.deleteButton.linkbutton({disabled: true});
    },

    _initListeners: function() {
        this.enableButton.click($.proxy(this._onClick$Enable, this));
        this.disableButton.click($.proxy(this._onClick$Disable, this));
        this.addButton.click($.proxy(this._onClick$Add, this));
        this.editButton.click($.proxy(this._onClick$Edit, this));
        this.deleteButton.click($.proxy(this._onClick$Delete, this));
    },

    _onSelect$Row: function() {
        this._adjustButtonState();
    },

    _onSort$Column: function() {
        this._adjustButtonState();
    },

    _onClick$Add: function () {
        this.listener.onAdd();
        return false;
    },

    _onClick$Edit: function () {
        if (this._buttonEnabled(this.editButton)) {
            this.listener.onEdit(this.table.datagrid("getSelected"));
        }
        return false;
    },

    _onClick$Delete: function() {
        if (this._buttonEnabled(this.deleteButton)) {
            this.listener.onDelete(this.table.datagrid("getSelected"));
        }
        return false;
    },

    _onClick$Enable: function () {
        if (this._buttonEnabled(this.enableButton)) {
            var admin = this.table.datagrid("getSelected");
            var index = this.table.datagrid("getRowIndex", admin);
            admin.enabled = true;
            this.table.datagrid("refreshRow", index);
            this.table.datagrid("selectRow", index);
        }

        return false;
    },

    _onClick$Disable: function () {
        if (this._buttonEnabled(this.disableButton)) {
            var admin = this.table.datagrid("getSelected");
            var index = this.table.datagrid("getRowIndex", admin);
            admin.enabled = false;
            this.table.datagrid("refreshRow", index);
            this.table.datagrid("selectRow", index);
        }

        return false;
    },

    _adjustButtonState: function() {
        var admin = this.table.datagrid("getSelected");
        var count = this.table.datagrid("getRows").length;
        var hasEnabledAdministrators = this._hasEnabledAdministrators(admin);

        this._setButtonEnabled(this.enableButton, admin && !admin.enabled);
        this._setButtonEnabled(this.disableButton, admin && admin.enabled && hasEnabledAdministrators);
        this._setButtonEnabled(this.editButton, admin != null);
        this._setButtonEnabled(this.deleteButton, admin != null && count > 1 && hasEnabledAdministrators);
    },

    _hasEnabledAdministrators: function (excludeAdmin) {
        var rows = this.table.datagrid("getRows");
        for (var i = 0; i < rows.length; ++i) {
            var admin = rows[i];
            if (admin != excludeAdmin && admin.enabled) {
                return true;
            }
        }
        return false;
    },

    _buttonEnabled: function(button) {
        return !button.linkbutton("options").disabled;
    },

    _setButtonEnabled: function(button, enabled) {
        button.linkbutton(enabled ? "enable" : "disable");
    }
});
/** @const */
jscape.settings.AdministratorsView.LISTENER ={
    onAdd: $.noop(),
    onEdit: $.noop(),
    onDelete: $.noop()
};

