var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.ServerManagementServiceConfiguration = Class.extend(
    /** @lends {jscape.ServerManagementServiceConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(host, port, timeout) {
        this.host = host;
        this.port = port;
        this.timeout = timeout;
    },

    toJSON: function() {
        return {
            host: this.host,
            port: this.port,
            connectionTimeoutMillis: this.timeout
        };
    }
});
jscape.ServerManagementServiceConfiguration.fromJSON = function(data) {
    return new jscape.ServerManagementServiceConfiguration(data.host, data.port, data.connectionTimeoutMillis);
};

/** @class */
jscape.settings.ManagerServiceController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.ManagerServiceController.prototype} */
{
    view: null,
    config: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.ManagerServiceView(resources);
    },

    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setServerManagementServiceConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    _initConfiguration: function() {
        this.config = this.serverSettings.getServerManagementServiceConfiguration();
    },

    _createConfiguration: function() {
        return new jscape.ServerManagementServiceConfiguration(this.view.getHost(),
                                                               this.view.getPort(),
                                                               this.view.getTimeout());
    },

    _setupView: function() {
        this.view.setHosts(this.serverSettings.getServerParameters().getInetAddresses());
        this.view.setHost(this.config.host);
        this.view.setPort(this.config.port);
        this.view.setTimeout(this.config.timeout);
        this.view.show(this);
    }
});

/** @class */
jscape.settings.ManagerServiceView = Class.extend(
    /** @lends {jscape.settings.ManagerServiceView.prototype} */
{
    /** @const */
    DEFAULT_PORT: 10880,
    /** @const */
    DEFAULT_TIMEOUT: 600,
    /** @const */
    TIMEOUT_FACTOR: 1000,

    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset#ssms-fields");
        this.host = $("#ssms-host");
        this.port = $("#ssms-port");
        this.timeout = $("#ssms-timeout");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.ManagerServiceView.LISTENER;
        this._initState();
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHost: function(value) {
        this.host.combobox("setValue", value);
        this.host.combobox("select", value)
    },

    setHosts: function(values) {
        var data = [];
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            data.push({host: value, value: value});
        }
        this.host.combobox("loadData", data);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value || this.DEFAULT_PORT);
    },

    getTimeout: function() {
        return this.timeout.numberspinner("getValue") * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", value ? value / this.TIMEOUT_FACTOR : this.DEFAULT_TIMEOUT);
    },

    _init: function() {
        this.host.combobox({
            valueField: "host",
            textField: "value",
            width: 120,
            required: true,
            missingMessage: this.resources.getString("MANAGER_SERVICE_ERROR_BAD_HOST")
        });
        this.port.numberspinner({min: 1, max: 65535, required: true});
        this.timeout.numberspinner({min: 1, max: 1000, required: true});
    },

    _initState: function() {
        this.fieldset.form("validate");
    }
});
/** @const */
jscape.settings.ManagerServiceView.LISTENER = {};