var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.SearchingServiceConfiguration = Class.extend(
    /** @lends {jscape.SearchingServiceConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(indexingPeriod, fileUpload, fileRename, accountUpdating, groupUpdating) {
        this.indexingPeriod = indexingPeriod !== undefined ? indexingPeriod : -1;
        this.fileUpload = fileUpload || false;
        this.fileRename = fileRename || false;
        this.accountUpdating = accountUpdating || false;
        this.groupUpdating = groupUpdating || false;
    },

    toJSON: function() {
        return {
            indexingPeriod: this.indexingPeriod != -1 ? this.indexingPeriod : null,
            updateFileIndexUponUpload: this.fileUpload,
            updateFileIndexUponRename: this.fileRename,
            updateAccountIndexUponUpdating: this.accountUpdating,
            updateGroupIndexUponUpdating: this.groupUpdating
        }
    }
});
jscape.SearchingServiceConfiguration.fromJSON = function(data) {
    return new jscape.SearchingServiceConfiguration(data.indexingPeriod,
                                                    data.updateFileIndexUponUpload,
                                                    data.updateFileIndexUponRename,
                                                    data.updateAccountIndexUponUpdating,
                                                    data.updateGroupIndexUponUpdating);
};

/** @class */
jscape.settings.SearchingController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.SearchingController.prototype} */
{
    config: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.SearchingView(resources);
    },

    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setSearchingServiceConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    onRebuildIndex: function() {
        this.managementService.startSearchFileIndexing()
                .done($.proxy(this._onSuccess$RebuildIndex, this))
                .fail($.proxy(this._onError$RebuildIndex, this));
    },

    _onSuccess$RebuildIndex: function() {
        // ok
    },

    _onError$RebuildIndex: function() {
        // ok
    },

    _initConfiguration: function() {
        this.config = this.serverSettings.getSearchingServiceConfiguration();
    },

    _createConfiguration: function() {
        return new jscape.SearchingServiceConfiguration(this.view.getIndexingPeriod(),
                                                        this.view.getUpdateFileIndexUponUpload(),
                                                        this.view.getUpdateFileIndexUponRename(),
                                                        this.view.getUpdateAccountIndexUponUpdate(),
                                                        this.view.getUpdateGroupIndexUponUpdate());
    },

    _setupView: function() {
        var config = this.config;
        this.view.setIndexingPeriod(config.indexingPeriod);
        this.view.setUpdateFileIndexUponUpload(config.fileUpload);
        this.view.setUpdateFileIndexUponRename(config.fileRename);
        this.view.setUpdateAccountIndexUponUpdate(config.accountUpdating);
        this.view.setUpdateGroupIndexUponUpdate(config.groupUpdating);
        this.view.show(this);
    }
});

/** @class */
jscape.settings.SearchingView = Class.extend(
    /** @lends {jscape.settings.SearchingView.prototype} */
{
    /** @const */
    INDEXING_PERIOD_FACTOR: 1000 * 60 * 60,
    /** @const */
    DEFAULT_INDEXING_PERIOD: 24,

    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.rebuildIndexEnabled = $("#sssearch-reindex");
        this.rebuildIndexPeriod = $("#sssearch-indexperiod");
        this.updateFileIndexUponUpload = $("#sssearch-ufiuu");
        this.updateFileIndexUponRename = $("#sssearch-ufiur");
        this.updateAccountIndexUponUpdate = $("#sssearch-uaiuu");
        this.updateGroupIndexUponUpdate = $("#sssearch-ugiuu");
        this.rebuildIndexButton = $("#sssearch-rebuildindex");

        this._initComponents();
        this._initListeners();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.SearchingView.LISTENER;
    },

    getIndexingPeriod: function() {
        return this.rebuildIndexEnabled.prop("checked")
                ? this.rebuildIndexPeriod.numberspinner("getValue") * this.INDEXING_PERIOD_FACTOR
                : null;
    },

    setIndexingPeriod: function(value) {
        if (value && value != -1) {
            this.rebuildIndexEnabled.prop("checked", true);
            this.rebuildIndexPeriod.numberspinner("setValue", value / this.INDEXING_PERIOD_FACTOR);
            this.rebuildIndexPeriod.numberspinner("enable");
        } else {
            this.rebuildIndexEnabled.prop("checked", false);
            this.rebuildIndexPeriod.numberspinner("setValue", this.DEFAULT_INDEXING_PERIOD);
            this.rebuildIndexPeriod.numberspinner("disable");
        }
    },

    getUpdateFileIndexUponUpload: function() {
        return this.updateFileIndexUponUpload.prop("checked");
    },

    setUpdateFileIndexUponUpload: function(value) {
        this.updateFileIndexUponUpload.prop("checked", value);
    },

    getUpdateFileIndexUponRename: function() {
        return this.updateFileIndexUponRename.prop("checked");
    },

    setUpdateFileIndexUponRename: function(value) {
        this.updateFileIndexUponRename.prop("checked", value);
    },

    getUpdateAccountIndexUponUpdate: function() {
        return this.updateAccountIndexUponUpdate.prop("checked");
    },

    setUpdateAccountIndexUponUpdate: function(value) {
        this.updateAccountIndexUponUpdate.prop("checked", value);
    },

    getUpdateGroupIndexUponUpdate: function() {
        return this.updateGroupIndexUponUpdate.prop("checked");
    },

    setUpdateGroupIndexUponUpdate: function(value) {
        this.updateGroupIndexUponUpdate.prop("checked", value);
    },

    _initComponents: function() {
        this.rebuildIndexPeriod.numberspinner({min: 1, max: 1000, disabled: true});
        this.rebuildIndexButton.linkbutton();
    },

    _initListeners: function() {
        this.rebuildIndexEnabled.click($.proxy(this._onClick$RebuildIndexEnable, this));
        this.rebuildIndexButton.click($.proxy(this._onClick$RebuildIndex, this));
    },

    _onClick$RebuildIndexEnable: function() {
        var enabled = this.rebuildIndexEnabled.prop("checked");
        this.rebuildIndexPeriod.numberspinner(enabled ? "enable" : "disable");
    },

    _onClick$RebuildIndex: function() {
        this.listener.onRebuildIndex(this);
        return false;
    }
});

/** @const */
jscape.settings.SearchingView.LISTENER = {
    onRebuildIndex: $.noop()
};
