var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.settings.ServerSettingsController = Class.extend(
    /** @lends {jscape.settings.ServerSettingsController.prototype} */
{
    view: null,
    modules: null,
    moduleQueue: null,
    serverSettings: null,
    managementService: null,

    /** @constructor */
    initialize: function(resources, modules) {
        this.modules = modules || [];
        this.view = new jscape.settings.ServerSettingsView(resources);
        this.menu = new jscape.Menu();
    },

    start: function(managementService) {
        this.managementService = managementService;

        this._initModuleQueue();

        this.managementService.getServerSettings().done($.proxy(function(settings) {
            this.serverSettings = settings;
            this._startNextModule();
        }, this));
    },

    onSubmit: function() {
        this._initModuleQueue();
        this._submitNextModule();
    },

    onCancel: function() {
        this._initModuleQueue();
        this._cancelNextModule();
    },

    onStarted: function() {
        if (this.moduleQueue.length == 0) {
            this.view.show(this);
        } else {
            this._startNextModule();
        }
    },

    onStartError: function() {},

    onSubmitSuccess: function() {
        this._redirect();
    },

    onSubmitError: function() {
    },

    onChangesSubmitted: function() {
        if (this.moduleQueue.length == 0) {
            this._storeSettings();
        } else {
            this._submitNextModule();
        }
    },

    onChangesCancelled: function() {
        if (this.moduleQueue.length == 0) {
            this._redirect();
        } else {
            this._cancelNextModule();
        }
    },

    _redirect: function() {
        window.location = "/";
    },

    _initModuleQueue: function() {
        this.moduleQueue = [].concat(this.modules);
    },

    _startNextModule: function() {
        var m = this.moduleQueue.pop();
        m.start(this, this.managementService, this.serverSettings);
        this.menu.bind(m, "settings");
    },

    _submitNextModule: function() {
        var m = this.moduleQueue.pop();
        m.submitChanges();
        this.menu.unbind("settings");
    },

    _cancelNextModule: function() {
        var m = this.moduleQueue.pop();
        m.cancelChanges();
        this.menu.unbind("settings");
    },

    _storeSettings: function () {
        this.managementService.setServerSettings(this.serverSettings)
                .done($.proxy(this.onSubmitSuccess, this))
                .fail($.proxy(this.onSubmitError, this));
    }
});

/** @class */
jscape.settings.ServerSettingsView = Class.extend(
    /** @lends {jscape.settings.ServerSettingsView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.panel = $("#server-settings-pane");
        this.tabs = $("#server-settings-tabs");
        this.saveButton = $("#server-settings-save");
        this.cancelButton = $("#server-settings-cancel");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.ServerSettingsView.LISTENER;
    },

    _init: function() {
        this._initComponents();
        this._initListeners();
    },

    _initComponents: function () {
        this.panel.panel({border: false});
        this.tabs.tabs({tabPosition: "left", headerWidth: 150, border: true});
        this.saveButton.linkbutton();
        this.cancelButton.linkbutton();
    },

    _initListeners: function() {
        this.saveButton.click($.proxy(this._onSubmit, this));
        this.cancelButton.click($.proxy(this._onCancel, this));
    },

    _onSubmit: function() {
        if (this._validate()) {
            this.listener.onSubmit();
        }
    },

    _onCancel: function() {
        this.listener.onCancel(this);
    },

    _validate: function() {
        var fields = this.panel.find(".validatebox-text:not(:disabled)");
        for (var i = 0; i < fields.length; i++) {
            var field = $(fields[i]);
            if (!field.validatebox("isValid")) {
                this._navigateFrom(field);
                field.focus();
                return false;
            }
        }
        return true;
    },

    _navigateFrom: function(source) {
        source.parents(".tabs-container").each($.proxy(function(_, elem) {
            var tabs = $(elem);
            var tabIndex = -1;
            $.each(tabs.tabs("tabs"), function(i, target) {
                if (target.has(source).length != 0) {
                    tabIndex = i;
                    return false;
                }
                return true;
            });

            if (tabIndex != -1) {
                tabs.tabs("select", tabIndex);
            }
            if (this.tabs != tabs) {
                this._navigateFrom(tabs);
            }
        }, this));
    }
});
jscape.settings.ServerSettingsView.LISTENER = {
    onSubmit: $.noop(),
    onCancel: $.noop()
};

/** @class */
jscape.settings.SettingsModule = jscape.ApplicationModule.extend(
    /** @lends {jscape.settings.SettingsModule.prototype} */
{
    serverSettings: null,
    managementService: null,

    /** @override */
    start: function(listener, managementService, serverSettings) {
        this.listener = listener || jscape.settings.SettingsModule.LISTENER;
        this.managementService = managementService;
        this.serverSettings = serverSettings;

        this._initConfiguration();
        this._setupView();

        this.listener.onStarted(this);
    },

    _initConfiguration: function() {
    },

    _setupView: function() {
    }
});
jscape.settings.SettingsModule.LISTENER = $.extend({}, jscape.ApplicationModule.LISTENER);