var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.WebServiceDescriptor = Class.extend(
    /** @lends {jscape.WebServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(config, resources, logoData, as2Config) {
        this.configuration = config;
        this.resources = resources || [];
        this.logoData = logoData || [];
        this.as2ServiceConfiguration = as2Config;
    }
});
jscape.WebServiceDescriptor.fromJSON = function(data) {
    var resources = [];
    if (data.resources) {
        var entries = data.resources;
        for (var i = 0; i < entries.length; i++) {
            resources.push(jscape.WebResources.fromJSON(entries[i]));
        }
    }
    return new jscape.WebServiceDescriptor(jscape.WebConfiguration.fromJSON(data.configuration),
                                           resources,
                                           data.logoData,
                                           jscape.AS2Configuration.fromJSON(data.as2ServiceConfiguration));
};
/** @class */
jscape.WebConfiguration = Class.extend(
    /** @lends {jscape.WebConfiguration.prototype} */
{
    /** @const */
    HTML_UI: 1,
    APPLET_UI: 2,
    DEFAULT_UI: this.HTML_UI | this.APPLET_UI,
    DEFAULT_SESSION_TTL: 30 * 60 * 1000,

    /** @constructor */
    initialize: function(httpEnabled,
                         httpHost,
                         httpPort,
                         httpsEnabled,
                         httpsHost,
                         httpsPort,
                         httpsKeyAlias,
                         excludedCiphers,
                         enabledSslProtocols,
                         lostPasswordEnabled,
                         redirectHttp,
                         domainVisible,
                         domainDropdown,
                         defaultDomain,
                         clientCertificateRequired,
                         serverName,
                         includeServerPort,
                         userInterface,
                         defaultUserInterface,
                         sslRenegotiationAllowed,
                         sessionTimeout,
                         captchaRequired) {
        this.httpEnabled = httpEnabled || false;
        this.httpHost = httpHost || "127.0.0.1";
        this.httpPort = httpPort || 80;
        this.httpsEnabled = httpsEnabled || false;
        this.httpsHost = httpsHost|| "127.0.0.1";
        this.httpsPort = httpsPort || 443;
        this.httpsKey = httpsKeyAlias || "";
        this.sessionTimeout = sessionTimeout || this.DEFAULT_SESSION_TTL;
        this.excludedCiphers = excludedCiphers || [];
        this.enabledSslProtocols = enabledSslProtocols || ["SSLv3", "TLSv1"];
        this.lostPasswordEnabled = lostPasswordEnabled != undefined ? lostPasswordEnabled : true;
        this.redirectHttp = redirectHttp || false;
        this.domainVisible = domainVisible != undefined ? domainVisible : true;
        this.domainDropdown = domainDropdown || false;
        this.domainDefault = defaultDomain || "";
        this.clientCertificateRequired = clientCertificateRequired || false;
        this.serverName = serverName || "";
        this.includeServerPort = includeServerPort != undefined ? includeServerPort : true;
        this.userInterface = userInterface || this.DEFAULT_UI;
        this.defaultUserInterface = defaultUserInterface || null;
        this.sslRenegotiationAllowed = sslRenegotiationAllowed || true;
        this.captchaRequired = captchaRequired || false;
    },

    toJSON: function() {
        return {
            httpEnabled: this.httpEnabled,
            httpAddress: this.httpHost,
            httpPort: this.httpPort,
            httpsEnabled: this.httpsEnabled,
            httpsAddress: this.httpsHost,
            httpsPort: this.httpsPort,
            keyAlias: this.httpsKey,
            excludedCiphers: this.excludedCiphers,
            enabledSslProtocols: this.enabledSslProtocols,
            lostPasswordEnabled: this.lostPasswordEnabled,
            redirectHttp: this.redirectHttp,
            domainVisible: this.domainVisible,
            domainDropdown: this.domainDropdown,
            defaultDomain: this.domainDefault,
            clientCertificateRequired: this.clientCertificateRequired,
            serverName: this.serverName,
            includeServerPort: this.includeServerPort,
            userInterface: this.userInterface,
            defaultUserInterface: this.defaultUserInterface,
            sslRenegotiationAllowed: this.sslRenegotiationAllowed,
            httpSessionTtlMillis: this.sessionTimeout,
            captchaRequired: this.captchaRequired
        }
    }
});
jscape.WebConfiguration.fromJSON = function(data) {
    return new jscape.WebConfiguration(data.httpEnabled,
                                       data.httpAddress,
                                       data.httpPort,
                                       data.httpsEnabled,
                                       data.httpsAddress,
                                       data.httpsPort,
                                       data.keyAlias,
                                       data.excludedCiphers,
                                       data.enabledSslProtocols,
                                       data.lostPasswordEnabled,
                                       data.redirectHttp,
                                       data.domainVisible,
                                       data.domainDropdown,
                                       data.defaultDomain,
                                       data.clientCertificateRequired,
                                       data.serverName,
                                       data.includeServerPort,
                                       data.userInterface,
                                       data.defaultUserInterface,
                                       data.sslRenegotiationAllowed,
                                       data.httpSessionTtlMillis,
                                       data.captchaRequired);
};

/** @class */
jscape.WebResources = Class.extend(
    /** @lends {jscape.WebResources.prototype} */
{
    /** @constructor */
    initialize: function(lang, displayName, properties) {
        this.lang = lang || "";
        this.displayName = displayName || lang;
        this.properties = properties || {};
    }
});
jscape.WebResources.fromJSON = function(data) {
    return new jscape.WebResources(data.lang, data.displayName, data.properties);
};

/** @class */
jscape.ManagementWebServerConfiguration = Class.extend(
    /** @lends {jscape.ManagementWebServerConfiguration.prototype} */
{
    /** @const */
    DEFAULT_HTTP_PORT: 11880,
    DEFAULT_HTTPS_PORT: 11443,
    DEFAULT_SESSION_TTL: 30 * 60 * 1000,

    /** @constructor */
    initialize: function(httpHost,
                         httpPort,
                         httpEnabled,
                         httpsHost,
                         httpsPort,
                         httpsEnabled,
                         httpsKeyAlias,
                         excludedCiphers,
                         enabledSslProtocols,
                         clientCertificateRequired,
                         sslRenegotiationAllowed,
                         sessionTimeout) {
        this.httpHost = httpHost;
        this.httpPort = httpPort;
        this.httpEnabled = httpEnabled;
        this.httpsHost = httpsHost;
        this.httpsPort = httpsPort;
        this.httpsEnabled = httpsEnabled;
        this.httpsKeyAlias = httpsKeyAlias;
        this.excludedCiphers = excludedCiphers;
        this.enabledSslProtocols = enabledSslProtocols;
        this.clientCertificateRequired = clientCertificateRequired;
        this.sslRenegotiationAllowed = sslRenegotiationAllowed;
        this.sessionTimeout = sessionTimeout;
    },

    toJSON: function() {
        return {
            httpHost: this.httpHost,
            httpPort: this.httpPort,
            httpEnabled: this.httpEnabled,
            httpsHost: this.httpsHost,
            httpsPort: this.httpsPort,
            httpsEnabled: this.httpsEnabled,
            httpsKeyAlias: this.httpsKeyAlias,
            excludedCiphers: this.excludedCiphers,
            enabledSslProtocols: this.enabledSslProtocols,
            clientCertificateRequired: this.clientCertificateRequired,
            sslRenegotiationAllowed: this.sslRenegotiationAllowed,
            httpSessionTtlMillis: this.sessionTimeout
        }
    }
});
jscape.ManagementWebServerConfiguration.fromJSON = function(data) {
    return new jscape.ManagementWebServerConfiguration(data.httpHost,
                                                       data.httpPort,
                                                       data.httpEnabled,
                                                       data.httpsHost,
                                                       data.httpsPort,
                                                       data.httpsEnabled,
                                                       data.httpsKeyAlias,
                                                       data.excludedCiphers,
                                                       data.enabledSslProtocols,
                                                       data.clientCertificateRequired,
                                                       data.sslRenegotiationAllowed,
                                                       data.httpSessionTtlMillis);
};

/** @class */
jscape.WebUIType = Class.extend(
    /** @lends {jscape.WebUIType.prototype} */
{
    /** @constructor */
    initialize: function(code, defaultCode, value) {
        this.code = code;
        this.defaultCode = defaultCode;
        this.value = value;
    }
});
/** @const */
jscape.WebUIType.HTML = new jscape.WebUIType(1, null, "HTML");
jscape.WebUIType.APPLET = new jscape.WebUIType(2, null, "APPLET");
jscape.WebUIType.HTML_APPLET = new jscape.WebUIType(3, 1, "HTML_APPLET");
jscape.WebUIType.APPLET_HTML = new jscape.WebUIType(3, 2, "APPLET_HTML");
jscape.WebUIType.typeFor = function(code, defaultCode) {
    if (jscape.WebUIType.HTML.code == code) {
        return jscape.WebUIType.HTML;
    } else if (jscape.WebUIType.APPLET.code == code) {
        return jscape.WebUIType.APPLET;
    } else if (jscape.WebUIType.APPLET_HTML.defaultCode == defaultCode) {
        return jscape.WebUIType.APPLET_HTML;
    } else {
        return jscape.WebUIType.HTML_APPLET;
    }
};
jscape.WebUIType.valueOf = function(value) {
    if (jscape.WebUIType.HTML.value == value) {
        return jscape.WebUIType.HTML;
    } else if (jscape.WebUIType.APPLET.value == value) {
        return jscape.WebUIType.APPLET;
    } else if (jscape.WebUIType.APPLET_HTML.value == value) {
        return jscape.WebUIType.APPLET_HTML;
    } else {
        return jscape.WebUIType.HTML_APPLET;
    }
};
/** @class */
jscape.AS2Configuration = Class.extend(
    /** @lends {jscape.AS2Configuration.prototype} */
{
    /** @constructor */
    initialize: function(enabled,
                         decryptionKeyProvider,
                         receiptSigningKeyProvider,
                         receiptSignatureAlgorithm,
                         messageEncryptionRequired,
                         messageSignatureRequired,
                         messageWithoutFilenameAllowed,
                         overwriteExistingFileAllowed,
                         uploadDirectory,
                         receiptText) {
        this.enabled = enabled;
        this.decryptionKeyProvider = decryptionKeyProvider;
        this.receiptSigningKeyProvider = receiptSigningKeyProvider;
        this.receiptSignatureAlgorithm = receiptSignatureAlgorithm;
        this.messageEncryptionRequired = messageEncryptionRequired;
        this.messageSignatureRequired = messageSignatureRequired;
        this.messageWithoutFilenameAllowed = messageWithoutFilenameAllowed;
        this.overwriteExistingFileAllowed = overwriteExistingFileAllowed;
        this.uploadDirectory = uploadDirectory;
        this.receiptText = receiptText
    },

    toJSON: function() {
        return {
            enabled: this.enabled,
            decryptionKeyProvider: this.decryptionKeyProvider ? this.decryptionKeyProvider.toJSON() : null,
            receiptSigningKeyProvider: this.receiptSigningKeyProvider ? this.receiptSigningKeyProvider.toJSON() : null,
            receiptSignatureAlgorithm: this.receiptSignatureAlgorithm ? this.receiptSignatureAlgorithm.toJSON() : null,
            messageEncryptionRequired: this.messageEncryptionRequired,
            messageSignatureRequired: this.messageSignatureRequired,
            messageWithoutFilenameAllowed: this.messageWithoutFilenameAllowed,
            overwriteExistingFileAllowed: this.overwriteExistingFileAllowed,
            uploadDirectory: this.uploadDirectory,
            receiptText: this.receiptText
        }
    }
});
jscape.AS2Configuration.fromJSON = function(data) {
    return new jscape.AS2Configuration(data.enabled,
                                       jscape.ServerKeyProvider.fromJSON(data.decryptionKeyProvider),
                                       jscape.ServerKeyProvider.fromJSON(data.receiptSigningKeyProvider),
                                       jscape.SignatureAlgorithm.fromJSON(data.receiptSignatureAlgorithm),
                                       data.messageEncryptionRequired,
                                       data.messageSignatureRequired,
                                       data.messageWithoutFilenameAllowed,
                                       data.overwriteExistingFileAllowed,
                                       data.uploadDirectory,
                                       data.receiptText);
};
/** @class */
jscape.SignatureAlgorithm = Class.extend(
    /** @lends {jscape.SignatureAlgorithm.prototype} */
{
    /** @constructor */
    initialize: function(hashAlgorithm, asymmetricalAlgorithm) {
        this.hashAlgorithm = hashAlgorithm;
        this.asymmetricalAlgorithm = asymmetricalAlgorithm;
        this.name = "{0}WITH{1}".supplant({0: hashAlgorithm, 1: asymmetricalAlgorithm}).toUpperCase();
    },
    toJSON: function() {
        return this.name;
    }
});
jscape.SignatureAlgorithm.forName = function(name) {
    var values = jscape.SignatureAlgorithm.values();
    for (var i = 0; i < values.length; i++) {
        var candidate = values[i];
        if (name === candidate.name) {
            return candidate;
        }
    }
    return null;
};
jscape.SignatureAlgorithm.fromJSON = jscape.SignatureAlgorithm.forName;
jscape.SignatureAlgorithm.values = function() {
    return [
        new jscape.SignatureAlgorithm("SHA1", "RSA"),
        new jscape.SignatureAlgorithm("SHA1", "DSA"),
        new jscape.SignatureAlgorithm("MD5", "RSA")
    ];
};
jscape.SignatureAlgorithm.algorithmsOf = function (asymmetricalAlgorithm) {
    var values = jscape.SignatureAlgorithm.values();
    for (var i = 0; i < values.length; i++) {
        if (asymmetricalAlgorithm !== values[i].asymmetricalAlgorithm) {
            values.splice(i, 1);
        }
    }
    return values;
};


/** @class */
jscape.ServerKeyProvider = Class.extend(
    /** @lends {jscape.ServerKeyProvider.prototype} */
{
    /** @constructor */
    initialize: function(key) {
        this.key = key;
    },

    toJSON: function() {
        return {
            '@class': "com.jscape.inet.mft.ServerKeyProvider",
            keyAlias: this.key
        }
    }
});
jscape.ServerKeyProvider.fromJSON = function(data) {
    return data && data.keyAlias ? new jscape.ServerKeyProvider(data.keyAlias) : null;
};
/** @class */
jscape.SelfRegistrationServiceDescriptor = Class.extend(
    /** @lends {jscape.SelfRegistrationServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(signupLinkVisible, useEmailAsLogin, captchaRequired) {
        this.signupLinkVisible = signupLinkVisible || false;
        this.useEmailAsLogin = useEmailAsLogin || false;
        this.captchaRequired = captchaRequired || false;
    },

    toJSON: function() {
        return {
            showRegistrationLink: this.signupLinkVisible,
            useEmailAddressAsLogin: this.useEmailAsLogin,
            captchaRequired: this.captchaRequired
        }
    }
});
jscape.SelfRegistrationServiceDescriptor.fromJSON = function(data) {
    return new jscape.SelfRegistrationServiceDescriptor(data.showRegistrationLink, data.useEmailAddressAsLogin, data.captchaRequired);
};

/** @class */
jscape.settings.WebConfigurationController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.WebConfigurationController.prototype} */
{
    descriptor: null,
    resources: [],
    signupDescriptor: null,
    excludedSslCiphers: [],

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.WebConfigurationView(resources);
        this.as2View = new jscape.settings.AS2ConfigurationView(resources);
        this.miscView = new jscape.settings.WebMiscView(resources);
        this.resourcesView = new jscape.settings.WebResourcesView(resources);

        this._initValidators();
    },

    submitChanges: function() {
        var descriptor = this._createDescriptor();
        this.serverSettings.setWebServiceDescriptor(descriptor);

        var restConfig = this._createRestConfiguration();
        this.serverSettings.setManagementWebServerConfiguration(restConfig);

        var selfRegistrationDescriptor = this._createSelfRegistrationDescriptor();
        this.serverSettings.setSelfRegistrationServiceDescriptor(selfRegistrationDescriptor);

        this.listener.onChangesSubmitted(this);
    },

    onChangeSslCiphers: function() {
        var ciphers = this.serverSettings.getServerParameters().getSslCiphers();
        var disabledCiphers = this.excludedSslCiphers;

        this.view.showSslCiphersDialog(ciphers, disabledCiphers, {
            onSubmit: $.proxy(this._handleChangeSslCiphersSubmit, this),
            onCancel: $.proxy(this._handleDialogCancel, this)
        });
    },

    onReceiptSigningKeyChanged: function(keyAlias) {
        if (keyAlias != null) {
            var keys = this.serverSettings.getKeys().getServerKeys();
            for (var i = 0; i < keys.length; i++) {
                var key = keys[i];
                if (key.alias == keyAlias) {
                    var algorithms = jscape.SignatureAlgorithm.algorithmsOf(key.algorithm);
                    this.as2View.setReceiptSignatureAlgorithms(algorithms);
                    this.as2View.setReceiptSignatureAlgorithm(algorithms[0]);
                    break;
                }
            }
        }
    },

    onAddLanguage: function() {
        var disabledLanguages = [];
        for (var i = 0; i < this.resources.length; i++) {
            disabledLanguages.push(this.resources[i].lang);
        }
        this.resourcesView.showAddLanguageDialog(disabledLanguages, {
            onSubmit: $.proxy(this._handleAddLanguageDialogSubmit, this),
            onCancel: $.proxy(this._handleDialogCancel, this)
        });
    },

    onResetLanguages: function() {
        this._initResources();
        this.resourcesView.setResources(this.resources);
    },

    _handleChangeSslCiphersSubmit: function(dialog) {
        this.excludedSslCiphers = dialog.getDisabledCiphers();
        dialog.hide();
    },

    _handleAddLanguageDialogSubmit: function(dialog) {
        var lang = dialog.getLanguage();
        var displayName = dialog.getDisplayName();
        dialog.hide();

        var resources = [].concat(this.resources);
        resources.push(this._createWebResource(lang, displayName));
        this.resourcesView.setResources(resources);
        this.resourcesView.setLanguage(lang);
    },

    _handleDialogCancel: function(dialog) {
        dialog.hide();
    },

    _initConfiguration: function() {
        this.descriptor = this.serverSettings.getWebServiceDescriptor();
        this.signupDescriptor = this.serverSettings.getSelfRegistrationServiceDescriptor();
        this.excludedSslCiphers = this.descriptor.configuration.excludedCiphers;
        this.enabledSslProtocols = this.descriptor.configuration.enabledSslProtocols;

        this._initResources();
    },

    _initResources: function () {
        this.resources = [];
        for (var i = 0; i < this.descriptor.resources.length; i++) {
            var resource = this.descriptor.resources[i];
            this.resources.push(new jscape.WebResources(resource.lang,
                                                        resource.displayName,
                                                        $.extend({}, resource.properties)));
        }
    },

    _createDescriptor: function() {
        var webConfig = this._createConfiguration();
        var as2Config = this._createAS2Configuration();
        return new jscape.WebServiceDescriptor(webConfig,
                                               this.resourcesView.getResources(),
                                               this.resourcesView.getLogoData(),
                                               as2Config);
    },

    _createConfiguration: function() {
        var uiType = jscape.WebUIType.valueOf(this.miscView.getUserInterface());
        return new jscape.WebConfiguration(this.view.getHttpEnabled(),
                                    this.view.getHttpHost(),
                                    this.view.getHttpPort(),
                                    this.view.getHttpsEnabled(),
                                    this.view.getHttpsHost(),
                                    this.view.getHttpsPort(),
                                    this.view.getHttpsKey(),
                                    this.excludedSslCiphers,
                                    this.enabledSslProtocols,
                                    this.miscView.getShowLostPasswordLink(),
                                    this.miscView.getRedirectHttp(),
                                    this.miscView.getDomainVisible(),
                                    this.miscView.getDomainDropdown(),
                                    this.miscView.getDomainDefault(),
                                    this.view.getClientCertificateRequired(),
                                    this.miscView.getServerName(),
                                    this.miscView.getIncludeServerPort(),
                                    uiType.code,
                                    uiType.defaultCode,
                                    this.view.getSslRenegotiationAllowed(),
                                    this.miscView.getSessionTimeout(),
                                    this.miscView.getLoginCaptchaRequired());
    },

    _createRestConfiguration: function() {
        return new jscape.ManagementWebServerConfiguration(this.view.getRestHttpHost(),
                                                           this.view.getRestHttpPort(),
                                                           this.view.getRestHttpEnabled(),
                                                           this.view.getRestHttpsHost(),
                                                           this.view.getRestHttpsPort(),
                                                           this.view.getRestHttpsEnabled(),
                                                           this.view.getHttpsKey(),
                                                           this.excludedSslCiphers,
                                                           this.enabledSslProtocols,
                                                           this.view.getClientCertificateRequired(),
                                                           this.view.getSslRenegotiationAllowed(),
                                                           this.miscView.getSessionTimeout());
    },

    _createSelfRegistrationDescriptor: function() {
        return new jscape.SelfRegistrationServiceDescriptor(this.miscView.getShowRegistrationLink(),
                                                            this.miscView.getUseEmailAsLogin(),
                                                            this.miscView.getSignupCaptchaRequired());
    },

    _createAS2Configuration: function() {
        var decryptionKey = this.as2View.getDecryptionKey();
        var signingKey = this.as2View.getReceiptSigningKey();
        return new jscape.AS2Configuration(this.as2View.getEnabled(),
                                           decryptionKey != null ? new jscape.ServerKeyProvider(decryptionKey) : null,
                                           signingKey != null ? new jscape.ServerKeyProvider(signingKey) : null,
                                           this.as2View.getReceiptSignatureAlgorithm(),
                                           this.as2View.getMessageEncryptionRequired(),
                                           this.as2View.getMessageSignatureRequired(),
                                           this.as2View.getMessageWithoutFilenameAllowed(),
                                           this.as2View.getOverwriteExistingFileAllowed(),
                                           this.as2View.getUploadDirectory(),
                                           this.as2View.getReceiptText());
    },

    _createWebResource: function(lang, displayName) {
        var properties = {};
        for (var i = 0; i < this.descriptor.resources.length; i++) {
            var resource = this.descriptor.resources[i];
            if (resource.lang == "") {
                $.extend(properties, resource.properties);
                break;
            }
        }
        return new jscape.WebResources(lang, displayName, properties);
    },

    _setupView: function() {
        var config = this.descriptor.configuration;
        var restConfig = this.serverSettings.getManagementWebServerConfiguration();
        var addresses = this.serverSettings.getServerParameters().getInetAddresses();
        this.view.setHttpEnabled(config.httpEnabled);
        this.view.setHttpHosts(addresses);
        this.view.setHttpsHosts(addresses);
        this.view.setHttpHost(config.httpHost);
        this.view.setHttpPort(config.httpPort);
        this.view.setHttpsEnabled(config.httpsEnabled);
        this.view.setHttpsHost(config.httpsHost);
        this.view.setHttpsPort(config.httpsPort);
        this.view.setRestHttpHosts(addresses);
        this.view.setRestHttpsHosts(addresses);
        this.view.setRestHttpHost(restConfig.httpHost);
        this.view.setRestHttpPort(restConfig.httpPort);
        this.view.setRestHttpEnabled(restConfig.httpEnabled);
        this.view.setRestHttpsHost(restConfig.httpsHost);
        this.view.setRestHttpsPort(restConfig.httpsPort);
        this.view.setRestHttpsEnabled(restConfig.httpsEnabled);
        this.view.setHttpsKeys(this._getKeys());
        this.view.setHttpsKey(config.httpsKey);
        this.view.setClientCertificateRequired(config.clientCertificateRequired);
        this.view.setSslRenegotiationAllowed(config.sslRenegotiationAllowed);
        this.view.show(this);


        var as2Config = this.descriptor.as2ServiceConfiguration;
        var keys = this._getKeys();
        this.as2View.show(this);
        this.as2View.setDecryptionKeys(keys);
        this.as2View.setReceiptSigningKeys(keys);
        this.as2View.setReceiptSignatureAlgorithms(jscape.SignatureAlgorithm.values());

        this.as2View.setEnabled(as2Config.enabled);
        this.as2View.setDecryptionKey(as2Config.decryptionKeyProvider && as2Config.decryptionKeyProvider.key);
        this.as2View.setReceiptSigningKey(as2Config.receiptSigningKeyProvider && as2Config.receiptSigningKeyProvider.key);
        this.as2View.setReceiptSignatureAlgorithm(as2Config.receiptSignatureAlgorithm);
        this.as2View.setMessageEncryptionRequired(as2Config.messageEncryptionRequired);
        this.as2View.setMessageSignatureRequired(as2Config.messageSignatureRequired);
        this.as2View.setMessageWithoutFilenameAllowed(as2Config.messageWithoutFilenameAllowed);
        this.as2View.setOverwriteExistingFileAllowed(as2Config.overwriteExistingFileAllowed);
        this.as2View.setUploadDirectory(as2Config.uploadDirectory);
        this.as2View.setReceiptText(as2Config.receiptText);

        this.miscView.show(this);
        this.miscView.setServerName(config.serverName);
        this.miscView.setSessionTimeout(config.sessionTimeout);
        this.miscView.setIncludeServerPort(config.includeServerPort);
        this.miscView.setRedirectHttp(config.redirectHttp);
        this.miscView.setUserInterface(jscape.WebUIType.typeFor(config.userInterface, config.defaultUserInterface).value);
        this.miscView.setDomainDefault(config.domainDefault);
        this.miscView.setDomainVisible(config.domainVisible);
        this.miscView.setDomainDropdown(config.domainDropdown);
        this.miscView.setShowLostPasswordLink(config.lostPasswordEnabled);
        this.miscView.setLoginCaptchaRequired(config.captchaRequired);
        this.miscView.setShowRegistrationLink(this.signupDescriptor.signupLinkVisible);
        this.miscView.setUseEmailAsLogin(this.signupDescriptor.useEmailAsLogin);
        this.miscView.setSignupCaptchaRequired(this.signupDescriptor.captchaRequired);

        this.resourcesView.show(this);
        this.resourcesView.setLogoData(this.descriptor.logoData);
        this.resourcesView.setResources(this.resources);
        this.resourcesView.setLanguage("");
    },

    _getKeys: function() {
        var aliases = [];
        var keys = this.serverSettings.getKeys().getServerKeys();
        for (var i = 0; i < keys.length; i++) {
            aliases.push(keys[i].alias);
        }
        return aliases;
    },

    _initValidators: function() {
        $.extend($.fn.validatebox.defaults.rules, {
            'astwo_uploaddir': {
                validator: $.proxy(this._validatorAS2UploadDir, this)
            }
        });
    },

    _validatorAS2UploadDir: function(value) {
        return value && value.length != 0 && value.charAt(0) == "/";
    }
});

/** @class */
jscape.settings.SslCiphersDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.settings.SslCiphersDialog.prototype} */
{
    /** @const */
    STRONG_CIPHERS: [
        "SSL_RSA_WITH_RC4_128_MD5",
        "SSL_RSA_WITH_RC4_128_SHA",
        "SSL_RSA_WITH_3DES_EDE_CBC_SHA",
        "SSL_RSA_WITH_AES_128_CBC_SHA",
        "SSL_RSA_FIPS_WITH_3DES_EDE_CBC_SHA",
        "SSL_DHE_RSA_WITH_AES_128_CBC_SHA",
        "SSL_DHE_DSS_WITH_AES_128_CBC_SHA",
        "SSL_DHE_RSA_WITH_3DES_EDE_CBC_SHA",
        "SSL_DHE_DSS_WITH_3DES_EDE_CBC_SHA",
        "SSL_DHE_DSS_WITH_RC4_128_SHA",
        "SSL_KRB5_WITH_RC4_128_SHA",
        "SSL_KRB5_WITH_RC4_128_MD5",
        "SSL_KRB5_WITH_3DES_EDE_CBC_SHA",
        "SSL_KRB5_WITH_3DES_EDE_CBC_MD5",
        "TLS_RSA_WITH_AES_128_CBC_SHA",
        "TLS_DHE_RSA_WITH_AES_128_CBC_SHA",
        "TLS_DHE_DSS_WITH_AES_128_CBC_SHA",
        "TLS_KRB5_WITH_RC4_128_SHA",
        "TLS_KRB5_WITH_RC4_128_MD5",
        "TLS_KRB5_WITH_3DES_EDE_CBC_SHA",
        "TLS_KRB5_WITH_3DES_EDE_CBC_MD5"
    ],
    /** @const */
    MEDIUM_CIPHERS: [
        "SSL_DHE_RSA_WITH_DES_CBC_SHA",
        "SSL_DHE_DSS_WITH_DES_CBC_SHA",
        "SSL_RSA_WITH_DES_CBC_SHA",
        "SSL_RSA_FIPS_WITH_DES_CBC_SHA",
        "SSL_KRB5_WITH_DES_CBC_SHA",
        "SSL_KRB5_WITH_DES_CBC_MD5",
        "TLS_KRB5_WITH_DES_CBC_SHA",
        "TLS_KRB5_WITH_DES_CBC_MD5"
    ],

    /** @constructor */
    initialize: function(resources) {
        this._super($("#ssweb-dialog-ciphers"), resources);

        this.MEDIUM_CIPHERS = this.MEDIUM_CIPHERS.concat(this.STRONG_CIPHERS);

        this.ciphers = $("#ssweb-ciphers-list");
        this.deselectAllButton = $("#ssweb-ciphers-deselectall");
        this.selectAllButton = $("#ssweb-ciphers-selectall");
        this.selectMediumButton = $("#ssweb-ciphers-selectmedium");
        this.selectStrongButton = $("#ssweb-ciphers-selectstrong");

        this._init();
    },

    setCiphers: function(values) {
        this.ciphers.empty();
        for (var i = 0; i < values.length; i++) {
            this._addCipherInput(values[i], true)
        }
    },

    getDisabledCiphers: function() {
        var values = [];
        this.ciphers.find("input:not(:checked)").each(function() {
            values.push($(this).val());
        });
        return values;
    },

    setDisabledCiphers: function(values) {
        for (var i = 0; i < values.length; i++) {
            this._setCipherChecked(values[i], false);
        }
    },

    _addCipherInput: function(value, checked) {
        var input = $("<div><label>&nbsp;"
                      + "<input type='checkbox' value='" + value + "'" + (checked ? "checked" : "") + "/>"
                      + value + "</label></div>");
         input.appendTo(this.ciphers);
    },

    _setCipherChecked: function(value, checked) {
        this.ciphers.find("input:checkbox[value=" + value + "]").prop("checked", checked);
    },

    _init: function() {
        this.ciphers.css({"width": "330px", "height": "150px"});
        this.deselectAllButton.linkbutton();
        this.deselectAllButton.click($.proxy(this._onClick$DeselectAll, this));

        this.selectAllButton.linkbutton();
        this.selectAllButton.click($.proxy(this._onClick$SelectAll, this));

        this.selectMediumButton.linkbutton();
        this.selectMediumButton.click($.proxy(this._onClick$SelectMedium, this));

        this.selectStrongButton.linkbutton();
        this.selectStrongButton.click($.proxy(this._onClick$SelectStrong, this));
    },

    _onClick$DeselectAll: function() {
        this.ciphers.find("input:checked").prop("checked", false);
        return false;
    },

    _onClick$SelectAll: function() {
        this.ciphers.find("input:not(:checked)").prop("checked", true);
        return false;
    },

    _onClick$SelectMedium: function() {
        this._applyFilter(this.MEDIUM_CIPHERS);
        return false;
    },

    _onClick$SelectStrong: function() {
        this._applyFilter(this.STRONG_CIPHERS);
        return false;
    },

    _applyFilter: function(filter) {
        this.ciphers.find("input:checkbox").each(function() {
            $(this).prop("checked", $.inArray($(this).val(), filter) != -1);
        });
    }
});

/** @class */
jscape.settings.WebConfigurationView = Class.extend(
    /** @lends {jscape.settings.WebConfigurationView.prototype} */
{
    /** @const */
    DEFAULT_HTTP_HOST: "127.0.0.1",
    /** @const */
    DEFAULT_HTTP_PORT: 80,
    /** @const */
    DEFAULT_HTTPS_HOST: "127.0.0.1",
    /** @const */
    DEFAULT_HTTPS_PORT: 443,
    /** @const */
    DEFAULT_REST_HTTP_PORT: 11880,
    /** @const */
    DEFAULT_REST_HTTPS_PORT: 11443,

    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.httpEnabled = $("#ssweb-http");
        this.httpHost = $("#ssweb-httphost");
        this.httpPort = $("#ssweb-httpport");
        this.httpsEnabled = $("#ssweb-https");
        this.httpsHost = $("#ssweb-httpshost");
        this.httpsPort = $("#ssweb-httpsport");
        this.restHttpEnabled = $("#ssweb-resthttp");
        this.restHttpHost = $("#ssweb-resthttphost");
        this.restHttpPort = $("#ssweb-resthttpport");
        this.restHttpsEnabled = $("#ssweb-resthttps");
        this.restHttpsHost = $("#ssweb-resthttpshost");
        this.restHttpsPort = $("#ssweb-resthttpsport");
        this.privateKey = $("#ssweb-privatekey");
        this.clientCertificateRequired = $("#ssweb-certrequired");
        this.sslRenegotiation = $("#ssweb-sslrenegotiation");
        this.sslCiphersButton = $("#ssweb-sslciphers");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.WebConfigurationView.LISTENER;
        this._initState();
    },

    showSslCiphersDialog: function(ciphers, disabledCiphers, listener) {
        var dialog = new jscape.settings.SslCiphersDialog(this.resources);
        dialog.setCiphers(ciphers);
        dialog.setDisabledCiphers(disabledCiphers);
        dialog.show(listener);
    },

    getHttpEnabled: function() {
        return this.httpEnabled.prop("checked");
    },

    setHttpEnabled: function(value) {
        this.httpEnabled.prop("checked", value);
    },

    getHttpHost: function() {
        return this.httpHost.combobox("getValue");
    },

    setHttpHost: function(value) {
        value = value || this.DEFAULT_HTTP_HOST;
        this.httpHost.combobox("setValue", value).combobox("select", value);
    },

    setHttpHosts: function(values) {
        this._setHostValues(this.httpHost, values);
    },

    getHttpPort: function() {
        return this.httpPort.numberspinner("getValue");
    },

    setHttpPort: function(value) {
        this.httpPort.numberspinner("setValue", value || this.DEFAULT_HTTP_PORT);
    },

    getHttpsEnabled: function() {
        return this.httpsEnabled.prop("checked");
    },

    setHttpsEnabled: function(value) {
        this.httpsEnabled.prop("checked", value);
    },

    getHttpsHost: function() {
        return this.httpsHost.combobox("getValue");
    },

    setHttpsHost: function(value) {
        value = value || this.DEFAULT_HTTPS_HOST;
        this.httpsHost.combobox("setValue", value).combobox("select", value);
    },

    setHttpsHosts: function(values) {
        this._setHostValues(this.httpsHost, values);
    },

    getHttpsPort: function() {
        return this.httpsPort.numberspinner("getValue");
    },

    setHttpsPort: function(value) {
        this.httpsPort.numberspinner("setValue", value || this.DEFAULT_HTTPS_PORT);
    },

    getRestHttpEnabled: function() {
        return this.restHttpEnabled.prop("checked");
    },

    setRestHttpEnabled: function(value) {
        this.restHttpEnabled.prop("checked", value);
    },

    getRestHttpHost: function() {
        return this.restHttpHost.combobox("getValue");
    },

    setRestHttpHost: function(value) {
        this.restHttpHost.combobox("setValue", value).combobox("select", value);
    },

    setRestHttpHosts: function(values) {
        this._setHostValues(this.restHttpHost, values);
    },

    getRestHttpPort: function() {
        return this.restHttpPort.numberspinner("getValue");
    },

    setRestHttpPort: function(value) {
        this.restHttpPort.numberspinner("setValue", value || this.DEFAULT_REST_HTTP_PORT);
    },

    getRestHttpsEnabled: function() {
        return this.restHttpsEnabled.prop("checked");
    },

    setRestHttpsEnabled: function(value) {
        this.restHttpsEnabled.prop("checked", value);
    },

    getRestHttpsHost: function() {
        return this.restHttpsHost.combobox("getValue");
    },

    setRestHttpsHost: function(value) {
        this.restHttpsHost.combobox("setValue", value).combobox("select", value);
    },

    setRestHttpsHosts: function(values) {
        this._setHostValues(this.restHttpsHost, values);
    },

    getRestHttpsPort: function() {
        return this.restHttpsPort.numberspinner("getValue");
    },

    setRestHttpsPort: function(value) {
        this.restHttpsPort.numberspinner("setValue", value || this.DEFAULT_REST_HTTPS_PORT);
    },

    getHttpsKey: function() {
        return this.privateKey.combobox("getValue");
    },

    setHttpsKey: function(value) {
        this.privateKey.combobox("setValue", value).combobox("select", value);
    },

    setHttpsKeys: function(values) {
        var data = [];
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            data.push({text: value, value: value});
        }
        this.privateKey.combobox("loadData", data);
    },

    getClientCertificateRequired: function() {
        return this.clientCertificateRequired.prop("checked");
    },

    setClientCertificateRequired: function(value) {
        this.clientCertificateRequired.prop("checked", value);
    },

    getSslRenegotiationAllowed: function() {
        return this.sslRenegotiation.prop("checked");
    },

    setSslRenegotiationAllowed: function(value) {
        this.sslRenegotiation.prop("checked", value);
    },

    _setHostValues: function(hostBox, values) {
        var data = [];
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            data.push({host: value, value: value});
        }
        hostBox.combobox("loadData", data);
    },

    _init: function() {
        this.httpEnabled.click($.proxy(this._onClick$HttpEnable, this));
        this.httpHost.combobox({valueField: "host", textField: "value", width: 150, required: true, missingMessage: this.resources.getString("WEB_SERVICE_ERROR_BAD_HOST")});
        this.httpPort.numberspinner({min: 1, max: 65535, disabled: true});

        this.httpsEnabled.click($.proxy(this._onClick$HttpsEnable, this));
        this.httpsHost.combobox({valueField: "host", textField: "value", width: 150, required: true, missingMessage: this.resources.getString("WEB_SERVICE_ERROR_BAD_HOST")});
        this.httpsPort.numberspinner({min: 1, max: 65535, disabled: true});

        this.restHttpEnabled.prop("readonly", "readonly").click(function () {return false;});
        this.restHttpHost.combobox({valueField: "host", textField: "value", width: 150, required: true, missingMessage: this.resources.getString("WEB_SERVICE_ERROR_BAD_HOST"), readonly: true});
        this.restHttpPort.numberspinner({min: 1, max: 65535, disabled: true, editable: false, spin: function() { return false; }});

        this.restHttpsEnabled.prop("readonly", "readonly").click(function () {return false;});
        this.restHttpsHost.combobox({valueField: "host", textField: "value", width: 150, required: true, missingMessage: this.resources.getString("WEB_SERVICE_ERROR_BAD_HOST"), readonly: true});
        this.restHttpsPort.numberspinner({min: 1, max: 65535, disabled: true, editable: false, spin: function() { return false; }});

        this.privateKey.combobox({width: 120, editable: false});

        this.sslCiphersButton.linkbutton();
        this.sslCiphersButton.click($.proxy(this._onClick$SslCiphers, this));
    },

    _initState: function() {
        this._adjustHostPortGroup(this.httpHost, this.httpPort, this.httpEnabled.prop("checked"));
        this._adjustHostPortGroup(this.httpsHost, this.httpsPort, this.httpsEnabled.prop("checked"));
        this._adjustHostPortGroup(this.restHttpHost, this.restHttpPort, this.restHttpEnabled.prop("checked"));
        this._adjustHostPortGroup(this.restHttpsHost, this.restHttpsPort, this.restHttpsEnabled.prop("checked"));
    },

    _onClick$HttpEnable: function() {
        this._adjustHostPortGroup(this.httpHost, this.httpPort, this.httpEnabled.prop("checked"));
    },

    _onClick$HttpsEnable: function() {
        this._adjustHostPortGroup(this.httpsHost, this.httpsPort, this.httpsEnabled.prop("checked"));
    },

    _onClick$SslCiphers: function() {
        this.listener.onChangeSslCiphers();
        return false;
    },

    _adjustHostPortGroup: function(host, port, enabled) {
        host.combobox(enabled ? "enable" : "disable");
        port.numberspinner(enabled ? "enable" : "disable");
    }
});
/** @const */
jscape.settings.WebConfigurationView.LISTENER = {
    onChangeSslCiphers: $.noop()
};

/** @class */
jscape.settings.AS2ConfigurationView = Class.extend(
    /** @lends {jscape.settings.AS2ConfigurationView.prototype} */
{
    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset#sswas2-fields");
        this.enabled = $("#sswas2-enable");
        this.useDecryptKey = $("#sswas2-usedecryptkey");
        this.decryptKey = $("#sswas2-decryptkey");
        this.useSignKey = $("#sswas2-usesignkey");
        this.signKey = $("#sswas2-receiptsingkey");
        this.receiptSignature = $("#sswas2-receiptsignature");
        this.messageEncryptRequired = $("#sswas2-encryptrequired");
        this.messageSignatureRequired = $("#sswas2-signaturerequired");
        this.messageWithoutFilenameAllowed = $("#sswas2-allowmessagewofile");
        this.overwriteExistingFileAllowed = $("#sswas2-overwrite");
        this.uploadDir = $("#sswas2-uploaddir");
        this.receiptText = $("#sswas2-receiptext");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.AS2ConfigurationView.LISTENER;
        this._initState();
    },

    getEnabled: function() {
        return this.enabled.prop("checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", value);
        this._adjustFieldState();
    },

    getDecryptionKey: function() {
        return this.useDecryptKey.prop("checked") ? this.decryptKey.combobox("getValue") : null;
    },

    setDecryptionKey: function(value) {
        if (value) {
            this.decryptKey.combobox("setValue", value).combobox("select", value);
            this.useDecryptKey.prop("checked", true);
        } else {
            this.useDecryptKey.prop("checked", false);
        }
        this._adjustDecryptKey();
    },

    setDecryptionKeys: function(values) {
        var data = [];
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            data.push({text: value, value: value});
        }
        this.decryptKey.combobox("loadData", data);
    },

    getReceiptSigningKey: function() {
        return this.useSignKey.prop("checked") ? this.signKey.combobox("getValue") : null;
    },

    setReceiptSigningKey: function(value) {
        if (value) {
            this.signKey.combobox("setValue", value).combobox("select", value);
            this.useSignKey.prop("checked", true);
        } else {
            this.signKey.combobox("select", this._defaultValue(this.signKey).value);
            this.signKey.combobox("disable");
        }
        this._adjustReceiptSigning();
    },

    setReceiptSigningKeys: function(values) {
        var data = [];
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            data.push({text: value, value: value});
        }
        this.signKey.combobox("loadData", data);
    },

    getReceiptSignatureAlgorithm: function() {
        return this.useSignKey.prop("checked")
                ? jscape.SignatureAlgorithm.forName(this.receiptSignature.combobox("getValue"))
                : null;
    },

    setReceiptSignatureAlgorithm: function(value) {
        value = value ? value.name : this._defaultValue(this.receiptSignature).value;
        this.receiptSignature.combobox("select", value);
    },

    setReceiptSignatureAlgorithms: function(entries) {
        var data = new Array(entries.length);
        for (var i = 0; i < entries.length; i++) {
            var entry = entries[i];
            data[i] = {
                text: this.resources.getString("AS2_SIGNATURE_ALGORITHM_TEMPLATE").supplant({0: entry.hashAlgorithm, 1: entry.asymmetricalAlgorithm}),
                value: entry.name,
                algorithm: entry
            }
        }
        this.receiptSignature.combobox("loadData", data);
    },

    getMessageEncryptionRequired: function() {
        return this.messageEncryptRequired.prop("checked");
    },

    setMessageEncryptionRequired: function(value) {
        this.messageEncryptRequired.prop("checked", value);
    },

    getMessageSignatureRequired: function() {
        return this.messageSignatureRequired.prop("checked");
    },

    setMessageSignatureRequired: function(value) {
        this.messageSignatureRequired.prop("checked", value);
    },

    getMessageWithoutFilenameAllowed: function() {
        return this.messageWithoutFilenameAllowed.prop("checked");
    },

    setMessageWithoutFilenameAllowed: function(value) {
        this.messageWithoutFilenameAllowed.prop("checked", value);
    },

    getOverwriteExistingFileAllowed: function() {
        return this.overwriteExistingFileAllowed.prop("checked");
    },

    setOverwriteExistingFileAllowed: function(value) {
        this.overwriteExistingFileAllowed.prop("checked", value);
    },

    getUploadDirectory: function() {
        return this.uploadDir.val();
    },

    setUploadDirectory: function(value) {
        this.uploadDir.val(value);
    },

    getReceiptText: function() {
        return this.receiptText.val();
    },

    setReceiptText: function(value) {
        this.receiptText.val(value);
    },

    _init: function() {
        this.enabled.click($.proxy(this._onClick$Enabled, this));

        this.useDecryptKey.click($.proxy(this._onClick$UseDecryptKey, this));
        this.decryptKey.combobox({width: 120, editable: false, disabled: true});

        this.useSignKey.click($.proxy(this._onClick$UseSingKey, this));
        this.signKey.combobox({width: 120, editable: false, disabled: true, onSelect: $.proxy(this._onSelect$SigningKey, this)});
        this.receiptSignature.combobox({width: 120, editable: false, disabled: true});

        this.uploadDir.validatebox({
            required: true,
            validType: "astwo_uploaddir",
            missingMessage: this.resources.getString("AS2_ERROR_BAD_UPLOAD_DIR"),
            invalidMessage: this.resources.getString("AS2_ERROR_BAD_UPLOAD_DIR")
        });
    },

    _initState: function() {
        this._adjustFieldState();
    },

    _onClick$Enabled: function() {
        this._adjustFieldState();
    },

    _onClick$UseDecryptKey: function() {
        this._adjustDecryptKey();
    },

    _onClick$UseSingKey: function() {
        this._adjustReceiptSigning();
    },

    _onSelect$SigningKey: function() {
        this.listener.onReceiptSigningKeyChanged(this.signKey.combobox("getValue"));
    },

    _adjustDecryptKey: function() {
        var enabled = this.useDecryptKey.prop("checked") && !this.useDecryptKey.prop("disabled");
        this.decryptKey.combobox(enabled ? "enable" : "disable");
    },

    _adjustReceiptSigning: function() {
        var enabled = this.useSignKey.prop("checked") && !this.useSignKey.prop("disabled");
        this.signKey.combobox(enabled ? "enable" : "disable");
        this.receiptSignature.combobox(enabled ? "enable" : "disable");
    },

    _adjustFieldState: function() {
        var enabled = this.getEnabled();
        this.useDecryptKey.prop("disabled", !enabled);
        this.useSignKey.prop("disabled", !enabled);
        this.messageEncryptRequired.prop("disabled", !enabled);
        this.messageSignatureRequired.prop("disabled", !enabled);
        this.messageWithoutFilenameAllowed.prop("disabled", !enabled);
        this.overwriteExistingFileAllowed.prop("disabled", !enabled);
        this.uploadDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
        this.receiptText.prop("disabled", !enabled);

        this._adjustDecryptKey();
        this._adjustReceiptSigning();

        this.fieldset.form("validate");
    },
    _defaultValue: function(box) {
        var data = box.combobox("getData");
        return data && data.length ? data[0] : null;
    }
});
/** @const */
jscape.settings.AS2ConfigurationView.LISTENER = {
    onReceiptSigningKeyChanged: $.noop()
};

/** @class */
jscape.settings.WebMiscView = Class.extend(
    /** @lends {jscape.settings.WebMiscView.prototype} */
{
    /** @const */
    TIMEOUT_FACTOR: 60 * 1000,
    /** @const */
    DEFAULT_SESSION_TIMEOUT: 30,

    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.serverName = $("#sswebmisc-server");
        this.sessionTimeout = $("#sswebmisc-ttl");
        this.redirectHttp = $("#sswebmisc-redirecthttp");
        this.includeServerPort = $("#sswebmisc-inclserverport");
        this.userInterface = $("#sswebmisc-ui");
        this.domainDefault = $("#sswebmisc-defaultdomain");
        this.domainDropdown = $("#sswebmisc-domaindropdown");
        this.hideDomain = $("#sswebmisc-hidedomain");
        this.lostPasswordLink = $("#sswebmisc-lostpwd");
        this.loginCaptchaRequired = $("#sswebmisc-logincaptcha");
        this.singupLink = $("#sswebmisc-signuplink");
        this.useEmailAsLogin = $("#sswebmisc-emailaslogin");
        this.signupCaptchaRequired = $("#sswebmisc-signupcaptcha");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.WebMiscView.LISTENER;
        this._initState();
    },

    getServerName: function() {
        return this.serverName.val();
    },

    setServerName: function(value) {
        this.serverName.val(value);
        this._adjustIncludePortState();
    },

    getSessionTimeout: function() {
        return this.sessionTimeout.numberspinner("getValue") * this.TIMEOUT_FACTOR;
    },

    setSessionTimeout: function(value) {
        this.sessionTimeout.numberspinner("setValue", value ? value / this.TIMEOUT_FACTOR : this.DEFAULT_SESSION_TIMEOUT);
    },

    getRedirectHttp: function() {
        return this.redirectHttp.prop("checked");
    },

    setRedirectHttp: function(value) {
        this.redirectHttp.prop("checked", value);
    },

    getIncludeServerPort: function() {
        return this.includeServerPort.prop("checked") && !this.includeServerPort.prop("disabled");
    },

    setIncludeServerPort: function(value) {
        this.includeServerPort.prop("checked", value);
    },

    getUserInterface: function() {
        return this.userInterface.combobox("getValue");
    },

    setUserInterface: function(value) {
        this.userInterface.combobox("setValue", value);
    },

    getDomainDefault: function() {
        return this.domainDefault.val();
    },

    setDomainDefault: function(value) {
        this.domainDefault.val(value);
    },

    getDomainVisible: function() {
        return !this.hideDomain.prop("checked");
    },

    setDomainVisible: function(value) {
        this.hideDomain.prop("checked", !value);
        this._adjustDomainDropDownState();
    },

    getDomainDropdown: function() {
        return this.domainDropdown.prop("checked");
    },

    setDomainDropdown: function(value) {
        this.domainDropdown.prop("checked", value);
    },

    getShowLostPasswordLink: function() {
        return this.lostPasswordLink.prop("checked");
    },

    setShowLostPasswordLink: function(value) {
        this.lostPasswordLink.prop("checked", value);
    },

    getShowRegistrationLink: function() {
        return this.singupLink.prop("checked");
    },

    setShowRegistrationLink: function(value) {
        this.singupLink.prop("checked", value);
    },

    getUseEmailAsLogin: function() {
        return this.useEmailAsLogin.prop("checked");
    },

    setUseEmailAsLogin: function(value) {
        this.useEmailAsLogin.prop("checked", value);
    },

    getLoginCaptchaRequired: function() {
        return this.loginCaptchaRequired.prop("checked");
    },

    setLoginCaptchaRequired: function(value) {
        this.loginCaptchaRequired.prop("checked", value);
    },

    getSignupCaptchaRequired: function() {
        return this.signupCaptchaRequired.prop("checked");
    },

    setSignupCaptchaRequired: function(value) {
        this.signupCaptchaRequired.prop("checked", value);
    },

    _init: function() {
        this.sessionTimeout.numberspinner({min: 1, max: 999});
        this.userInterface.combobox({editable: false});

        var changeHandler = $.proxy(this._onChange$ServerName, this);
        this.serverName.change(changeHandler);
        this.serverName.on("input", changeHandler);
        this.hideDomain.click($.proxy(this._onClick$HideDomain, this));
    },

    _initState: function() {
        this._adjustIncludePortState();
        this._adjustDomainDropDownState();
    },

    _onChange$ServerName: function(e) {
        this._adjustIncludePortState(e);
    },

    _onClick$HideDomain: function() {
        this._adjustDomainDropDownState();
    },

    _adjustIncludePortState: function() {
        this.includeServerPort.prop("disabled", this.getServerName().trim().length == 0);
    },

    _adjustDomainDropDownState: function() {
        this.domainDropdown.prop("disabled", !this.getDomainVisible());
    }

});
/** @const */
jscape.settings.WebMiscView.LISTENER = {};

/** @class */
jscape.settings.AddLanguageDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.settings.AddLanguageDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#sswres-dialog-addlang"), resources);
        this.languages = $("#sswres-lang-add");
    },

    getLanguage: function() {
        return this.languages.val();
    },

    disableLanguages: function(values) {
        this.languages.find("option").each(function() {
            var disable = $.inArray($(this).val(), values) != -1;
            $(this).prop("disabled", disable);
        });
    },

    getDisplayName: function() {
        return this.languages.find("option:selected").html();
    }
});

/** @class */
jscape.settings.WebResourcesView = Class.extend(
    /** @lends {jscape.settings.WebResourcesView.prototype} */
{
    /** @const */
    DATA_URL_PATTERN: /data:image\/.*;base64,(.+)/,
    DATA_URL_PREFIX: "data:image/png;base64,",
    DEFAULT_DISPLAY_NAME: "default",

    languages: [],
    logoDataUrl: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.logoPreview = $("#sswres-logopreview");
        this.logoInput = $("#sswres-logo");
        this.languageBox = $("#sswres-lang");
        this.languageData = $("#sswres-data");
        this.addLanguageButton = $("#sswres-add");
        this.resetLanguagesButton = $("#sswres-reset");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.WebResourcesView.LISTENER;
        this._initState();
    },

    showAddLanguageDialog: function(disabledLanguages, listener) {
        var dialog = new jscape.settings.AddLanguageDialog(this.resources);
        dialog.disableLanguages(disabledLanguages);
        dialog.show(listener);
    },

    getLanguage: function() {
        return this.languageBox.combobox("getValue");
    },

    setLanguage: function(value) {
        this.languageBox.combobox("select", value);
    },

    getResources: function() {
        return this.languages;
    },

    setResources: function(languages) {
        this.languages = languages || [];
        var data = [];
        for (var i = 0; i < languages.length; i++) {
            var entry = languages[i];
            var text = entry.lang == "" ? this.DEFAULT_DISPLAY_NAME : entry.displayName;
            data.push({text: text, value: entry.lang});
        }
        this.languageBox.combobox("loadData", data);
        this.setLanguage(languages.length != 0 ? languages[0].lang : null);
    },

    getLogoData: function() {
        return this.logoDataUrl.replace(this.DATA_URL_PATTERN, "$1");
    },

    setLogoData: function(data) {
        this.logoDataUrl = this._asDataUrl(data);
        this._adjustLogoPreview();
    },

    _asDataUrl: function(data) {
        return this.DATA_URL_PATTERN.test(data) ? data : this.DATA_URL_PREFIX + data;
    },

    _init: function() {
        this.logoInput.change($.proxy(this._onChange$Logo, this));

        this.languageBox.combobox({
            onSelect: $.proxy(this._onChange$Language, this),
            editable: false,
            width: 120
        });
        this.languageData.propertygrid({
            onClickRow: $.proxy(this._onClick$Row, this),
            onAfterEdit: $.proxy(this._onEdit$Commit, this),
            height: 280,
            showGroup: false,
            scrollbarSize: 0,
            sortName: "name",
            sortOrder: "asc"
        });

        this.addLanguageButton.linkbutton();
        this.addLanguageButton.click($.proxy(this._onClick$AddLanguage, this));

        this.resetLanguagesButton.linkbutton();
        this.resetLanguagesButton.click($.proxy(this._onClick$ResetToDefault, this));
    },

    _initState: function() {
        this._adjustButtonsState();
    },

    _onChange$Logo: function(e) {
        var files = e.target.files;
        for (var i = 0; i < files.length; i++) {
            var file = files[i];
            if (file.type.match("image.*")) {
                this._previewFile(file);
                break;
            }
        }
    },

    _onChange$Language: function() {
        this._adjustLanguageProperties();
    },

    _onClick$Row: function() {
        this._adjustButtonsState();
    },

    _onClick$AddLanguage: function() {
        this.listener.onAddLanguage();
        return false;
    },

    _onClick$ResetToDefault: function() {
        this.listener.onResetLanguages();
        return false;
    },

    _onEdit$Commit: function(rowIndex, rowData) {
        var properties = this._languagePropertiesFor(this.getLanguage());
        properties[rowData.name] = rowData.value;
    },

    _adjustLanguageProperties: function() {
        var properties = this._languagePropertiesFor(this.getLanguage());

        var data = [];
        for (var key in properties) {
            if (properties.hasOwnProperty(key)) {
                data.push({name: key, value: properties[key], editor: "text"});
            }
        }
        this.languageData.datagrid("loadData", data);
    },

    _adjustButtonsState: function() {
    },

    _languagePropertiesFor: function(language) {
        for (var i = 0; i < this.languages.length; i++) {
            var entry = this.languages[i];
            if (entry.lang == language) {
                return entry.properties;
            }
        }
        return {};
    },

    _previewFile: function(file) {
        if (typeof FileReader != undefined) {
            var reader = new FileReader();
            reader.onload = $.proxy(function(e) {
                this.setLogoData(e.target.result);
            }, this);
            reader.readAsDataURL(file);
        }
    },

    _adjustLogoPreview: function() {
        var preview = this.logoPreview;
        var image = new Image();
        image.onload = function() { preview.attr("src", this.src); };
        image.src = this.logoDataUrl;
    }
});
/** @const */
jscape.settings.WebResourcesView.LISTENER = {
    onAddLanguage: $.noop(),
    onResetLanguages: $.noop()
};