var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.DocumentViewerConfiguration = Class.extend(
    /** @lends {jscape.DocumentViewerConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(enabled, officeDir, swfToolDir, outputDir, threadCount) {
        this.enabled = enabled || false;
        this.officeDir = officeDir;
        this.swfDir = swfToolDir;
        this.outDir = outputDir;
        this.threadCount = threadCount;
    },

    toJSON: function() {
        return {
            enabled: this.enabled,
            officeDirectory: this.officeDir,
            swfToolDirectory: this.swfDir,
            outputDirectory: this.outDir,
            threads: this.threadCount
        }
    }
});
jscape.DocumentViewerConfiguration.fromJSON = function(data) {
    return new jscape.DocumentViewerConfiguration(data.enabled,
                                                  data.officeDirectory,
                                                  data.swfToolDirectory,
                                                  data.outputDirectory,
                                                  data.threads);
};

/** @class */
jscape.settings.DocumentViewerController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.DocumentViewerController.prototype} */
{
    /** @const */
    SERVICE_KEY: "com.jscape.inet.mft.subsystems.docviewer.DocumentViewerService",

    view: null,
    config: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.DocumentViewerView(resources);

        this._initValidators();
    },

    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setDocumentViewerConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    _initConfiguration: function() {
        this.config = this.serverSettings.getDocumentViewerConfiguration();
    },

    _createConfiguration: function() {
        return new jscape.DocumentViewerConfiguration(this.view.getEnabled(),
                                                      this.view.getOfficeDir(),
                                                      this.view.getSwfToolDir(),
                                                      this.view.getOutputDir(),
                                                      this.view.getThreadCount());
    },

    _setupView: function() {
        this.view.setEnabled(this.config.enabled);
        this.view.setOfficeDir(this.config.officeDir);
        this.view.setSwfToolDir(this.config.swfDir);
        this.view.setOutputDir(this.config.outDir);
        this.view.setThreadCount(this.config.threadCount);
        this.view.show(this);
    },

    _initValidators: function() {
        $.extend($.fn.validatebox.defaults.rules, {
            'docviewer_allowed': {
                validator: $.proxy(this._validatorAllowed, this)
            }
        });
    },

    _validatorAllowed: function() {
        if (this.view.getEnabled()) {
            return this.serverSettings.getServerParameters().allowsService(this.SERVICE_KEY);
        }
        return true;
    }
});

/** @class */
jscape.settings.DocumentViewerView = Class.extend(
    /** @lends {jscape.settings.DocumentViewerView.prototype} */
{
    /** @const */
    DEFAULT_THREAD_COUNT: 10,

    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset#sswdv-fields");
        this.enabled = $("#sswdv-enable");
        this.officeDir = $("#sswdv-officedir");
        this.swfToolDir = $("#sswdv-swftooldir");
        this.outputDir = $("#sswdv-outdir");
        this.threadCount = $("#sswdv-threads");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.DocumentViewerView.LISTENER;
        this._initState();
    },

    getEnabled: function() {
        return this.enabled.prop("checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", value);
        this._adjustFieldState();
    },

    getOfficeDir: function() {
        return this.officeDir.val();
    },

    setOfficeDir: function(value) {
        this.officeDir.val(value);
    },

    getSwfToolDir: function() {
        return this.swfToolDir.val();
    },

    setSwfToolDir: function(value) {
        this.swfToolDir.val(value);
    },

    getOutputDir: function() {
        return this.outputDir.val();
    },

    setOutputDir: function(value) {
        this.outputDir.val(value);
    },

    getThreadCount: function() {
        return this.threadCount.numberspinner("getValue");
    },

    setThreadCount: function(value) {
        this.threadCount.numberspinner("setValue", value || this.DEFAULT_THREAD_COUNT);
    },

    _init: function() {
        this.enabled.validatebox({validType: "docviewer_allowed", invalidMessage: this.resources.getString("DOC_VIEWER_ERROR_NOT_ALLOWED")});
        this.enabled.click($.proxy(this._onClick$Enabled, this));

        this.officeDir.validatebox({required: true, missingMessage: this.resources.getString("DOC_VIEWER_ERROR_BAD_OFFICE_DIR")});
        this.threadCount.numberspinner({min: 1, max: 100, required: true});
    },

    _initState: function() {
        this._adjustFieldState();
    },

    _onClick$Enabled: function() {
        if (!this.enabled.validatebox("isValid")) {
            $.messager.alert(this.resources.getString("DIALOG_TITLE_ERROR"),
                             this.resources.getString("DOC_VIEWER_ERROR_NOT_ALLOWED"));
            return false;
        }
        this._adjustFieldState();
        return true;
    },

    _adjustFieldState: function() {
        var enabled = this.getEnabled();
        this.officeDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
        this.swfToolDir.prop("disabled", !enabled);
        this.outputDir.prop("disabled", !enabled);
        this.threadCount.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");

        this.fieldset.form("validate");
    }
});
jscape.settings.DocumentViewerView.LISTENER = {};